@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc 11480 2012-10-03 20:24:29Z sshwarts $
+// $Id: apic.cc 13152 2017-03-26 19:14:15Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (c) 2002-2012 Zwane Mwaikambo, Stanislav Shwartsman
+//  Copyright (c) 2002-2017 Zwane Mwaikambo, Stanislav Shwartsman
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -191,12 +191,19 @@ bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu, unsigned id)
 #if BX_SUPPORT_VMX >= 2
   // Register a non-active timer for VMX preemption timer.
   vmx_timer_handle = bx_pc_system.register_timer_ticks(this,
-            bx_local_apic_c::vmx_preemption_timer_expired, 0, 0, 0, "vmx_preemtion");
+            bx_local_apic_c::vmx_preemption_timer_expired, 0, 0, 0, "vmx_preemption");
   BX_DEBUG(("vmx_timer is = %d", vmx_timer_handle));
   vmx_preemption_timer_rate = VMX_MISC_PREEMPTION_TIMER_RATE;
   vmx_timer_active = 0;
 #endif
 
+#if BX_SUPPORT_MONITOR_MWAIT
+  mwaitx_timer_handle = bx_pc_system.register_timer_ticks(this,
+            bx_local_apic_c::mwaitx_timer_expired, 0, 0, 0, "mwaitx_timer");
+  BX_DEBUG(("mwaitx_timer is = %d", mwaitx_timer_handle));
+  mwaitx_timer_active = 0;
+#endif
+
   xapic = simulate_xapic; // xAPIC or legacy APIC
 
   reset(BX_RESET_HARDWARE);
@@ -240,6 +247,13 @@ void bx_local_apic_c::reset(unsigned type)
   }
 #endif
 
+#if BX_SUPPORT_MONITOR_MWAIT
+  if(mwaitx_timer_active) {
+    bx_pc_system.deactivate_timer(mwaitx_timer_handle);
+    mwaitx_timer_active = 0;
+  }
+#endif
+
   for(i=0; i<APIC_LVT_ENTRIES; i++) {
     lvt[i] = 0x10000;	// all LVT are masked
   }
@@ -340,7 +354,7 @@ Bit32u bx_local_apic_c::read_aligned(bx_phy_address addr)
   BX_DEBUG(("LAPIC read from register 0x%04x", apic_reg));
 
 #if BX_CPU_LEVEL >= 6
-  if (apic_reg >= 0x400 && !cpu->bx_cpuid_support_xapic_extensions())
+  if (apic_reg >= 0x400 && !cpu->is_cpu_extension_supported(BX_ISA_XAPIC_EXT))
     apic_reg = 0xffffffff; // choose some obviosly invalid register if extended xapic is not supported
 #endif
 
@@ -494,7 +508,7 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
   BX_DEBUG(("LAPIC write 0x%08x to register 0x%04x", value, apic_reg));
 
 #if BX_CPU_LEVEL >= 6
-  if (apic_reg >= 0x400 && !cpu->bx_cpuid_support_xapic_extensions())
+  if (apic_reg >= 0x400 && !cpu->is_cpu_extension_supported(BX_ISA_XAPIC_EXT))
     apic_reg = 0xffffffff; // choose some obviosly invalid register if extended xapic is not supported
 #endif
   
@@ -625,7 +639,7 @@ void bx_local_apic_c::set_lvt_entry(unsigned apic_reg, Bit32u value)
   unsigned lvt_entry = (apic_reg - BX_LAPIC_LVT_TIMER) >> 4;
 #if BX_CPU_LEVEL >= 6
   if (apic_reg == BX_LAPIC_LVT_TIMER) {
-    if (! cpu->bx_cpuid_support_tsc_deadline()) {
+    if (! cpu->is_cpu_extension_supported(BX_ISA_TSC_DEADLINE)) {
       value &= ~0x40000; // cannot enable TSC-Deadline when not supported
     }
     else {
@@ -1121,7 +1135,7 @@ void bx_local_apic_c::set_tsc_deadline(Bit64u deadline)
 
   ticksInitial = deadline;
   if (deadline != 0) {
-    BX_INFO(("APIC: TSC-Deadline is set to " FMT_LL "d", deadline));
+    BX_DEBUG(("APIC: TSC-Deadline is set to " FMT_LL "d", deadline));
     Bit64u currtime = bx_pc_system.time_ticks();
     timer_active = 1;
     bx_pc_system.activate_timer_ticks(timer_handle, (deadline > currtime) ? (deadline - currtime) : 1 , 0);
@@ -1154,7 +1168,7 @@ void bx_local_apic_c::set_vmx_preemption_timer(Bit32u value)
   vmx_preemption_timer_value = value;
   vmx_preemption_timer_initial = bx_pc_system.time_ticks();
   vmx_preemption_timer_fire = ((vmx_preemption_timer_initial >> vmx_preemption_timer_rate) + value) << vmx_preemption_timer_rate;
-  BX_DEBUG(("VMX Preemption timer. value = %u, rate = %u, init = %u, fire = %u", value, vmx_preemption_timer_rate, vmx_preemption_timer_initial, vmx_preemption_timer_fire));
+  BX_DEBUG(("VMX Preemption timer: value = %u, rate = %u, init = %u, fire = %u", value, vmx_preemption_timer_rate, vmx_preemption_timer_initial, vmx_preemption_timer_fire));
   bx_pc_system.activate_timer_ticks(vmx_timer_handle, vmx_preemption_timer_fire - vmx_preemption_timer_initial, 0);
   vmx_timer_active = 1;
 }
@@ -1174,6 +1188,32 @@ void bx_local_apic_c::vmx_preemption_timer_expired(void *this_ptr)
   class_ptr->deactivate_vmx_preemption_timer();
 }
 #endif
+
+#if BX_SUPPORT_MONITOR_MWAIT
+
+void bx_local_apic_c::set_mwaitx_timer(Bit32u value)
+{
+  BX_DEBUG(("MWAITX timer: value = %u", value));
+  bx_pc_system.activate_timer_ticks(mwaitx_timer_active, value, 0);
+  mwaitx_timer_active = 1;
+}
+
+void bx_local_apic_c::deactivate_mwaitx_timer(void)
+{
+  if (! mwaitx_timer_active) return;
+  bx_pc_system.deactivate_timer(mwaitx_timer_handle);
+  mwaitx_timer_active = 0;
+}
+
+// Invoked when MWAITX timer expired
+void bx_local_apic_c::mwaitx_timer_expired(void *this_ptr)
+{
+  bx_local_apic_c *class_ptr = (bx_local_apic_c *) this_ptr;
+  class_ptr->cpu->wakeup_monitor();
+  class_ptr->deactivate_mwaitx_timer();
+}
+
+#endif
   
 #if BX_CPU_LEVEL >= 6
 // return false when x2apic is not supported/not readable
@@ -1365,7 +1405,7 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   }
 
 #if BX_CPU_LEVEL >= 6
-  if (cpu->bx_cpuid_support_xapic_extensions()) {
+  if (cpu->is_cpu_extension_supported(BX_ISA_XAPIC_EXT)) {
     BXRS_HEX_PARAM_SIMPLE(lapic, xapic_ext);
     bx_list_c *IER = new bx_list_c(lapic, "ier");
     for (i=0; i<BX_LAPIC_MAX_INTS; i++) {
@@ -1382,7 +1422,7 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
 
   bx_list_c *LVT = new bx_list_c(lapic, "lvt");
   for (i=0; i<APIC_LVT_ENTRIES; i++) {
-    sprintf(name, "%d", i);
+    sprintf(name, "%u", i);
     new bx_shadow_num_c(LVT, name, &lvt[i], BASE_HEX);
   }
 
@@ -1402,6 +1442,11 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_rate);
   BXRS_PARAM_BOOL(lapic, vmx_timer_active, vmx_timer_active);
 #endif
+
+#if BX_SUPPORT_MONITOR_MWAIT
+  BXRS_DEC_PARAM_SIMPLE(lapic, mwaitx_timer_handle);
+  BXRS_PARAM_BOOL(lapic, mwaitx_timer_active, mwaitx_timer_active);
+#endif
 }
 
 #endif /* if BX_SUPPORT_APIC */
