@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bmi32.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: bmi32.cc 12224 2014-03-02 19:18:05Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011-2012 Stanislav Shwartsman
+//   Copyright (c) 2011-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -27,6 +27,8 @@
 
 #if BX_SUPPORT_AVX
 
+#include "scalar_arith.h"
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ANDN_GdBdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src1());
@@ -158,28 +160,17 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BEXTR_GdEdBdR(bxInstruction_c *i)
   Bit16u control = BX_READ_16BIT_REG(i->src2());
   unsigned start = control & 0xff;
   unsigned len   = control >> 8;
-  Bit32u op1_32 = 0;
-
-  if (start < 32 && len > 0) {
-    op1_32 = BX_READ_32BIT_REG(i->src1());
-    op1_32 >>= start;
-
-    if (len < 32) {
-      Bit32u extract_mask = (1 << len) - 1;
-      op1_32 &= extract_mask;
-    }
-  }
-
+  
+  Bit32u op1_32 = bextrd(BX_READ_32BIT_REG(i->src1()), start, len);
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-
   BX_WRITE_32BIT_REGZ(i->dst(), op1_32);
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BZHI_GdEdBdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BZHI_GdBdEdR(bxInstruction_c *i)
 {
-  unsigned control = BX_READ_16BIT_REG(i->src1()) & 0xff;
+  unsigned control = BX_READ_8BIT_REGL(i->src1());
   bx_bool tmpCF = 0;
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src2());
   
@@ -200,7 +191,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BZHI_GdEdBdR(bxInstruction_c *i)
 }
 
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXT_GdEdBdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXT_GdBdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src1());
   Bit32u op2_32 = BX_READ_32BIT_REG(i->src2()), result_32 = 0;
@@ -221,7 +212,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXT_GdEdBdR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PDEP_GdEdBdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PDEP_GdBdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src1());
   Bit32u op2_32 = BX_READ_32BIT_REG(i->src2()), result_32 = 0;
