@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: crregs.cc 12481 2014-08-31 20:05:25Z sshwarts $
+// $Id: crregs.cc 12912 2016-05-02 17:33:06Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2010-2014 Stanislav Shwartsman
@@ -872,7 +872,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LMSW_Ew(bxInstruction_c *i)
   }
   else {
     /* use RMAddr(i) to save address for VMexit */
-    RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
     /* pointer, segment address pair */
     msw = read_virtual_word(i->seg(), RMAddr(i));
   }
@@ -899,6 +899,13 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LMSW_Ew(bxInstruction_c *i)
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SMSW_EwR(bxInstruction_c *i)
 {
+#if BX_CPU_LEVEL >= 5
+  if (CPL!=0 && BX_CPU_THIS_PTR cr4.get_UMIP()) {
+    BX_ERROR(("SMSW: CPL != 0 causes #GP when CR4.UMIP set"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+#endif
+
   Bit32u msw = (Bit32u) read_CR0();  // handle CR0 shadow in VMX
 
   if (i->os32L()) {
@@ -913,8 +920,15 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SMSW_EwR(bxInstruction_c *i)
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SMSW_EwM(bxInstruction_c *i)
 {
+#if BX_CPU_LEVEL >= 5
+  if (CPL!=0 && BX_CPU_THIS_PTR cr4.get_UMIP()) {
+    BX_ERROR(("SMSW: CPL != 0 causes #GP when CR4.UMIP set"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+#endif
+
   Bit16u msw = read_CR0() & 0xffff;   // handle CR0 shadow in VMX
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   write_virtual_word(i->seg(), eaddr, msw);
 
   BX_NEXT_INSTR(i);
@@ -1112,8 +1126,12 @@ bx_bool BX_CPU_C::SetCR0(bxInstruction_c *i, bx_address val)
   // Modification of PG,PE flushes TLB cache according to docs.
   // Additionally, the TLB strategy is based on the current value of
   // WP, so if that changes we must also flush the TLB.
-  if ((oldCR0 & 0x80010001) != (val_32 & 0x80010001))
+  if ((oldCR0 & 0x80010001) != (val_32 & 0x80010001)) {
     TLB_flush(); // Flush Global entries also
+#if BX_SUPPORT_PKEYS
+    set_PKRU(BX_CPU_THIS_PTR pkru); // recalculate protection keys due to CR0.WP change
+#endif
+  }
 
   return 1;
 }
@@ -1125,6 +1143,7 @@ Bit32u BX_CPU_C::get_cr4_allow_mask(void)
 
   // CR4 bits definitions:
   //   [31-22] Reserved, Must be Zero
+  //   [22]    PKE: Protection Keys Enable R/W
   //   [21]    SMAP: Supervisor Mode Access Prevention R/W
   //   [20]    SMEP: Supervisor Mode Execution Protection R/W
   //   [19]    Reserved, Must be Zero
@@ -1134,7 +1153,8 @@ Bit32u BX_CPU_C::get_cr4_allow_mask(void)
   //   [15]    Reserved, Must be Zero
   //   [14]    SMXE: SMX Extensions R/W
   //   [13]    VMXE: VMX Extensions R/W
-  //   [12-11] Reserved, Must be Zero
+  //   [12] Reserved, Must be Zero
+  //   [11]    UMIP: User Mode Instruction Prevention R/W
   //   [10]    OSXMMEXCPT: Operating System Unmasked Exception Support R/W
   //   [9]     OSFXSR: Operating System FXSAVE/FXRSTOR Support R/W
   //   [8]     PCE: Performance-Monitoring Counter Enable R/W
@@ -1207,6 +1227,12 @@ Bit32u BX_CPU_C::get_cr4_allow_mask(void)
 
   if (is_cpu_extension_supported(BX_ISA_SMAP))
     allowMask |= BX_CR4_SMAP_MASK;
+
+  if (is_cpu_extension_supported(BX_ISA_PKU))
+    allowMask |= BX_CR4_PKE_MASK;
+
+  if (is_cpu_extension_supported(BX_ISA_UMIP))
+    allowMask |= BX_CR4_UMIP_MASK;
 #endif
 
   return allowMask;
@@ -1305,6 +1331,11 @@ bx_bool BX_CPU_C::SetCR4(bxInstruction_c *i, bx_address val)
 #endif
 #endif
 
+  // re-calculate protection keys if CR4.PKE was set
+#if BX_SUPPORT_PKEYS
+  set_PKRU(BX_CPU_THIS_PTR pkru);
+#endif
+
   return 1;
 }
 #endif // BX_CPU_LEVEL >= 5
