@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_arith.cc 11415 2012-09-12 21:08:40Z vruppert $
+// $Id: fpu_arith.cc 12769 2015-05-16 21:06:59Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003-2012 Stanislav Shwartsman
+//   Copyright (c) 2003-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -28,7 +28,7 @@
 
 #if BX_SUPPORT_FPU
 
-float_status_t FPU_pre_exception_handling(Bit16u control_word)
+float_status_t i387cw_to_softfloat_status_word(Bit16u control_word)
 {
   float_status_t status;
 
@@ -56,6 +56,7 @@ float_status_t FPU_pre_exception_handling(Bit16u control_word)
   status.float_nan_handling_mode = float_first_operand_nan;
   status.float_rounding_mode = (control_word & FPU_CW_RC) >> 10;
   status.flush_underflow_to_zero = 0;
+  status.float_suppress_exception = 0;
   status.float_exception_masks = control_word & FPU_CW_Exceptions_Mask;
   status.denormals_are_zeros = 0;
 
@@ -72,7 +73,7 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float32 b32, int bIsNaN, float_s
     if (aIsSignalingNaN | bIsSignalingNaN)
         float_raise(status, float_flag_invalid);
 
-    // propogate QNaN to SNaN
+    // propagate QNaN to SNaN
     a = propagateFloatx80NaN(a, status);
 
     if (aIsNaN & !bIsNaN) return a;
@@ -121,7 +122,7 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, float_s
     if (aIsSignalingNaN | bIsSignalingNaN)
         float_raise(status, float_flag_invalid);
 
-    // propogate QNaN to SNaN
+    // propagate QNaN to SNaN
     a = propagateFloatx80NaN(a, status);
 
     if (aIsNaN & !bIsNaN) return a;
@@ -178,8 +179,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+  float_status_t status = 
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
 
@@ -208,7 +209,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
 
@@ -225,7 +226,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -238,7 +239,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -254,7 +255,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -267,7 +268,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -283,7 +284,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -299,7 +300,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
 
@@ -313,7 +314,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -329,7 +330,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_
   floatx80 b = int32_to_floatx80(load_reg);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
 
@@ -356,7 +357,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(i->src());
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
 
@@ -385,7 +386,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
 
@@ -402,7 +403,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -415,7 +416,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -431,7 +432,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -444,7 +445,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -460,7 +461,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -476,7 +477,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
 
@@ -490,7 +491,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -506,7 +507,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_
   floatx80 b = int32_to_floatx80(load_reg);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
 
@@ -533,7 +534,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(i->src());
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -560,7 +561,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -589,7 +590,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -621,7 +622,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(i->dst());
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -638,7 +639,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -651,7 +652,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -667,7 +668,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -680,7 +681,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
@@ -696,7 +697,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -709,7 +710,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -725,7 +726,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -738,7 +739,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
 
   floatx80 b = BX_READ_FPU_REG(0), result;
@@ -755,7 +756,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -771,7 +772,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -785,7 +786,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -801,7 +802,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -815,7 +816,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -828,7 +829,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(BX_READ_FPU_REG(0), int32_to_floatx80(load_reg), status);
 
@@ -842,7 +843,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -858,7 +859,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
 
@@ -885,7 +886,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(i->src());
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -912,7 +913,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -941,7 +942,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -973,7 +974,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
   floatx80 b = BX_READ_FPU_REG(i->dst());
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -990,7 +991,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1003,7 +1004,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -1019,7 +1020,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1032,7 +1033,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
@@ -1048,7 +1049,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1061,7 +1062,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
@@ -1077,7 +1078,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1090,7 +1091,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
@@ -1106,7 +1107,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1122,7 +1123,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -1136,7 +1137,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1152,7 +1153,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -1166,7 +1167,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1182,7 +1183,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_
   floatx80 b = int32_to_floatx80(load_reg);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -1196,7 +1197,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
@@ -1212,7 +1213,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction
   floatx80 b = BX_READ_FPU_REG(0);
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
 
@@ -1235,7 +1236,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSQRT(bxInstruction_c *i)
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sqrt(BX_READ_FPU_REG(0), status);
 
@@ -1259,7 +1260,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FRNDINT(bxInstruction_c *i)
   }
 
   float_status_t status =
-     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+     i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_round_to_int(BX_READ_FPU_REG(0), status);
 
