@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_load_store.cc 12736 2015-05-02 20:08:36Z sshwarts $
+// $Id: fpu_load_store.cc 12769 2015-05-16 21:06:59Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003-2012 Stanislav Shwartsman
+//   Copyright (c) 2003-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -70,7 +70,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_SINGLE_REAL(bxInstruction_c *i
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
 
   FPU_update_last_instruction(i);
@@ -101,7 +101,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_DOUBLE_REAL(bxInstruction_c *i
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
 
   FPU_update_last_instruction(i);
@@ -134,7 +134,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_EXTENDED_REAL(bxInstruction_c
 
   floatx80 result;
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   result.fraction = read_virtual_qword(i->seg(), RMAddr(i));
   result.exp      = read_virtual_word(i->seg(), (RMAddr(i)+8) & i->asize_mask());
 
@@ -158,7 +158,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
 
   FPU_update_last_instruction(i);
@@ -182,7 +182,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_DWORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
 
   FPU_update_last_instruction(i);
@@ -206,7 +206,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_QWORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit64s load_reg = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
 
   FPU_update_last_instruction(i);
@@ -230,7 +230,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FBLD_PACKED_BCD(bxInstruction_c *i
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16u hi2 = read_virtual_word(i->seg(), (RMAddr(i) + 8) & i->asize_mask());
   Bit64u lo8 = read_virtual_qword(i->seg(), RMAddr(i));
 
@@ -297,7 +297,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -345,7 +345,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -394,7 +394,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FSTP_EXTENDED_REAL(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -426,7 +426,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -474,7 +474,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -522,7 +522,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTP_QWORD_INTEGER(bxInstruction_
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -566,7 +566,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -645,7 +645,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP16(bxInstruction_c *i)
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -690,7 +690,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP32(bxInstruction_c *i)
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
@@ -735,7 +735,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP64(bxInstruction_c *i)
 {
   BX_CPU_THIS_PTR prepareFPU(i);
 
-  RMAddr(i) = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
   FPU_update_last_instruction(i);
 
