@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: io.cc 11117 2012-03-28 21:11:19Z sshwarts $
+// $Id: io.cc 12843 2015-09-28 18:37:35Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2011  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,26 +32,33 @@
 //
 
 #if BX_SUPPORT_REPEAT_SPEEDUPS
-Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, bx_address dstOff, Bit16u port, Bit32u wordCount)
+Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, Bit32u dstOff, Bit16u port, Bit32u wordCount)
 {
   Bit32u wordsFitDst;
   signed int pointerDelta;
   Bit8u *hostAddrDst;
   unsigned count;
+  bx_address laddrDst;
 
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
+
+    laddrDst = get_laddr32(BX_SEG_REG_ES, dstOff);
+  }
 
-  bx_address laddrDst = get_laddr32(BX_SEG_REG_ES, dstOff);
   // check that the address is word aligned
   if (laddrDst & 1) return 0;
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -105,27 +112,33 @@ Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, bx_address dstOff, Bit16u port,
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepOUTSW(bxInstruction_c *i, unsigned srcSeg, bx_address srcOff, Bit16u port, Bit32u wordCount)
+Bit32u BX_CPU_C::FastRepOUTSW(bxInstruction_c *i, unsigned srcSeg, Bit32u srcOff, Bit16u port, Bit32u wordCount)
 {
   Bit32u wordsFitSrc;
   signed int pointerDelta;
   Bit8u *hostAddrSrc;
   unsigned count;
+  bx_address laddrSrc;
 
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *srcSegPtr = &BX_CPU_THIS_PTR sregs[srcSeg];
-  if (!(srcSegPtr->cache.valid & SegAccessROK))
-    return 0;
-  if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (srcSegPtr->cache.valid & SegAccessROK4G) {
+    laddrSrc = srcOff;
+  }
+  else {
+    if (!(srcSegPtr->cache.valid & SegAccessROK))
+      return 0;
+    if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
+      return 0;
+
+    laddrSrc = get_laddr32(srcSeg, srcOff);
+  }
 
-  bx_address laddrSrc = get_laddr32(srcSeg, srcOff);
   // check that the address is word aligned
   if (laddrSrc & 1) return 0;
 
-  hostAddrSrc = v2h_read_byte(laddrSrc, BX_CPU_THIS_PTR user_pl);
-
+  hostAddrSrc = v2h_read_byte(laddrSrc, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrSrc) return 0;
 
@@ -218,7 +231,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB16_YbDX(bxInstruction_c *i)
 
   value8 = BX_INP(DX, 1);
 
-  write_RMW_virtual_byte(value8);
+  write_RMW_linear_byte(value8);
 
   if (BX_CPU_THIS_PTR get_DF())
     DI--;
@@ -235,7 +248,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB32_YbDX(bxInstruction_c *i)
   value8 = BX_INP(DX, 1);
 
   /* no seg override possible */
-  write_RMW_virtual_byte(value8);
+  write_RMW_linear_byte(value8);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     RDI = EDI - 1;
@@ -251,11 +264,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB32_YbDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB64_YbDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit8u value8 = read_RMW_virtual_byte_64(BX_SEG_REG_ES, RDI);
+  Bit8u value8 = read_RMW_linear_byte(BX_SEG_REG_ES, RDI);
 
   value8 = BX_INP(DX, 1);
 
-  write_RMW_virtual_byte(value8);
+  write_RMW_linear_byte(value8);
 
   if (BX_CPU_THIS_PTR get_DF())
     RDI--;
@@ -297,7 +310,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW16_YwDX(bxInstruction_c *i)
 
   value16 = BX_INP(DX, 2);
 
-  write_RMW_virtual_word(value16);
+  write_RMW_linear_word(value16);
 
   if (BX_CPU_THIS_PTR get_DF())
     DI -= 2;
@@ -336,7 +349,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
 
       value16 = BX_INP(DX, 2);
 
-      write_RMW_virtual_word(value16);
+      write_RMW_linear_word(value16);
     }
   }
   else
@@ -347,7 +360,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
 
     value16 = BX_INP(DX, 2);
 
-    write_RMW_virtual_word(value16);
+    write_RMW_linear_word(value16);
   }
 
   if (BX_CPU_THIS_PTR get_DF())
@@ -362,11 +375,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW64_YwDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit16u value16 = read_RMW_virtual_word_64(BX_SEG_REG_ES, RDI);
+  Bit16u value16 = read_RMW_linear_word(BX_SEG_REG_ES, RDI);
 
   value16 = BX_INP(DX, 2);
 
-  write_RMW_virtual_word(value16);
+  write_RMW_linear_word(value16);
 
   if (BX_CPU_THIS_PTR get_DF())
     RDI -= 2;
@@ -408,7 +421,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD16_YdDX(bxInstruction_c *i)
 
   value32 = BX_INP(DX, 4);
 
-  write_RMW_virtual_dword(value32);
+  write_RMW_linear_dword(value32);
 
   if (BX_CPU_THIS_PTR get_DF())
     DI -= 4;
@@ -424,7 +437,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD32_YdDX(bxInstruction_c *i)
 
   value32 = BX_INP(DX, 4);
 
-  write_RMW_virtual_dword(value32);
+  write_RMW_linear_dword(value32);
 
   if (BX_CPU_THIS_PTR get_DF())
     RDI = EDI - 4;
@@ -438,11 +451,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD32_YdDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD64_YdDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit32u value32 = read_RMW_virtual_dword_64(BX_SEG_REG_ES, RDI);
+  Bit32u value32 = read_RMW_linear_dword(BX_SEG_REG_ES, RDI);
 
   value32 = BX_INP(DX, 4);
 
-  write_RMW_virtual_dword(value32);
+  write_RMW_linear_dword(value32);
 
   if (BX_CPU_THIS_PTR get_DF())
     RDI -= 4;
@@ -509,7 +522,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSB32_DXXb(bxInstruction_c *i)
 // 64-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSB64_DXXb(bxInstruction_c *i)
 {
-  Bit8u value8 = read_virtual_byte_64(i->seg(), RSI);
+  Bit8u value8 = read_linear_byte(i->seg(), get_laddr64(i->seg(), RSI));
   BX_OUTP(DX, value8, 1);
 
   if (BX_CPU_THIS_PTR get_DF())
@@ -600,7 +613,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSW32_DXXw(bxInstruction_c *i)
 // 16-bit operand size, 64-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSW64_DXXw(bxInstruction_c *i)
 {
-  Bit16u value16 = read_virtual_word_64(i->seg(), RSI);
+  Bit16u value16 = read_linear_word(i->seg(), get_laddr64(i->seg(), RSI));
   BX_OUTP(DX, value16, 2);
 
   if (BX_CPU_THIS_PTR get_DF())
@@ -664,7 +677,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSD32_DXXd(bxInstruction_c *i)
 // 32-bit operand size, 64-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUTSD64_DXXd(bxInstruction_c *i)
 {
-  Bit32u value32 = read_virtual_dword_64(i->seg(), RSI);
+  Bit32u value32 = read_linear_dword(i->seg(), get_laddr64(i->seg(), RSI));
   BX_OUTP(DX, value32, 4);
 
   if (BX_CPU_THIS_PTR get_DF())
