@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift8.cc 12613 2015-01-25 20:55:10Z sshwarts $
+// $Id: shift8.cc 13165 2017-03-31 07:34:08Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2015  The Bochs Project
+//  Copyright (C) 2001-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -72,7 +72,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EbM(bxInstruction_c *i)
   else
     count = i->Ib();
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
@@ -151,7 +151,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EbM(bxInstruction_c *i)
   else
     count = i->Ib();
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
@@ -201,11 +201,13 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbR(bxInstruction_c *i)
 
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->dst(), i->extend8bitL());
 
+  unsigned temp_CF = getB_CF();
+
   if (count==1) {
-    result_8 = (op1_8 << 1) | getB_CF();
+    result_8 = (op1_8 << 1) | temp_CF;
   }
   else {
-    result_8 = (op1_8 << count) | (getB_CF() << (count - 1)) |
+    result_8 = (op1_8 << count) | (temp_CF << (count - 1)) |
                (op1_8 >> (9 - count));
   }
 
@@ -229,7 +231,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
   else
     count = i->Ib();
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
@@ -239,11 +241,13 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EbM(bxInstruction_c *i)
     BX_NEXT_INSTR(i);
   }
 
+  unsigned temp_CF = getB_CF();
+
   if (count==1) {
-    result_8 = (op1_8 << 1) | getB_CF();
+    result_8 = (op1_8 << 1) | temp_CF;
   }
   else {
-    result_8 = (op1_8 << count) | (getB_CF() << (count - 1)) |
+    result_8 = (op1_8 << count) | (temp_CF << (count - 1)) |
                (op1_8 >> (9 - count));
   }
 
@@ -271,7 +275,9 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbR(bxInstruction_c *i)
   if (count) {
     Bit8u op1_8 = BX_READ_8BIT_REGx(i->dst(), i->extend8bitL());
 
-    Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
+    unsigned temp_CF = getB_CF();
+
+    Bit8u result_8 = (op1_8 >> count) | (temp_CF << (8 - count)) |
                      (op1_8 << (9 - count));
 
     BX_WRITE_8BIT_REGx(i->dst(), i->extend8bitL(), result_8);
@@ -294,14 +300,16 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EbM(bxInstruction_c *i)
   else
     count = i->Ib();
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
   count = (count & 0x1f) % 9;
 
   if (count) {
-    Bit8u result_8 = (op1_8 >> count) | (getB_CF() << (8 - count)) |
+    unsigned temp_CF = getB_CF();
+
+    Bit8u result_8 = (op1_8 >> count) | (temp_CF << (8 - count)) |
                      (op1_8 << (9 - count));
 
     write_RMW_linear_byte(result_8);
@@ -363,7 +371,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EbM(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
@@ -427,7 +435,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EbM(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
@@ -485,7 +493,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EbM(bxInstruction_c *i)
 
   count &= 0x1f;
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   /* pointer, segment address pair */
   Bit8u op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
 
