@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: string.cc 11117 2012-03-28 21:11:19Z sshwarts $
+// $Id: string.cc 12843 2015-09-28 18:37:35Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2011  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@
 //
 
 #if BX_SUPPORT_REPEAT_SPEEDUPS
-Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address srcOff, unsigned dstSeg, bx_address dstOff, Bit32u count)
+Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, Bit32u srcOff, unsigned dstSeg, Bit32u dstOff, Bit32u count)
 {
   Bit32u bytesFitSrc, bytesFitDst;
   signed int pointerDelta;
@@ -39,25 +39,36 @@ Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *srcSegPtr = &BX_CPU_THIS_PTR sregs[srcSeg];
-  if (!(srcSegPtr->cache.valid & SegAccessROK))
-    return 0;
-  if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (srcSegPtr->cache.valid & SegAccessROK4G) {
+    laddrSrc = srcOff;
+  }
+  else {
+    if (!(srcSegPtr->cache.valid & SegAccessROK))
+      return 0;
+    if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+    laddrSrc = get_laddr32(srcSeg, srcOff);
+  }
 
-  laddrSrc = get_laddr32(srcSeg, srcOff);
-
-  hostAddrSrc = v2h_read_byte(laddrSrc, BX_CPU_THIS_PTR user_pl);
+  hostAddrSrc = v2h_read_byte(laddrSrc, USER_PL);
+  // Check that native host access was not vetoed for that page
   if (! hostAddrSrc) return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
+
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -99,7 +110,7 @@ Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address srcOff, unsigned dstSeg, bx_address dstOff, Bit32u count)
+Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, Bit32u srcOff, unsigned dstSeg, Bit32u dstOff, Bit32u count)
 {
   Bit32u wordsFitSrc, wordsFitDst;
   signed int pointerDelta;
@@ -109,25 +120,36 @@ Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *srcSegPtr = &BX_CPU_THIS_PTR sregs[srcSeg];
-  if (!(srcSegPtr->cache.valid & SegAccessROK))
-    return 0;
-  if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (srcSegPtr->cache.valid & SegAccessROK4G) {
+    laddrSrc = srcOff;
+  }
+  else {
+    if (!(srcSegPtr->cache.valid & SegAccessROK))
+      return 0;
+    if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+    laddrSrc = get_laddr32(srcSeg, srcOff);
+  }
 
-  laddrSrc = get_laddr32(srcSeg, srcOff);
-
-  hostAddrSrc = v2h_read_byte(laddrSrc, BX_CPU_THIS_PTR user_pl);
+  hostAddrSrc = v2h_read_byte(laddrSrc, USER_PL);
+  // Check that native host access was not vetoed for that page
   if (! hostAddrSrc) return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
+
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -172,7 +194,7 @@ Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address srcOff, unsigned dstSeg, bx_address dstOff, Bit32u count)
+Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, Bit32u srcOff, unsigned dstSeg, Bit32u dstOff, Bit32u count)
 {
   Bit32u dwordsFitSrc, dwordsFitDst;
   signed int pointerDelta;
@@ -182,25 +204,36 @@ Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *srcSegPtr = &BX_CPU_THIS_PTR sregs[srcSeg];
-  if (!(srcSegPtr->cache.valid & SegAccessROK))
-    return 0;
-  if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (srcSegPtr->cache.valid & SegAccessROK4G) {
+    laddrSrc = srcOff;
+  }
+  else {
+    if (!(srcSegPtr->cache.valid & SegAccessROK))
+      return 0;
+    if ((srcOff | 0xfff) > srcSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+    laddrSrc = get_laddr32(srcSeg, srcOff);
+  }
 
-  laddrSrc = get_laddr32(srcSeg, srcOff);
-
-  hostAddrSrc = v2h_read_byte(laddrSrc, BX_CPU_THIS_PTR user_pl);
+  hostAddrSrc = v2h_read_byte(laddrSrc, USER_PL);
+  // Check that native host access was not vetoed for that page
   if (! hostAddrSrc) return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+  bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
+
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -245,7 +278,7 @@ Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, bx_address dstOff, Bit8u val, Bit32u count)
+Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, Bit32u dstOff, Bit8u val, Bit32u count)
 {
   Bit32u bytesFitDst;
   signed int pointerDelta;
@@ -255,14 +288,19 @@ Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -299,7 +337,7 @@ Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, bx_address dstOff, Bit16u val, Bit32u count)
+Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, Bit32u dstOff, Bit16u val, Bit32u count)
 {
   Bit32u wordsFitDst;
   signed int pointerDelta;
@@ -309,14 +347,19 @@ Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -355,7 +398,7 @@ Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   return 0;
 }
 
-Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address dstOff, Bit32u val, Bit32u count)
+Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, Bit32u dstOff, Bit32u val, Bit32u count)
 {
   Bit32u dwordsFitDst;
   signed int pointerDelta;
@@ -365,14 +408,19 @@ Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
   bx_segment_reg_t *dstSegPtr = &BX_CPU_THIS_PTR sregs[dstSeg];
-  if (!(dstSegPtr->cache.valid & SegAccessWOK))
-    return 0;
-  if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
-    return 0;
+  if (dstSegPtr->cache.valid & SegAccessWOK4G) {
+    laddrDst = dstOff;
+  }
+  else {
+    if (!(dstSegPtr->cache.valid & SegAccessWOK))
+      return 0;
+    if ((dstOff | 0xfff) > dstSegPtr->cache.u.segment.limit_scaled)
+      return 0;
 
-  laddrDst = get_laddr32(dstSeg, dstOff);
+    laddrDst = get_laddr32(dstSeg, dstOff);
+  }
 
-  hostAddrDst = v2h_write_byte(laddrDst, BX_CPU_THIS_PTR user_pl);
+  hostAddrDst = v2h_write_byte(laddrDst, USER_PL);
   // Check that native host access was not vetoed for that page
   if (!hostAddrDst) return 0;
 
@@ -416,71 +464,71 @@ Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address ds
 // REP MOVS methods
 //
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSB_XbYb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSB_YbXb(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L())
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB64_XbYb);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB64_YbXb);
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB32_XbYb);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB32_YbXb);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RSI); // always clear upper part of RSI/RDI
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI);
   }
   else {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB16_XbYb);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSB16_YbXb);
   }
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSW_XwYw(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSW_YwXw(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L())
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW64_XwYw);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW64_YwXw);
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW32_XwYw);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW32_YwXw);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RSI); // always clear upper part of RSI/RDI
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI);
   }
   else {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW16_XwYw);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSW16_YwXw);
   }
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSD_XdYd(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSD_YdXd(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L())
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD64_XdYd);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD64_YdXd);
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD32_XdYd);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD32_YdXd);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RSI); // always clear upper part of RSI/RDI
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI);
   }
   else {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD16_XdYd);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSD16_YdXd);
   }
 
   BX_NEXT_INSTR(i);
 }
 
 #if BX_SUPPORT_X86_64
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSQ_XqYq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSQ_YqXq(bxInstruction_c *i)
 {
   if (i->as64L()) {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSQ64_XqYq);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSQ64_YqXq);
   }
   else {
-    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSQ32_XqYq);
+    BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::MOVSQ32_YqXq);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RSI); // always clear upper part of RSI/RDI
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI);
   }
@@ -494,7 +542,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_MOVSQ_XqYq(bxInstruction_c *i)
 //
 
 // 16 bit address size
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB16_XbYb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB16_YbXb(bxInstruction_c *i)
 {
   Bit8u temp8 = read_virtual_byte_32(i->seg(), SI);
   write_virtual_byte_32(BX_SEG_REG_ES, DI, temp8);
@@ -512,7 +560,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB16_XbYb(bxInstruction_c *i)
 }
 
 // 32 bit address size
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB32_XbYb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB32_YbXb(bxInstruction_c *i)
 {
   Bit8u temp8;
 
@@ -561,15 +609,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB32_XbYb(bxInstruction_c *i)
 
 #if BX_SUPPORT_X86_64
 // 64 bit address size
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB64_XbYb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB64_YbXb(bxInstruction_c *i)
 {
   Bit8u temp8;
 
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  temp8 = read_virtual_byte_64(i->seg(), rsi);
-  write_virtual_byte_64(BX_SEG_REG_ES, rdi, temp8);
+  temp8 = read_linear_byte(i->seg(), get_laddr64(i->seg(), rsi));
+  write_linear_byte(BX_SEG_REG_ES, rdi, temp8);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     /* decrement RSI, RDI */
@@ -588,7 +636,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSB64_XbYb(bxInstruction_c *i)
 #endif
 
 /* 16 bit opsize mode, 16 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW16_XwYw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW16_YwXw(bxInstruction_c *i)
 {
   Bit16u si = SI;
   Bit16u di = DI;
@@ -612,7 +660,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW16_XwYw(bxInstruction_c *i)
 }
 
 /* 16 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW32_XwYw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW32_YwXw(bxInstruction_c *i)
 {
   Bit16u temp16;
 
@@ -638,15 +686,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW32_XwYw(bxInstruction_c *i)
 
 #if BX_SUPPORT_X86_64
 /* 16 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW64_XwYw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW64_YwXw(bxInstruction_c *i)
 {
   Bit16u temp16;
 
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  temp16 = read_virtual_word_64(i->seg(), rsi);
-  write_virtual_word_64(BX_SEG_REG_ES, rdi, temp16);
+  temp16 = read_linear_word(i->seg(), get_laddr64(i->seg(), rsi));
+  write_linear_word(BX_SEG_REG_ES, rdi, temp16);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 2;
@@ -663,7 +711,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSW64_XwYw(bxInstruction_c *i)
 #endif
 
 /* 32 bit opsize mode, 16 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD16_XdYd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD16_YdXd(bxInstruction_c *i)
 {
   Bit32u temp32;
 
@@ -687,7 +735,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD16_XdYd(bxInstruction_c *i)
 }
 
 /* 32 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD32_XdYd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD32_YdXd(bxInstruction_c *i)
 {
   Bit32u temp32;
 
@@ -745,15 +793,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD32_XdYd(bxInstruction_c *i)
 #if BX_SUPPORT_X86_64
 
 /* 32 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD64_XdYd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD64_YdXd(bxInstruction_c *i)
 {
   Bit32u temp32;
 
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  temp32 = read_virtual_dword_64(i->seg(), rsi);
-  write_virtual_dword_64(BX_SEG_REG_ES, rdi, temp32);
+  temp32 = read_linear_dword(i->seg(), get_laddr64(i->seg(), rsi));
+  write_linear_dword(BX_SEG_REG_ES, rdi, temp32);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 4;
@@ -769,15 +817,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSD64_XdYd(bxInstruction_c *i)
 }
 
 /* 64 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSQ32_XqYq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSQ32_YqXq(bxInstruction_c *i)
 {
   Bit64u temp64;
 
   Bit32u esi = ESI;
   Bit32u edi = EDI;
 
-  temp64 = read_virtual_qword_64(i->seg(), esi);
-  write_virtual_qword_64(BX_SEG_REG_ES, edi, temp64);
+  temp64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), esi));
+  write_linear_qword(BX_SEG_REG_ES, edi, temp64);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     esi -= 8;
@@ -794,15 +842,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSQ32_XqYq(bxInstruction_c *i)
 }
 
 /* 64 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSQ64_XqYq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSQ64_YqXq(bxInstruction_c *i)
 {
   Bit64u temp64;
 
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  temp64 = read_virtual_qword_64(i->seg(), rsi);
-  write_virtual_qword_64(BX_SEG_REG_ES, rdi, temp64);
+  temp64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), rsi));
+  write_linear_qword(BX_SEG_REG_ES, rdi, temp64);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 8;
@@ -969,8 +1017,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSB64_XbYb(bxInstruction_c *i)
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  op1_8 = read_virtual_byte_64(i->seg(), rsi);
-  op2_8 = read_virtual_byte_64(BX_SEG_REG_ES, rdi);
+  op1_8 = read_linear_byte(i->seg(), get_laddr64(i->seg(), rsi));
+  op2_8 = read_linear_byte(BX_SEG_REG_ES, rdi);
 
   diff_8 = op1_8 - op2_8;
 
@@ -1056,8 +1104,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSW64_XwYw(bxInstruction_c *i)
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  op1_16 = read_virtual_word_64(i->seg(), rsi);
-  op2_16 = read_virtual_word_64(BX_SEG_REG_ES, rdi);
+  op1_16 = read_linear_word(i->seg(), get_laddr64(i->seg(), rsi));
+  op2_16 = read_linear_word(BX_SEG_REG_ES, rdi);
 
   diff_16 = op1_16 - op2_16;
 
@@ -1144,8 +1192,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSD64_XdYd(bxInstruction_c *i)
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  op1_32 = read_virtual_dword_64(i->seg(), rsi);
-  op2_32 = read_virtual_dword_64(BX_SEG_REG_ES, rdi);
+  op1_32 = read_linear_dword(i->seg(), get_laddr64(i->seg(), rsi));
+  op2_32 = read_linear_dword(BX_SEG_REG_ES, rdi);
 
   diff_32 = op1_32 - op2_32;
 
@@ -1172,8 +1220,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSQ32_XqYq(bxInstruction_c *i)
   Bit32u esi = ESI;
   Bit32u edi = EDI;
 
-  op1_64 = read_virtual_qword_64(i->seg(), esi);
-  op2_64 = read_virtual_qword_64(BX_SEG_REG_ES, edi);
+  op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), esi));
+  op2_64 = read_linear_qword(BX_SEG_REG_ES, edi);
 
   diff_64 = op1_64 - op2_64;
 
@@ -1201,8 +1249,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSQ64_XqYq(bxInstruction_c *i)
   Bit64u rsi = RSI;
   Bit64u rdi = RDI;
 
-  op1_64 = read_virtual_qword_64(i->seg(), rsi);
-  op2_64 = read_virtual_qword_64(BX_SEG_REG_ES, rdi);
+  op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), rsi));
+  op2_64 = read_linear_qword(BX_SEG_REG_ES, rdi);
 
   diff_64 = op1_64 - op2_64;
 
@@ -1227,71 +1275,71 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSQ64_XqYq(bxInstruction_c *i)
 // REP SCAS methods
 //
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASB_ALXb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASB_ALYb(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB64_ALXb);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB64_ALYb);
   }
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB32_ALXb);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB32_ALYb);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI); // always clear upper part of RDI
   }
   else {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB16_ALXb);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASB16_ALYb);
   }
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASW_AXXw(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASW_AXYw(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW64_AXXw);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW64_AXYw);
   }
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW32_AXXw);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW32_AXYw);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI); // always clear upper part of RDI
   }
   else {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW16_AXXw);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASW16_AXYw);
   }
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASD_EAXXd(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASD_EAXYd(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD64_EAXXd);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD64_EAXYd);
   }
   else
 #endif
   if (i->as32L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD32_EAXXd);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD32_EAXYd);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI); // always clear upper part of RDI
   }
   else {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD16_EAXXd);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASD16_EAXYd);
   }
 
   BX_NEXT_INSTR(i);
 }
 
 #if BX_SUPPORT_X86_64
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASQ_RAXXq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASQ_RAXYq(bxInstruction_c *i)
 {
   if (i->as64L()) {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASQ64_RAXXq);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASQ64_RAXYq);
   }
   else {
-    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASQ32_RAXXq);
+    BX_CPU_THIS_PTR repeat_ZF(i, &BX_CPU_C::SCASQ32_RAXYq);
     BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RDI); // always clear upper part of RDI
   }
 
@@ -1304,7 +1352,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_SCASQ_RAXXq(bxInstruction_c *i
 //
 
 /* 16 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB16_ALXb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB16_ALYb(bxInstruction_c *i)
 {
   Bit8u op1_8 = AL, op2_8, diff_8;
 
@@ -1327,7 +1375,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB16_ALXb(bxInstruction_c *i)
 }
 
 /* 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB32_ALXb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB32_ALYb(bxInstruction_c *i)
 {
   Bit8u op1_8 = AL, op2_8, diff_8;
 
@@ -1351,13 +1399,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB32_ALXb(bxInstruction_c *i)
 
 #if BX_SUPPORT_X86_64
 /* 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB64_ALXb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB64_ALYb(bxInstruction_c *i)
 {
   Bit8u op1_8 = AL, op2_8, diff_8;
 
   Bit64u rdi = RDI;
 
-  op2_8 = read_virtual_byte_64(BX_SEG_REG_ES, rdi);
+  op2_8 = read_virtual_byte(BX_SEG_REG_ES, rdi);
 
   diff_8 = op1_8 - op2_8;
 
@@ -1375,7 +1423,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASB64_ALXb(bxInstruction_c *i)
 #endif
 
 /* 16 bit opsize mode, 16 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW16_AXXw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW16_AXYw(bxInstruction_c *i)
 {
   Bit16u op1_16 = AX, op2_16, diff_16;
 
@@ -1397,7 +1445,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW16_AXXw(bxInstruction_c *i)
 }
 
 /* 16 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW32_AXXw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW32_AXYw(bxInstruction_c *i)
 {
   Bit16u op1_16 = AX, op2_16, diff_16;
 
@@ -1421,13 +1469,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW32_AXXw(bxInstruction_c *i)
 
 #if BX_SUPPORT_X86_64
 /* 16 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW64_AXXw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW64_AXYw(bxInstruction_c *i)
 {
   Bit16u op1_16 = AX, op2_16, diff_16;
 
   Bit64u rdi = RDI;
 
-  op2_16 = read_virtual_word_64(BX_SEG_REG_ES, rdi);
+  op2_16 = read_virtual_word(BX_SEG_REG_ES, rdi);
 
   diff_16 = op1_16 - op2_16;
 
@@ -1445,7 +1493,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASW64_AXXw(bxInstruction_c *i)
 #endif
 
 /* 32 bit opsize mode, 16 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD16_EAXXd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD16_EAXYd(bxInstruction_c *i)
 {
   Bit32u op1_32 = EAX, op2_32, diff_32;
 
@@ -1467,7 +1515,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD16_EAXXd(bxInstruction_c *i)
 }
 
 /* 32 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD32_EAXXd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD32_EAXYd(bxInstruction_c *i)
 {
   Bit32u op1_32 = EAX, op2_32, diff_32;
 
@@ -1492,13 +1540,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD32_EAXXd(bxInstruction_c *i)
 #if BX_SUPPORT_X86_64
 
 /* 32 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD64_EAXXd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD64_EAXYd(bxInstruction_c *i)
 {
   Bit32u op1_32 = EAX, op2_32, diff_32;
 
   Bit64u rdi = RDI;
 
-  op2_32 = read_virtual_dword_64(BX_SEG_REG_ES, rdi);
+  op2_32 = read_virtual_dword(BX_SEG_REG_ES, rdi);
 
   diff_32 = op1_32 - op2_32;
 
@@ -1515,13 +1563,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASD64_EAXXd(bxInstruction_c *i)
 }
 
 /* 64 bit opsize mode, 32 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASQ32_RAXXq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASQ32_RAXYq(bxInstruction_c *i)
 {
   Bit64u op1_64 = RAX, op2_64, diff_64;
 
   Bit32u edi = EDI;
 
-  op2_64 = read_virtual_qword_64(BX_SEG_REG_ES, edi);
+  op2_64 = read_virtual_qword(BX_SEG_REG_ES, edi);
 
   diff_64 = op1_64 - op2_64;
 
@@ -1539,13 +1587,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASQ32_RAXXq(bxInstruction_c *i)
 }
 
 /* 64 bit opsize mode, 64 bit address size */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASQ64_RAXXq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SCASQ64_RAXYq(bxInstruction_c *i)
 {
   Bit64u op1_64 = RAX, op2_64, diff_64;
 
   Bit64u rdi = RDI;
 
-  op2_64 = read_virtual_qword_64(BX_SEG_REG_ES, rdi);
+  op2_64 = read_virtual_qword(BX_SEG_REG_ES, rdi);
 
   diff_64 = op1_64 - op2_64;
 
@@ -1710,7 +1758,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STOSB64_YbAL(bxInstruction_c *i)
 {
   Bit64u rdi = RDI;
 
-  write_virtual_byte_64(BX_SEG_REG_ES, rdi, AL);
+  write_linear_byte(BX_SEG_REG_ES, rdi, AL);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rdi--;
@@ -1764,7 +1812,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STOSW64_YwAX(bxInstruction_c *i)
 {
   Bit64u rdi = RDI;
 
-  write_virtual_word_64(BX_SEG_REG_ES, rdi, AX);
+  write_linear_word(BX_SEG_REG_ES, rdi, AX);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rdi -= 2;
@@ -1819,7 +1867,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STOSD64_YdEAX(bxInstruction_c *i)
 {
   Bit64u rdi = RDI;
 
-  write_virtual_dword_64(BX_SEG_REG_ES, rdi, EAX);
+  write_linear_dword(BX_SEG_REG_ES, rdi, EAX);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rdi -= 4;
@@ -1836,7 +1884,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STOSQ32_YqRAX(bxInstruction_c *i)
 {
   Bit32u edi = EDI;
 
-  write_virtual_qword_64(BX_SEG_REG_ES, edi, RAX);
+  write_linear_qword(BX_SEG_REG_ES, edi, RAX);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     edi -= 8;
@@ -1854,7 +1902,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STOSQ64_YqRAX(bxInstruction_c *i)
 {
   Bit64u rdi = RDI;
 
-  write_virtual_qword_64(BX_SEG_REG_ES, rdi, RAX);
+  write_linear_qword(BX_SEG_REG_ES, rdi, RAX);
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rdi -= 8;
@@ -1986,7 +2034,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LODSB64_ALXb(bxInstruction_c *i)
 {
   Bit64u rsi = RSI;
 
-  AL = read_virtual_byte_64(i->seg(), rsi);
+  AL = read_linear_byte(i->seg(), get_laddr64(i->seg(), rsi));
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi--;
@@ -2040,7 +2088,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LODSW64_AXXw(bxInstruction_c *i)
 {
   Bit64u rsi = RSI;
 
-  AX = read_virtual_word_64(i->seg(), rsi);
+  AX = read_linear_word(i->seg(), get_laddr64(i->seg(), rsi));
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 2;
@@ -2095,7 +2143,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LODSD64_EAXXd(bxInstruction_c *i)
 {
   Bit64u rsi = RSI;
 
-  RAX = read_virtual_dword_64(i->seg(), rsi);
+  RAX = read_linear_dword(i->seg(), get_laddr64(i->seg(), rsi));
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 4;
@@ -2112,7 +2160,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LODSQ32_RAXXq(bxInstruction_c *i)
 {
   Bit32u esi = ESI;
 
-  RAX = read_virtual_qword_64(i->seg(), esi);
+  RAX = read_linear_qword(i->seg(), get_laddr64(i->seg(), esi));
 
   if (BX_CPU_THIS_PTR get_DF()) {
     esi -= 8;
@@ -2130,7 +2178,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LODSQ64_RAXXq(bxInstruction_c *i)
 {
   Bit64u rsi = RSI;
 
-  RAX = read_virtual_qword_64(i->seg(), rsi);
+  RAX = read_linear_qword(i->seg(), get_laddr64(i->seg(), rsi));
 
   if (BX_CPU_THIS_PTR get_DF()) {
     rsi -= 8;
