@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: tasking.cc 12392 2014-07-03 06:40:42Z sshwarts $
+// $Id: tasking.cc 12836 2015-09-22 20:10:22Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2014  The Bochs Project
@@ -196,8 +196,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   // used in the task switch are paged in.
   if (BX_CPU_THIS_PTR cr0.get_PG())
   {
-    translate_linear(BX_TLB_ENTRY_OF(nbase32), nbase32, 0, BX_READ);  // old TSS
-    translate_linear(BX_TLB_ENTRY_OF(nbase32 + new_TSS_max), nbase32 + new_TSS_max, 0, BX_READ);
+    translate_linear(BX_TLB_ENTRY_OF(nbase32,               0), nbase32,               0, BX_READ); // old TSS
+    translate_linear(BX_TLB_ENTRY_OF(nbase32 + new_TSS_max, 0), nbase32 + new_TSS_max, 0, BX_READ);
 
     // ??? Humm, we check the new TSS region with READ above,
     // but sometimes we need to write the link field in that
@@ -208,8 +208,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
 
     if (source == BX_TASK_FROM_CALL || source == BX_TASK_FROM_INT)
     {
-      translate_linear(BX_TLB_ENTRY_OF(nbase32),     nbase32,     0, BX_WRITE);
-      translate_linear(BX_TLB_ENTRY_OF(nbase32 + 1), nbase32 + 1, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(nbase32,     0), nbase32,     0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(nbase32 + 1, 0), nbase32 + 1, 0, BX_WRITE);
     }
   }
 
@@ -252,8 +252,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
     if (BX_CPU_THIS_PTR cr0.get_PG()) {
       Bit32u start = Bit32u(obase32 + 14), end = Bit32u(obase32 + 41);
 
-      translate_linear(BX_TLB_ENTRY_OF(start), start, 0, BX_WRITE);
-      translate_linear(BX_TLB_ENTRY_OF(end),   end,   0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(start, 0), start, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(end, 0),   end,   0, BX_WRITE);
     }
 
     system_write_word(Bit32u(obase32 + 14), IP);
@@ -281,8 +281,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
     if (BX_CPU_THIS_PTR cr0.get_PG()) {
       Bit32u start = Bit32u(obase32 + 0x20), end = Bit32u(obase32 + 0x5d);
 
-      translate_linear(BX_TLB_ENTRY_OF(start), start, 0, BX_WRITE);
-      translate_linear(BX_TLB_ENTRY_OF(end),   end,   0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(start, 0), start, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(end, 0),   end,   0, BX_WRITE);
     }
 
     system_write_dword(Bit32u(obase32 + 0x20), EIP);
