@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vmexit.cc 11321 2012-08-06 20:41:16Z sshwarts $
+// $Id: vmexit.cc 12770 2015-05-16 21:25:43Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2009-2012 Stanislav Shwartsman
+//   Copyright (c) 2009-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -57,9 +57,9 @@ Bit32u gen_instruction_info(bxInstruction_c *i, Bit32u reason, bx_bool rw_form)
   //  instruction information field format
   // --------------------------------------
   //
-  // [.2:.0] | Memory operand scale field (encoded)
-  // [.6:.3] | Reg1, undefined when memory operand
-  // [.9:.7] | Memory operand address size
+  // [02:00] | Memory operand scale field (encoded)
+  // [06:03] | Reg1, undefined when memory operand
+  // [09:07] | Memory operand address size
   // [10:10] | Memory/Register format (0 - mem, 1 - reg)
   // [14:11] | Reserved
   // [17:15] | Memory operand segment register field
@@ -86,7 +86,8 @@ Bit32u gen_instruction_info(bxInstruction_c *i, Bit32u reason, bx_bool rw_form)
 
     instr_info |= i->seg() << 15;
 
-    if (i->sibIndex() != BX_NIL_REGISTER)
+    // index field is always initialized because of gather but not always valid
+    if (i->sibIndex() != BX_NIL_REGISTER && i->sibIndex() != 4)
         instr_info |= i->sibScale() | (i->sibIndex() << 18);
     else
         instr_info |= 1 << 22; // index invalid
@@ -126,7 +127,10 @@ void BX_CPP_AttrRegparmN(3) BX_CPU_C::VMexit_Instruction(bxInstruction_c *i, Bit
       }
       else
 #endif
+      {
         qualification = (Bit64u) ((Bit32u) i->displ32s());
+        qualification &= i->asize_mask();
+      }
 
       instr_info = gen_instruction_info(i, reason, rw_form);
       VMwrite32(VMCS_32BIT_VMEXIT_INSTRUCTION_INFO, instr_info);
@@ -144,29 +148,28 @@ void BX_CPU_C::VMexit_PAUSE(void)
   BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
 
   if (VMEXIT(VMX_VM_EXEC_CTRL2_PAUSE_VMEXIT)) {
-    BX_DEBUG(("VMEXIT: PAUSE"));
     VMexit(VMX_VMEXIT_PAUSE, 0);
   }
 
 #if BX_SUPPORT_VMX >= 2
   if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_PAUSE_LOOP_VMEXIT) && CPL == 0) {
-    VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+    VMX_PLE *ple = &BX_CPU_THIS_PTR vmcs.ple;
     Bit64u currtime = bx_pc_system.time_ticks();
-    if ((currtime - vm->last_pause_time) > vm->pause_loop_exiting_gap) {
-      vm->first_pause_time = currtime;
+    if ((currtime - ple->last_pause_time) > ple->pause_loop_exiting_gap) {
+      ple->first_pause_time = currtime;
     }
     else {
-      if ((currtime - vm->first_pause_time) > vm->pause_loop_exiting_window)
+      if ((currtime - ple->first_pause_time) > ple->pause_loop_exiting_window)
         VMexit(VMX_VMEXIT_PAUSE, 0);
     }
-    vm->last_pause_time = currtime;
+    ple->last_pause_time = currtime;
   }
 #endif
 }
 
 void BX_CPU_C::VMexit_ExtInterrupt(void)
 {
-  if (! BX_CPU_THIS_PTR in_vmx_guest) return;
+  BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
 
   if (PIN_VMEXIT(VMX_VM_EXEC_CTRL1_EXTERNAL_INTERRUPT_VMEXIT)) {
     VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
@@ -178,18 +181,6 @@ void BX_CPU_C::VMexit_ExtInterrupt(void)
   }
 }
 
-#if BX_SUPPORT_VMX >= 2  
-void BX_CPU_C::VMexit_PreemptionTimerExpired(void)
-{
-  BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
-
-  if (PIN_VMEXIT(VMX_VM_EXEC_CTRL1_VMX_PREEMPTION_TIMER_VMEXIT)) {
-    BX_DEBUG(("VMEXIT: VMX Preemption Timer Expired"));
-    VMexit(VMX_VMEXIT_VMX_PREEMPTION_TIMER_EXPIRED, 0);
-  }
-}
-#endif
-
 void BX_CPU_C::VMexit_Event(unsigned type, unsigned vector, Bit16u errcode, bx_bool errcode_valid, Bit64u qualification)
 {
   if (! BX_CPU_THIS_PTR in_vmx_guest) return;
@@ -206,7 +197,7 @@ void BX_CPU_C::VMexit_Event(unsigned type, unsigned vector, Bit16u errcode, bx_b
       break;
 
     case BX_NMI:
-      if (PIN_VMEXIT(VMX_VM_EXEC_CTRL1_NMI_VMEXIT))
+      if (PIN_VMEXIT(VMX_VM_EXEC_CTRL1_NMI_EXITING))
          vmexit = 1;
       break;
 
@@ -235,8 +226,8 @@ void BX_CPU_C::VMexit_Event(unsigned type, unsigned vector, Bit16u errcode, bx_b
   // ----------------------------------------------------
   //              VMExit interruption info
   // ----------------------------------------------------
-  // [.7:.0] | Interrupt/Exception vector
-  // [10:.8] | Interrupt/Exception type
+  // [07:00] | Interrupt/Exception vector
+  // [10:08] | Interrupt/Exception type
   // [11:11] | error code pushed to the stack
   // [12:12] | NMI unblocking due to IRET
   // [30:13] | reserved
@@ -249,6 +240,8 @@ void BX_CPU_C::VMexit_Event(unsigned type, unsigned vector, Bit16u errcode, bx_b
     vm->idt_vector_info = vector | (type << 8);
     if (errcode_valid)
       vm->idt_vector_info |= (1 << 11); // error code delivered
+
+    BX_CPU_THIS_PTR nmi_unblocking_iret = 0;
     return;
   }
 
@@ -273,6 +266,9 @@ void BX_CPU_C::VMexit_Event(unsigned type, unsigned vector, Bit16u errcode, bx_b
     interruption_info |= (1 << 11); // error code delivered
   interruption_info   |= (1 << 31); // valid
 
+  if (BX_CPU_THIS_PTR nmi_unblocking_iret)
+    interruption_info |= (1 << 12);
+
   VMwrite32(VMCS_32BIT_VMEXIT_INTERRUPTION_INFO, interruption_info);
   VMwrite32(VMCS_32BIT_VMEXIT_INTERRUPTION_ERR_CODE, errcode);
 
@@ -283,8 +279,6 @@ void BX_CPU_C::VMexit_TripleFault(void)
 {
   if (! BX_CPU_THIS_PTR in_vmx_guest) return;
 
-  BX_ERROR(("VMEXIT: triple fault"));
-
   // VMEXIT is not considered to occur during event delivery if it results
   // in a triple fault exception (that causes VMEXIT directly)
   BX_CPU_THIS_PTR in_event = 0;
@@ -296,8 +290,6 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::VMexit_TaskSwitch(Bit16u tss_selector, uns
 {
   BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
 
-  BX_ERROR(("VMEXIT: task switch"));
-
   VMexit(VMX_VMEXIT_TASK_SWITCH, tss_selector | (source << 30));
 }
 
@@ -322,7 +314,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::VMexit_MSR(unsigned op, Bit32u msr)
          // check MSR-HI bitmaps
          bx_phy_address pAddr = vm->msr_bitmap_addr + ((msr - BX_VMX_HI_MSR_START) >> 3) + 1024 + ((op == VMX_VMEXIT_RDMSR) ? 0 : 2048);
          access_read_physical(pAddr, 1, &field);
-         BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_MSR_BITMAP_ACCESS, &field);
+         BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_MSR_BITMAP_ACCESS, &field);
          if (field & (1 << (msr & 7)))
             vmexit = 1;
        }
@@ -333,7 +325,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::VMexit_MSR(unsigned op, Bit32u msr)
          // check MSR-LO bitmaps
          bx_phy_address pAddr = vm->msr_bitmap_addr + (msr >> 3) + ((op == VMX_VMEXIT_RDMSR) ? 0 : 2048);
          access_read_physical(pAddr, 1, &field);
-         BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_MSR_BITMAP_ACCESS, &field);
+         BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_MSR_BITMAP_ACCESS, &field);
          if (field & (1 << (msr & 7)))
             vmexit = 1;
        }
@@ -369,21 +361,21 @@ void BX_CPP_AttrRegparmN(3) BX_CPU_C::VMexit_IO(bxInstruction_c *i, unsigned por
           // special case - the IO access split cross both I/O bitmaps
           pAddr = BX_CPU_THIS_PTR vmcs.io_bitmap_addr[0] + 0xfff;
           access_read_physical(pAddr, 1, &bitmap[0]);
-          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[0]);
+          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[0]);
 
           pAddr = BX_CPU_THIS_PTR vmcs.io_bitmap_addr[1];
           access_read_physical(pAddr, 1, &bitmap[1]);
-          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[1]);
+          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[1]);
         }
         else {
           // access_read_physical cannot read 2 bytes cross 4K boundary :(
           pAddr = BX_CPU_THIS_PTR vmcs.io_bitmap_addr[(port >> 15) & 1] + ((port & 0x7fff) / 8);
           access_read_physical(pAddr, 1, &bitmap[0]);
-          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[0]);
+          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[0]);
 
           pAddr++;
           access_read_physical(pAddr, 1, &bitmap[1]);
-          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[1]);
+          BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[1]);
         }
 
         Bit16u combined_bitmap = bitmap[1];
@@ -442,7 +434,7 @@ void BX_CPP_AttrRegparmN(3) BX_CPU_C::VMexit_IO(bxInstruction_c *i, unsigned por
          break;
 
        default:
-         BX_PANIC(("VMexit_IO: I/O instruction %s unknown", i->getIaOpcodeName()));
+         BX_PANIC(("VMexit_IO: I/O instruction %s unknown", i->getIaOpcodeNameShort()));
      }
 
      if (qualification & VMX_VMEXIT_IO_INSTR_STRING) {
@@ -472,11 +464,11 @@ void BX_CPP_AttrRegparmN(3) BX_CPU_C::VMexit_IO(bxInstruction_c *i, unsigned por
 // ----------------------------------------------------------------
 //                 Exit qualification for CR access
 // ----------------------------------------------------------------
-// [.3:.0] | Number of CR register (CR0, CR3, CR4, CR8)
-// [.5:.4] | CR access type (0 - MOV to CR, 1 - MOV from CR, 2 - CLTS, 3 - LMSW)
-// [.6:.6] | LMSW operand reg/mem (cleared for CR access and CLTS)
-// [.7:.7] | reserved
-// [11:.8] | Source Operand Register for CR access (cleared for CLTS and LMSW)
+// [03:00] | Number of CR register (CR0, CR3, CR4, CR8)
+// [05:04] | CR access type (0 - MOV to CR, 1 - MOV from CR, 2 - CLTS, 3 - LMSW)
+// [06:06] | LMSW operand reg/mem (cleared for CR access and CLTS)
+// [07:07] | reserved
+// [11:08] | Source Operand Register for CR access (cleared for CLTS and LMSW)
 // [15:12] | reserved
 // [31:16] | LMSW source data (cleared for CR access and CLTS)
 // [63:32] | reserved
@@ -490,8 +482,6 @@ bx_bool BX_CPU_C::VMexit_CLTS(void)
 
   if (vm->vm_cr0_mask & vm->vm_cr0_read_shadow & 0x8)
   {
-    BX_DEBUG(("VMEXIT: CLTS"));
-
     // all rest of the fields cleared to zero
     Bit64u qualification = VMX_VMEXIT_CR_ACCESS_CLTS << 4;
 
@@ -558,9 +548,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMexit_CR3_Read(bxInstruction_c *i)
 
   if (VMEXIT(VMX_VM_EXEC_CTRL2_CR3_READ_VMEXIT)) {
     BX_DEBUG(("VMEXIT: CR3 read"));
-
     Bit64u qualification = 3 | (VMX_VMEXIT_CR_ACCESS_CR_READ << 4) | (i->dst() << 8);
-
     VMexit(VMX_VMEXIT_CR_ACCESS, qualification);
   }
 }
@@ -605,9 +593,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMexit_CR8_Read(bxInstruction_c *i)
 
   if (VMEXIT(VMX_VM_EXEC_CTRL2_CR8_READ_VMEXIT)) {
     BX_DEBUG(("VMEXIT: CR8 read"));
-
     Bit64u qualification = 8 | (VMX_VMEXIT_CR_ACCESS_CR_READ << 4) | (i->dst() << 8);
-
     VMexit(VMX_VMEXIT_CR_ACCESS, qualification);
   }
 }
@@ -627,10 +613,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMexit_CR8_Write(bxInstruction_c *i)
 // ----------------------------------------------------------------
 //                 Exit qualification for DR access
 // ----------------------------------------------------------------
-// [.3:.0] | Number of DR register
-// [.4:.4] | DR access type (0 - MOV to DR, 1 - MOV from DR)
-// [.7:.5] | reserved
-// [11:.8] | Source Operand Register
+// [03:00] | Number of DR register
+// [04:04] | DR access type (0 - MOV to DR, 1 - MOV from DR)
+// [07:05] | reserved
+// [11:08] | Source Operand Register
 // [63:12] | reserved
 //
 
@@ -650,92 +636,6 @@ void BX_CPU_C::VMexit_DR_Access(unsigned read, unsigned dr, unsigned reg)
   }
 }
 
-#if BX_SUPPORT_X86_64
-Bit32u BX_CPU_C::VMX_Read_VTPR(void)
-{
-  bx_phy_address pAddr = BX_CPU_THIS_PTR vmcs.virtual_apic_page_addr + 0x80;
-  Bit32u vtpr;
-  access_read_physical(pAddr, 4, (Bit8u*)(&vtpr));
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, BX_READ, BX_VMX_VTPR_ACCESS, (Bit8u*)(&vtpr));
-  return vtpr;
-}
-
-void BX_CPU_C::VMX_Write_VTPR(Bit8u vtpr)
-{
-  VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
-  bx_phy_address pAddr = vm->virtual_apic_page_addr + 0x80;
-  Bit32u field32 = vtpr;
-
-  access_write_physical(pAddr, 4, (Bit8u*)(&field32));
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, BX_WRITE, BX_VMX_VTPR_ACCESS, (Bit8u*)(&field32));
-
-  Bit8u tpr_shadow = vtpr >> 4;
-  if (tpr_shadow < vm->vm_tpr_threshold) {
-    // commit current instruction to produce trap-like VMexit
-    BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
-    VMexit(VMX_VMEXIT_TPR_THRESHOLD, 0);
-  }
-}
-
-// apic virtualization
-bx_bool BX_CPP_AttrRegparmN(1) BX_CPU_C::is_virtual_apic_page(bx_phy_address paddr)
-{
-  if (BX_CPU_THIS_PTR in_vmx_guest) {
-    VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
-    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES))
-      if (PPFOf(paddr) == PPFOf(vm->apic_access_page)) return 1;
-  }
-
-  return 0;
-}
-
-// apic virtualization
-void BX_CPU_C::VMX_Virtual_Apic_Read(bx_phy_address paddr, unsigned len, void *data)
-{
-  BX_ASSERT(SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES));
-
-  Bit32u offset = PAGE_OFFSET(paddr);
-
-  // access is not instruction fetch because cpu::prefetch will crash them
-  if (VMEXIT(VMX_VM_EXEC_CTRL2_TPR_SHADOW) && offset == 0x80 && len <= 4) {
-     // VTPR access
-     Bit32u vtpr = VMX_Read_VTPR();
-     if (len == 1)
-       *((Bit8u *) data) = vtpr & 0xff;
-     else if (len == 2)
-       *((Bit16u *) data) = vtpr & 0xffff;
-     else if (len == 4)
-       *((Bit32u *) data) = vtpr;
-     else
-       BX_PANIC(("PANIC: Unsupported Virtual APIC access len = 3 !"));
-     return;
-  }
-
-  Bit32u qualification = offset | 
-      (BX_CPU_THIS_PTR in_event) ? VMX_APIC_ACCESS_DURING_EVENT_DELIVERY : VMX_APIC_READ_INSTRUCTION_EXECUTION;
-  VMexit(VMX_VMEXIT_APIC_ACCESS, qualification);
-}
-
-// apic virtualization
-void BX_CPU_C::VMX_Virtual_Apic_Write(bx_phy_address paddr, unsigned len, void *data)
-{
-  BX_ASSERT(SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES));
-
-  Bit32u offset = PAGE_OFFSET(paddr);
-
-  if (VMEXIT(VMX_VM_EXEC_CTRL2_TPR_SHADOW) && offset == 0x80 && len <= 4) {
-    // VTPR access
-    VMX_Write_VTPR(*((Bit8u *) data));
-    return;
-  }
-
-  Bit32u qualification = offset | 
-      (BX_CPU_THIS_PTR in_event) ? VMX_APIC_ACCESS_DURING_EVENT_DELIVERY : VMX_APIC_WRITE_INSTRUCTION_EXECUTION;
-  VMexit(VMX_VMEXIT_APIC_ACCESS, qualification);
-}
-
-#endif
-
 #if BX_SUPPORT_VMX >= 2
 Bit16u BX_CPU_C::VMX_Get_Current_VPID(void)
 {
@@ -746,4 +646,131 @@ Bit16u BX_CPU_C::VMX_Get_Current_VPID(void)
 }
 #endif
 
+#if BX_SUPPORT_VMX >= 2
+bx_bool BX_CPP_AttrRegparmN(1) BX_CPU_C::Vmexit_Vmread(bxInstruction_c *i)
+{
+  BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
+
+  if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VMCS_SHADOWING)) return BX_TRUE;
+
+#if BX_SUPPORT_X86_64
+  if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+    if (BX_READ_64BIT_REG_HIGH(i->src())) return BX_TRUE;
+  }
+#endif
+  unsigned encoding = BX_READ_32BIT_REG(i->src());
+  if (encoding > 0x7fff) return BX_TRUE;
+
+  VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+  Bit8u bitmap;
+  bx_phy_address pAddr = vm->vmread_bitmap_addr | (encoding >> 3);
+  access_read_physical(pAddr, 1, &bitmap);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_VMREAD_BITMAP_ACCESS, &bitmap);
+  
+  if (bitmap & (1 << (encoding & 7)))
+    return BX_TRUE;
+
+  return BX_FALSE;
+}
+
+bx_bool BX_CPP_AttrRegparmN(1) BX_CPU_C::Vmexit_Vmwrite(bxInstruction_c *i)
+{
+  BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
+
+  if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VMCS_SHADOWING)) return BX_TRUE;
+
+#if BX_SUPPORT_X86_64
+  if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+    if (BX_READ_64BIT_REG_HIGH(i->dst())) return BX_TRUE;
+  }
+#endif
+  unsigned encoding = BX_READ_32BIT_REG(i->dst());
+  if (encoding > 0x7fff) return BX_TRUE;
+
+  VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+  Bit8u bitmap;
+  bx_phy_address pAddr = vm->vmwrite_bitmap_addr | (encoding >> 3);
+  access_read_physical(pAddr, 1, &bitmap);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_VMWRITE_BITMAP_ACCESS, &bitmap);
+  
+  if (bitmap & (1 << (encoding & 7)))
+    return BX_TRUE;
+
+  return BX_FALSE;
+}
+
+void BX_CPU_C::Virtualization_Exception(Bit64u qualification, Bit64u guest_physical, Bit64u guest_linear)
+{
+  BX_ASSERT(BX_CPU_THIS_PTR in_vmx_guest);
+
+  // A convertible EPT violation causes a virtualization exception if the following all hold:
+  //  - CR0.PE is set
+  //  - the logical processor is not in the process of delivering an event through the IDT
+  //  - the 32 bits at offset 4 in the virtualization-exception information area are all 0
+
+  if (! BX_CPU_THIS_PTR cr0.get_PE() || BX_CPU_THIS_PTR in_event) return;
+
+  VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+  Bit32u magic;
+  access_read_physical(vm->ve_info_addr + 4, 4, &magic);
+#if BX_SUPPORT_MEMTYPE
+  BxMemtype ve_info_memtype = resolve_memtype(vm->ve_info_addr);
+#endif
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 4, 4, MEMTYPE(ve_info_memtype), BX_READ, 0, (Bit8u*)(&magic));
+  if (magic != 0) return;
+
+  struct ve_info {
+    Bit32u reason; // always VMX_VMEXIT_EPT_VIOLATION
+    Bit32u magic;
+    Bit64u qualification;
+    Bit64u guest_linear_addr;
+    Bit64u guest_physical_addr;
+    Bit16u eptp_index;
+  } ve_info = { VMX_VMEXIT_EPT_VIOLATION, 0xffffffff, qualification, guest_linear, guest_physical, vm->eptp_index };
+
+  access_write_physical(vm->ve_info_addr, 4, &ve_info.reason);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr, 4, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.reason));
+
+  access_write_physical(vm->ve_info_addr + 4, 4, &ve_info.magic);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 4, 4, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.magic));
+
+  access_write_physical(vm->ve_info_addr + 8, 8, &ve_info.qualification);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 8, 8, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.qualification));
+
+  access_write_physical(vm->ve_info_addr + 16, 8, &ve_info.guest_linear_addr);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 16, 8, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.guest_linear_addr));
+
+  access_write_physical(vm->ve_info_addr + 24, 8, &ve_info.guest_physical_addr);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 24, 8, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.guest_physical_addr));
+
+  access_write_physical(vm->ve_info_addr + 32, 8, &ve_info.eptp_index);
+  BX_NOTIFY_PHY_MEMORY_ACCESS(vm->ve_info_addr + 32, 8, MEMTYPE(ve_info_memtype), BX_WRITE, 0, (Bit8u*)(&ve_info.eptp_index));
+
+  exception(BX_VE_EXCEPTION, 0);
+}
+
+void BX_CPU_C::vmx_page_modification_logging(Bit64u guest_paddr, unsigned dirty_update)
+{
+  VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+  if (vm->pml_index >= 512) {
+    Bit32u vmexit_qualification = 0;
+    if (BX_CPU_THIS_PTR nmi_unblocking_iret)
+      vmexit_qualification |= (1 << 12);
+
+    VMexit(VMX_VMEXIT_PML_LOGFULL, vmexit_qualification);
+  }
+
+  if (dirty_update) {
+    Bit64u pAddr = vm->pml_address + 8 * vm->pml_index;
+    access_write_physical(pAddr, 8, &guest_paddr);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 8, MEMTYPE(resolve_memtype(pAddr)), BX_WRITE, BX_VMX_PML_WRITE, (Bit8u*)(&guest_paddr));
+    vm->pml_index--;
+  }
+}
+#endif
+
 #endif // BX_SUPPORT_VMX
