@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: xsave.cc 12258 2014-03-23 20:01:58Z sshwarts $
+// $Id: xsave.cc 13123 2017-03-16 20:13:42Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008-2014 Stanislav Shwartsman
+//   Copyright (c) 2008-2017 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -34,9 +34,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVE(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 6
   BX_CPU_THIS_PTR prepareXSAVE();
 
+  bx_bool xsaveopt = (i->getIaOpcode() == BX_IA_XSAVEOPT);
+
   BX_DEBUG(("%s: save processor state XCR0=0x%08x", i->getIaOpcodeNameShort(), BX_CPU_THIS_PTR xcr0.get32()));
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   bx_address laddr = get_laddr(i->seg(), eaddr);
 
 #if BX_SUPPORT_ALIGNMENT_CHECK && BX_CPU_LEVEL >= 4
@@ -64,8 +66,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVE(bxInstruction_c *i)
   Bit32u requested_feature_bitmap = BX_CPU_THIS_PTR xcr0.get32() & EAX;
   Bit32u xinuse = get_xinuse_vector(requested_feature_bitmap);
 
-  bx_bool xsaveopt = (i->getIaOpcode() == BX_IA_XSAVEOPT);
-
   /////////////////////////////////////////////////////////////////////////////
   if ((requested_feature_bitmap & BX_XCR0_FPU_MASK) != 0)
   {
@@ -146,6 +146,19 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVE(bxInstruction_c *i)
   }
 #endif
 
+#if BX_SUPPORT_PKEYS
+  if ((requested_feature_bitmap & BX_XCR0_PKRU_MASK) != 0)
+  {
+    if (! xsaveopt || (xinuse & BX_XCR0_PKRU_MASK) != 0)
+      xsave_pkru_state(i, eaddr+XSAVE_PKRU_STATE_OFFSET);
+
+    if (xinuse & BX_XCR0_PKRU_MASK)
+      xstate_bv |=  BX_XCR0_PKRU_MASK;
+    else
+      xstate_bv &= ~BX_XCR0_PKRU_MASK;
+  }
+#endif
+
   // always update header to 'dirty' state
   write_virtual_qword(i->seg(), (eaddr + 512) & asize_mask, xstate_bv);
 #endif
@@ -159,9 +172,31 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 6
   BX_CPU_THIS_PTR prepareXSAVE();
 
+  bx_bool xsaves = (i->getIaOpcode() == BX_IA_XSAVES);
+  if (xsaves) {
+    if (CPL != 0) {
+      BX_ERROR(("%s: with CPL != 0", i->getIaOpcodeNameShort()));
+      exception(BX_GP_EXCEPTION, 0);
+    }
+
+#if BX_SUPPORT_VMX >= 2
+    if (BX_CPU_THIS_PTR in_vmx_guest) {
+      if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_XSAVES_XRSTORS)) {
+        BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
+        exception(BX_UD_EXCEPTION, 0);
+      }
+
+      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+      Bit64u requested_features = (((Bit64u) EDX) << 32) | EAX;
+      if (requested_features & BX_CPU_THIS_PTR msr.msr_xss & vm->xss_exiting_bitmap)
+        VMexit(VMX_VMEXIT_XSAVES, 0);
+    }
+#endif
+  }
+
   BX_DEBUG(("%s: save processor state XCR0=0x%08x", i->getIaOpcodeNameShort(), BX_CPU_THIS_PTR xcr0.get32()));
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   bx_address laddr = get_laddr(i->seg(), eaddr);
 
 #if BX_SUPPORT_ALIGNMENT_CHECK && BX_CPU_LEVEL >= 4
@@ -182,7 +217,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
   // We will go feature-by-feature and not run over all XCR0 bits
   //
 
-  Bit32u requested_feature_bitmap = BX_CPU_THIS_PTR xcr0.get32() & EAX;
+  Bit32u xcr0 = BX_CPU_THIS_PTR xcr0.get32();
+  if (xsaves)
+    xcr0 |= BX_CPU_THIS_PTR msr.msr_xss;
+
+  Bit32u requested_feature_bitmap = xcr0 & EAX;
   Bit32u xinuse = get_xinuse_vector(requested_feature_bitmap);
   Bit64u xstate_bv = requested_feature_bitmap & xinuse;
   Bit64u xcomp_bv = requested_feature_bitmap | XSAVEC_COMPACTION_ENABLED;
@@ -243,6 +282,16 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
   }
 #endif
 
+#if BX_SUPPORT_PKEYS
+  if ((requested_feature_bitmap & BX_XCR0_PKRU_MASK) != 0)
+  {
+    if (xinuse & BX_XCR0_PKRU_MASK)
+      xsave_pkru_state(i, eaddr+offset);
+
+    offset += XSAVE_PKRU_STATE_LEN;
+  }
+#endif
+
   bx_address asize_mask = i->asize_mask();
 
   // always update header to 'dirty' state
@@ -259,22 +308,44 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 6
   BX_CPU_THIS_PTR prepareXSAVE();
 
-  BX_DEBUG(("XRSTOR: restore processor state XCR0=0x%08x", BX_CPU_THIS_PTR xcr0.get32()));
+  bx_bool xrstors = (i->getIaOpcode() == BX_IA_XRSTORS);
+  if (xrstors) {
+    if (CPL != 0) {
+      BX_ERROR(("%s: with CPL != 0", i->getIaOpcodeNameShort()));
+      exception(BX_GP_EXCEPTION, 0);
+    }
 
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+#if BX_SUPPORT_VMX >= 2
+    if (BX_CPU_THIS_PTR in_vmx_guest) {
+      if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_XSAVES_XRSTORS)) {
+        BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
+        exception(BX_UD_EXCEPTION, 0);
+      }
+
+      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+      Bit64u requested_features = (((Bit64u) EDX) << 32) | EAX;
+      if (requested_features & BX_CPU_THIS_PTR msr.msr_xss & vm->xss_exiting_bitmap)
+        VMexit(VMX_VMEXIT_XRSTORS, 0);
+    }
+#endif
+  }
+
+  BX_DEBUG(("%s: restore processor state XCR0=0x%08x", i->getIaOpcodeNameShort(), BX_CPU_THIS_PTR xcr0.get32()));
+
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
   bx_address laddr = get_laddr(i->seg(), eaddr);
 
 #if BX_SUPPORT_ALIGNMENT_CHECK && BX_CPU_LEVEL >= 4
   if (BX_CPU_THIS_PTR alignment_check()) {
     if (laddr & 0x3) {
-      BX_ERROR(("XRSTOR: access not aligned to 4-byte cause model specific #AC(0)"));
+      BX_ERROR(("%s: access not aligned to 4-byte cause model specific #AC(0)", i->getIaOpcodeNameShort()));
       exception(BX_AC_EXCEPTION, 0);
     }
   }
 #endif
 
   if (laddr & 0x3f) {
-    BX_ERROR(("XRSTOR: access not aligned to 64-byte"));
+    BX_ERROR(("%s: access not aligned to 64-byte", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -285,7 +356,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
   Bit64u header3 = read_virtual_qword(i->seg(), (eaddr + 528) & asize_mask);
 
   if (header3 != 0) {
-    BX_ERROR(("XRSTOR: Reserved header state is not '0"));
+    BX_ERROR(("%s: Reserved header state is not '0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -293,25 +364,29 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
 
   if (! BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XSAVEC) || ! compaction) {
     if (xcomp_bv != 0) {
-      BX_ERROR(("XRSTOR: Reserved header state is not '0"));
+      BX_ERROR(("%s: Reserved header state is not '0", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
   }
 
+  Bit32u xcr0 = BX_CPU_THIS_PTR xcr0.get32();
+  if (xrstors)
+    xcr0 |= BX_CPU_THIS_PTR msr.msr_xss;
+
   if (! compaction) {
-    if ((~BX_CPU_THIS_PTR xcr0.get32() & xstate_bv) != 0 || (GET32H(xstate_bv) << 1) != 0) {
-      BX_ERROR(("XRSTOR: Invalid xsave_bv state"));
+    if ((~xcr0 & xstate_bv) != 0 || (GET32H(xstate_bv) << 1) != 0) {
+      BX_ERROR(("%s: Invalid xsave_bv state", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
   }
   else {
-    if ((~BX_CPU_THIS_PTR xcr0.get32() & xcomp_bv) != 0 || (GET32H(xcomp_bv) << 1) != 0) {
-      BX_ERROR(("XRSTOR: Invalid xcomp_bv state"));
+    if ((~xcr0 & xcomp_bv) != 0 || (GET32H(xcomp_bv) << 1) != 0) {
+      BX_ERROR(("%s: Invalid xcomp_bv state", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
     
     if (xstate_bv & ~xcomp_bv) {
-      BX_ERROR(("XRSTOR: Invalid xcomp_bv state"));
+      BX_ERROR(("%s: Invalid xcomp_bv state", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
 
@@ -322,7 +397,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
     Bit64u header8 = read_virtual_qword(i->seg(), (eaddr + 568) & asize_mask);
 
     if (header4 | header5 | header6 | header7 | header8) {
-      BX_ERROR(("XRSTOR: Reserved header state is not '0"));
+      BX_ERROR(("%s: Reserved header state is not '0", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
   }
@@ -331,7 +406,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
   // We will go feature-by-feature and not run over all XCR0 bits
   //
 
-  Bit32u requested_feature_bitmap = BX_CPU_THIS_PTR xcr0.get32() & EAX;
+  Bit32u requested_feature_bitmap = xcr0 & EAX;
 
   /////////////////////////////////////////////////////////////////////////////
   if ((requested_feature_bitmap & BX_XCR0_FPU_MASK) != 0)
@@ -412,6 +487,19 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
       offset += XSAVE_HI_ZMM_STATE_LEN;
     }
 #endif
+
+#if BX_SUPPORT_PKEYS
+    /////////////////////////////////////////////////////////////////////////////
+    if ((requested_feature_bitmap & BX_XCR0_PKRU_MASK) != 0)
+    {
+      if (xstate_bv & BX_XCR0_PKRU_MASK)
+        xrstor_pkru_state(i, eaddr+offset);
+      else
+        xrstor_init_pkru_state();
+
+      offset += XSAVE_PKRU_STATE_LEN;
+    }
+#endif
   }
   else {
 #if BX_SUPPORT_AVX
@@ -453,6 +541,18 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
         xrstor_init_hi_zmm_state();
     }
 #endif
+
+#if BX_SUPPORT_PKEYS
+    /////////////////////////////////////////////////////////////////////////////
+    if ((requested_feature_bitmap & BX_XCR0_PKRU_MASK) != 0)
+    {
+      if (xstate_bv & BX_XCR0_PKRU_MASK)
+        xrstor_pkru_state(i, eaddr+XSAVE_PKRU_STATE_OFFSET);
+      else
+        xrstor_init_pkru_state();
+    }
+#endif
+
   }
 #endif // BX_CPU_LEVEL >= 6
 
@@ -870,6 +970,31 @@ bx_bool BX_CPU_C::xsave_hi_zmm_state_xinuse(void)
 
 #endif // BX_SUPPORT_AVX
 
+#if BX_SUPPORT_PKEYS
+// PKRU state management //
+
+void BX_CPU_C::xsave_pkru_state(bxInstruction_c *i, bx_address offset)
+{
+  write_virtual_qword(i->seg(), offset, (Bit64u) BX_CPU_THIS_PTR pkru);
+}
+
+void BX_CPU_C::xrstor_pkru_state(bxInstruction_c *i, bx_address offset)
+{
+  Bit32u pkru = read_virtual_dword(i->seg(), offset);
+  set_PKRU(pkru);
+}
+
+void BX_CPU_C::xrstor_init_pkru_state(void)
+{
+  set_PKRU(0);
+}
+
+bx_bool BX_CPU_C::xsave_pkru_state_xinuse(void)
+{
+  return (BX_CPU_THIS_PTR pkru != 0);
+}
+#endif
+
 Bit32u BX_CPU_C::get_xinuse_vector(Bit32u requested_feature_bitmap)
 {
   Bit32u xinuse = 0;
@@ -903,6 +1028,13 @@ Bit32u BX_CPU_C::get_xinuse_vector(Bit32u requested_feature_bitmap)
 #endif
 #endif
 
+#if BX_SUPPORT_PKEYS
+  if (requested_feature_bitmap & BX_XCR0_PKRU_MASK) {
+    if (xsave_pkru_state_xinuse()) 
+      xinuse |= BX_XCR0_PKRU_MASK;
+  }
+#endif
+
   return xinuse;
 }
 
