@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.h 11612 2013-02-04 18:51:07Z vruppert $
+// $Id: siminterface.h 13073 2017-02-16 21:43:52Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2013  The Bochs Project
+//  Copyright (C) 2001-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -168,19 +168,22 @@ typedef enum {
 typedef enum {
   ACT_IGNORE = 0,
   ACT_REPORT,
+  ACT_WARN,
   ACT_ASK,
   ACT_FATAL,
   N_ACT
 } bx_log_actions;
 
+#define BX_NULL_PREFIX  "[      ]"
+
 // normally all action choices are available for all event types. The exclude
 // expression allows some choices to be eliminated if they don't make any
 // sense.  For example, it would be stupid to ignore a panic.
-#define BX_LOG_OPTS_EXCLUDE(type, choice)  (                           \
-   /* can't die or ask, on debug or info events */                     \
-   (type <= LOGLEV_INFO && (choice == ACT_ASK || choice == ACT_FATAL)) \
-   /* can't ignore panics */                                           \
-   || (type == LOGLEV_PANIC && choice == ACT_IGNORE)                   \
+#define BX_LOG_OPTS_EXCLUDE(type, choice)  (             \
+   /* can't die, ask or warn, on debug or info events */ \
+   (type <= LOGLEV_INFO && (choice >= ACT_WARN))         \
+   /* can't ignore panics */                             \
+   || (type == LOGLEV_PANIC && choice == ACT_IGNORE)     \
    )
 
 // floppy / cdrom media status
@@ -270,7 +273,7 @@ typedef enum {
   BX_SYNC_EVT_GET_PARAM,          // CI -> simulator -> CI
   BX_SYNC_EVT_ASK_PARAM,          // simulator -> CI -> simulator
   BX_SYNC_EVT_TICK,               // simulator -> CI, wait for response.
-  BX_SYNC_EVT_LOG_ASK,            // simulator -> CI, wait for response.
+  BX_SYNC_EVT_LOG_DLG,            // simulator -> CI, wait for response.
   BX_SYNC_EVT_GET_DBG_COMMAND,    // simulator -> CI, wait for response.
   __ALL_EVENTS_BELOW_ARE_ASYNC__,
   BX_ASYNC_EVT_KEY,               // vga window -> simulator
@@ -369,7 +372,7 @@ typedef struct {
 // some kind of "flow control" since the simulator will be able to produce
 // new events much faster than the gui can accept them.
 
-// Event type: BX_ASYNC_EVT_LOG_MSG   (unused)
+// Event type: BX_ASYNC_EVT_LOG_MSG
 //
 // Asynchronous event from the simulator to the CI.  When a BX_PANIC,
 // BX_ERROR, BX_INFO, or BX_DEBUG is found in the simulator code, this
@@ -390,6 +393,7 @@ typedef struct {
 // synchronizing threads, etc. for each.
 typedef struct {
   Bit8u level;
+  Bit8u mode;
   const char *prefix;
   const char *msg;
 } BxLogMsgEvent;
@@ -399,11 +403,11 @@ typedef struct {
 // Also uses BxLogMsgEvent, but this is a message to be displayed in
 // the debugger history window.
 
-// Event type: BX_SYNC_EVT_LOG_ASK
+// Event type: BX_SYNC_EVT_LOG_DLG
 //
 // This is a synchronous version of BX_ASYNC_EVT_LOG_MSG, which is used
 // when the "action=ask" setting is used.  If the simulator runs into a
-// panic, it sends a synchronous BX_SYNC_EVT_LOG_ASK to the CI to be
+// panic, it sends a synchronous BX_SYNC_EVT_LOG_DLG to the CI to be
 // displayed.  The CI shows a dialog that asks if the user wants to
 // continue, quit, etc. and sends the answer back to the simulator.
 // This event also uses BxLogMsgEvent.
@@ -417,6 +421,12 @@ enum {
   BX_LOG_NOTIFY_FAILED
 };
 
+enum {
+  BX_LOG_DLG_ASK,
+  BX_LOG_DLG_WARN,
+  BX_LOG_DLG_QUIT
+};
+
 // Event type: BX_SYNC_EVT_GET_DBG_COMMAND
 //
 // This is a synchronous event sent from the simulator to the debugger
@@ -445,7 +455,7 @@ typedef struct {
 // a type and a spot for a return code (only used for synchronous events).
 typedef struct {
   BxEventType type; // what kind is this?
-  Bit32s retcode;   // sucess or failure. only used for synchronous events.
+  Bit32s retcode;   // success or failure. only used for synchronous events.
   union {
     BxKeyEvent key;
     BxMouseEvent mouse;
@@ -480,6 +490,7 @@ enum {
   BX_MOUSE_TYPE_PS2,
   BX_MOUSE_TYPE_IMPS2,
 #if BX_SUPPORT_BUSMOUSE
+  BX_MOUSE_TYPE_INPORT,
   BX_MOUSE_TYPE_BUS,
 #endif
   BX_MOUSE_TYPE_SERIAL,
@@ -519,9 +530,9 @@ enum {
 #define BX_ATA_DEVICE_DISK       1
 #define BX_ATA_DEVICE_CDROM      2
 
-#define BX_ATA_BIOSDETECT_NONE   0
-#define BX_ATA_BIOSDETECT_AUTO   1
-#define BX_ATA_BIOSDETECT_CMOS   2
+#define BX_ATA_BIOSDETECT_AUTO   0
+#define BX_ATA_BIOSDETECT_CMOS   1
+#define BX_ATA_BIOSDETECT_NONE   2
 
 enum {
   BX_ATA_TRANSLATION_NONE,
@@ -544,9 +555,10 @@ enum {
   BX_HDIMAGE_MODE_GROWING,
   BX_HDIMAGE_MODE_VOLATILE,
   BX_HDIMAGE_MODE_VVFAT,
-  BX_HDIMAGE_MODE_VPC
+  BX_HDIMAGE_MODE_VPC,
+  BX_HDIMAGE_MODE_VBOX
 };
-#define BX_HDIMAGE_MODE_LAST     BX_HDIMAGE_MODE_VPC
+#define BX_HDIMAGE_MODE_LAST     BX_HDIMAGE_MODE_VBOX
 #define BX_HDIMAGE_MODE_UNKNOWN  -1
 
 enum {
@@ -569,7 +581,14 @@ enum {
   BX_CPUID_SUPPORT_SSE3,
   BX_CPUID_SUPPORT_SSSE3,
   BX_CPUID_SUPPORT_SSE4_1,
-  BX_CPUID_SUPPORT_SSE4_2
+  BX_CPUID_SUPPORT_SSE4_2,
+#if BX_SUPPORT_AVX
+  BX_CPUID_SUPPORT_AVX,
+  BX_CPUID_SUPPORT_AVX2,
+#if BX_SUPPORT_EVEX
+  BX_CPUID_SUPPORT_AVX512
+#endif
+#endif
 };
 
 enum {
@@ -584,12 +603,33 @@ enum {
 #define BX_CLOCK_TIME0_LOCAL     1
 #define BX_CLOCK_TIME0_UTC       2
 
+enum {
+  BX_SOUNDDRV_DUMMY,
+#if BX_HAVE_SOUND_ALSA
+  BX_SOUNDDRV_ALSA,
+#endif
+#if BX_HAVE_SOUND_OSS
+  BX_SOUNDDRV_OSS,
+#endif
+#if BX_HAVE_SOUND_OSX
+  BX_SOUNDDRV_OSX,
+#endif
+#if BX_HAVE_SOUND_SDL
+  BX_SOUNDDRV_SDL,
+#endif
+#if BX_HAVE_SOUND_WIN
+  BX_SOUNDDRV_WIN,
+#endif
+  BX_SOUNDDRV_FILE
+};
+
 BOCHSAPI extern const char *floppy_devtype_names[];
 BOCHSAPI extern const char *floppy_type_names[];
 BOCHSAPI extern int floppy_type_n_sectors[];
 BOCHSAPI extern const char *media_status_names[];
 BOCHSAPI extern const char *bochs_bootdisk_names[];
 BOCHSAPI extern const char *hdimage_mode_names[];
+BOCHSAPI extern const char *sound_driver_names[];
 
 ////////////////////////////////////////////////////////////////////
 // base class simulator interface, contains just virtual functions.
@@ -624,8 +664,8 @@ public:
   bx_simulator_interface_c() {}
   virtual ~bx_simulator_interface_c() {}
   virtual void set_quit_context(jmp_buf *context) {}
-  virtual int get_init_done() { return -1; }
-  virtual int set_init_done(int n) {return -1;}
+  virtual int get_init_done() { return 0; }
+  virtual int set_init_done(int n) {return 0;}
   virtual void reset_all_param() {}
   // new param methods
   virtual bx_param_c *get_param(const char *pname, bx_param_c *base=NULL) {return NULL;}
@@ -635,15 +675,16 @@ public:
   virtual bx_param_enum_c *get_param_enum(const char *pname, bx_param_c *base=NULL) {return NULL;}
   virtual unsigned gen_param_id() {return 0;}
   virtual int get_n_log_modules() {return -1;}
-  virtual const char *get_logfn_name(int mod) {return 0;}
+  virtual const char *get_logfn_name(int mod) {return NULL;}
   virtual int get_logfn_id(const char *name) {return -1;}
-  virtual const char *get_prefix(int mod) {return 0;}
+  virtual const char *get_prefix(int mod) {return NULL;}
   virtual int get_log_action(int mod, int level) {return -1;}
   virtual void set_log_action(int mod, int level, int action) {}
   virtual int get_default_log_action(int level) {return -1;}
   virtual void set_default_log_action(int level, int action) {}
-  virtual const char *get_action_name(int action) {return 0;}
-  virtual const char *get_log_level_name(int level) {return 0;}
+  virtual const char *get_action_name(int action) {return NULL;}
+  virtual int is_action_name(const char *val) {return -1;}
+  virtual const char *get_log_level_name(int level) {return NULL;}
   virtual int get_max_log_level() {return -1;}
 
   // exiting is somewhat complicated!  The preferred way to exit bochs is
@@ -665,7 +706,6 @@ public:
   virtual int set_log_prefix(char *prefix) {return -1;}
   virtual int get_debugger_log_file(char *path, int len) {return -1;}
   virtual int set_debugger_log_file(char *path) {return -1;}
-  virtual int get_cdrom_options(int drive, bx_list_c **out, int *where = NULL) {return -1;}
   virtual int hdimage_get_mode(const char *mode)  {return -1;}
 
   // The CI calls set_notify_callback to register its event handler function.
@@ -684,9 +724,16 @@ public:
   // send an event from the simulator to the CI.
   virtual BxEvent* sim_to_ci_event(BxEvent *event) {return NULL;}
 
-  // called from simulator when it hits serious errors, to ask if the user
-  // wants to continue or not
-  virtual int log_msg(const char *prefix, int level, const char *msg) {return -1;}
+  // called from simulator to display a gui dialog in particular situations.
+  // 1. When it hits serious errors, to ask if the user wants to continue or not.
+  // 2. When it hits errors, to warn the user before continuing simulation
+  // 3. When it hits critical errors, inform the user before terminating simulation.
+  virtual int log_dlg(const char *prefix, int level, const char *msg, int mode) {return -1;}
+  // called from simulator when writing a message to log file
+  virtual void log_msg(const char *prefix, int level, const char *msg) {}
+  // set this to 1 if the gui has a log viewer
+  virtual void set_log_viewer(bx_bool val) {}
+  virtual bx_bool has_log_viewer() const {return 0;}
 
   // tell the CI to ask the user for the value of a parameter.
   virtual int ask_param(bx_param_c *param) {return -1;}
@@ -730,7 +777,8 @@ public:
     void *userdata) {}
   virtual int configuration_interface(const char* name, ci_command_t command) {return -1; }
   virtual int begin_simulation(int argc, char *argv[]) {return -1;}
-  virtual bx_bool register_runtime_config_handler(void *dev, rt_conf_handler_t handler) {return 0;}
+  virtual int register_runtime_config_handler(void *dev, rt_conf_handler_t handler) {return 0;}
+  virtual void unregister_runtime_config_handler(int id) {}
   virtual void update_runtime_options() {}
   typedef bx_bool (*is_sim_thread_func_t)();
   is_sim_thread_func_t is_sim_thread_func;
@@ -745,12 +793,19 @@ public:
   // interfaces to use.
   virtual void set_display_mode(disp_mode_t newmode) {}
   virtual bx_bool test_for_text_console() {return 1;}
+
   // add-on config option support
   virtual bx_bool register_addon_option(const char *keyword, addon_option_parser_t parser, addon_option_save_t save_func) {return 0;}
   virtual bx_bool unregister_addon_option(const char *keyword) {return 0;}
   virtual bx_bool is_addon_option(const char *keyword) {return 0;}
   virtual Bit32s parse_addon_option(const char *context, int num_params, char *params []) {return -1;}
   virtual Bit32s save_addon_options(FILE *fp) {return -1;}
+
+  // statistics
+  virtual void init_statistics() {}
+  virtual void cleanup_statistics() {}
+  virtual bx_list_c *get_statistics_root() {return NULL;}
+ 
   // save/restore support
   virtual void init_save_restore() {}
   virtual void cleanup_save_restore() {}
@@ -760,6 +815,7 @@ public:
   virtual bx_bool restore_hardware() {return 0;}
   virtual bx_list_c *get_bochs_root() {return NULL;}
   virtual bx_bool restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name) { return 0; }
+
   // special config parameter and options functions for plugins
   virtual bx_bool opt_plugin_ctrl(const char *plugname, bx_bool load) {return 0;}
   virtual void init_std_nic_options(const char *name, bx_list_c *menu) {}
@@ -770,12 +826,16 @@ public:
                                      const char *param, int maxports, bx_list_c *base) {return 0;}
   virtual int  write_param_list(FILE *fp, bx_list_c *base, const char *optname, bx_bool multiline) {return 0;}
   virtual int  write_usb_options(FILE *fp, int maxports, bx_list_c *base) {return 0;}
+
+#if BX_USE_GUI_CONSOLE
+  virtual int  bx_printf(const char *fmt, ...) {return 0;}
+  virtual char* bx_gets(char *s, int size, FILE *stream) {return NULL;}
+#endif
 };
 
 BOCHSAPI extern bx_simulator_interface_c *SIM;
 
-BOCHSAPI extern void bx_init_siminterface();
-BOCHSAPI extern int bx_init_main(int argc, char *argv[]);
+extern void bx_init_siminterface();
 
 #if defined(__WXMSW__) || defined(WIN32)
 // Just to provide HINSTANCE, etc. in files that have not included bochs.h.
