@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: term.cc 12571 2014-12-18 20:29:37Z vruppert $
+// $Id: term.cc 13042 2017-01-15 11:44:43Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2000-2014  The Bochs Project
+//  Copyright (C) 2000-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,8 @@ extern "C" {
 #include <signal.h>
 };
 
+#define BX_DEBUGGER_TERM (BX_DEBUGGER && !defined(__OpenBSD__))
+
 class bx_term_gui_c : public bx_gui_c {
 public:
   bx_term_gui_c(void) {}
@@ -50,7 +52,7 @@ public:
 // declare one instance of the gui object and call macro to insert the
 // plugin code
 static bx_term_gui_c *theGui = NULL;
-#if BX_DEBUGGER
+#if BX_DEBUGGER_TERM
 static int scr_fd = -1;
 #endif
 IMPLEMENT_GUI_PLUGIN_CODE(term)
@@ -164,18 +166,7 @@ void bx_term_gui_c::sighandler(int signo)
   }
 }
 
-// ::SPECIFIC_INIT()
-//
-// Called from gui.cc, once upon program startup, to allow for the
-// specific GUI code (X11, Win32, ...) to be initialized.
-//
-// argc, argv: not used right now, but the intention is to pass native GUI
-//     specific options from the command line.  (X11 options, Win32 options,...)
-//
-// headerbar_y:  A headerbar (toolbar) is display on the top of the
-//     VGA window, showing floppy status, and other information.  It
-//     always assumes the width of the current VGA mode width, but
-//     it's height is defined by this parameter.
+// TERM implementation of the bx_gui_c methods (see nogui.cc for details)
 
 void bx_term_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 {
@@ -183,7 +174,7 @@ void bx_term_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 
   // the ask menu causes trouble
   io->set_log_action(LOGLEV_PANIC, ACT_FATAL);
-#if !BX_DEBUGGER
+#if !BX_DEBUGGER_TERM
   // logfile should be different from stderr, otherwise terminal mode
   // really ends up having fun
   if (!strcmp(SIM->get_param_string(BXPN_LOG_FILENAME)->getptr(), "-"))
@@ -200,7 +191,7 @@ void bx_term_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
   }
 #endif
   initscr();
-#if BX_DEBUGGER
+#if BX_DEBUGGER_TERM
   stdin = old_stdin;
   stdout = old_stdout;
 #endif
@@ -422,12 +413,6 @@ void do_char(int character, int alt)
   }
 }
 
-// ::HANDLE_EVENTS()
-//
-// Called periodically (vga_update_interval in .bochsrc) so the
-// the gui code can poll for keyboard, mouse, and other
-// relevant events.
-
 void bx_term_gui_c::handle_events(void)
 {
   int character;
@@ -437,21 +422,11 @@ void bx_term_gui_c::handle_events(void)
   }
 }
 
-// ::FLUSH()
-//
-// Called periodically, requesting that the gui code flush all pending
-// screen update requests.
-
 void bx_term_gui_c::flush(void)
 {
   if (initialized) refresh();
 }
 
-// ::CLEAR_SCREEN()
-//
-// Called to request that the VGA region is cleared.  Don't
-// clear the area that defines the headerbar.
-
 void bx_term_gui_c::clear_screen(void)
 {
   clear();
@@ -550,25 +525,6 @@ chtype get_term_char(Bit8u vga_char[])
   return term_char;
 }
 
-// ::TEXT_UPDATE()
-//
-// Called in a VGA text mode, to update the screen with
-// new content.
-//
-// old_text: array of character/attributes making up the contents
-//           of the screen from the last call.  See below
-// new_text: array of character/attributes making up the current
-//           contents, which should now be displayed.  See below
-//
-// format of old_text & new_text: each is tm_info->line_offset*text_rows
-//     bytes long. Each character consists of 2 bytes.  The first by is
-//     the character value, the second is the attribute byte.
-//
-// cursor_x: new x location of cursor
-// cursor_y: new y location of cursor
-// tm_info:  this structure contains information for additional
-//           features in text mode (cursor shape, line offset,...)
-
 void bx_term_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
         unsigned long cursor_x, unsigned long cursor_y,
         bx_vga_tminfo_t *tm_info)
@@ -648,13 +604,6 @@ int bx_term_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
   return 0;
 }
 
-// ::PALETTE_CHANGE()
-//
-// Allocate a color in the native GUI, for this color, and put
-// it in the colormap location 'index'.
-// returns: 0=no screen update needed (color map change has direct effect)
-//          1=screen updated needed (redraw using current colormap)
-
 bx_bool bx_term_gui_c::palette_change(Bit8u index, Bit8u red, Bit8u green, Bit8u blue)
 {
   BX_DEBUG(("color pallete request (%d,%d,%d,%d) ignored", index,red,green,blue));
@@ -662,37 +611,10 @@ bx_bool bx_term_gui_c::palette_change(Bit8u index, Bit8u red, Bit8u green, Bit8u
 }
 
 
-// ::GRAPHICS_TILE_UPDATE()
-//
-// Called to request that a tile of graphics be drawn to the
-// screen, since info in this region has changed.
-//
-// tile: array of 8bit values representing a block of pixels with
-//       dimension equal to the 'x_tilesize' & 'y_tilesize' members.
-//       Each value specifies an index into the
-//       array of colors you allocated for ::palette_change()
-// x0: x origin of tile
-// y0: y origin of tile
-//
-// note: origin of tile and of window based on (0,0) being in the upper
-//       left of the window.
-
 void bx_term_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 {
 }
 
-// ::DIMENSION_UPDATE()
-//
-// Called when the VGA mode changes it's X,Y dimensions.
-// Resize the window to this size, but you need to add on
-// the height of the headerbar to the Y value.
-//
-// x: new VGA x size
-// y: new VGA y size (add headerbar_y parameter from ::specific_init().
-// fheight: new VGA character height in text mode
-// fwidth : new VGA character width in text mode
-// bpp : bits per pixel in graphics mode
-
 void bx_term_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, unsigned fwidth, unsigned bpp)
 {
   if (bpp > 8) {
@@ -724,76 +646,28 @@ void bx_term_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, u
   }
 }
 
-// ::CREATE_BITMAP()
-//
-// Create a monochrome bitmap of size 'xdim' by 'ydim', which will
-// be drawn in the headerbar.  Return an integer ID to the bitmap,
-// with which the bitmap can be referenced later.
-//
-// bmap: packed 8 pixels-per-byte bitmap.  The pixel order is:
-//       bit0 is the left most pixel, bit7 is the right most pixel.
-// xdim: x dimension of bitmap
-// ydim: y dimension of bitmap
-
 unsigned bx_term_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim, unsigned ydim)
 {
   return(0);
 }
 
-// ::HEADERBAR_BITMAP()
-//
-// Called to install a bitmap in the bochs headerbar (toolbar).
-//
-// bmap_id: will correspond to an ID returned from
-//     ::create_bitmap().  'alignment' is either BX_GRAVITY_LEFT
-//     or BX_GRAVITY_RIGHT, meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// alignment: is either BX_GRAVITY_LEFT or BX_GRAVITY_RIGHT,
-//     meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// f: a 'C' function pointer to callback when the mouse is clicked in
-//     the boundaries of this bitmap.
-
 unsigned bx_term_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment, void (*f)(void))
 {
   return(0);
 }
 
-// ::SHOW_HEADERBAR()
-//
-// Show (redraw) the current headerbar, which is composed of
-// currently installed bitmaps.
-
 void bx_term_gui_c::show_headerbar(void)
 {
 }
 
-// ::REPLACE_BITMAP()
-//
-// Replace the bitmap installed in the headerbar ID slot 'hbar_id',
-// with the one specified by 'bmap_id'.  'bmap_id' will have
-// been generated by ::create_bitmap().  The old and new bitmap
-// must be of the same size.  This allows the bitmap the user
-// sees to change, when some action occurs.  For example when
-// the user presses on the floppy icon, it then displays
-// the ejected status.
-//
-// hbar_id: headerbar slot ID
-// bmap_id: bitmap ID
-
 void bx_term_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 {
 }
 
-// ::EXIT()
-//
-// Called before bochs terminates, to allow for a graceful
-// exit from the native GUI mechanism.
-
 void bx_term_gui_c::exit(void)
 {
   if (!initialized) return;
-#if BX_DEBUGGER
+#if BX_DEBUGGER_TERM
   if(scr_fd > 0)
     close(scr_fd);
 #endif
