@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vncsrv.cc 12606 2015-01-17 19:53:03Z vruppert $
+// $Id: vncsrv.cc 13075 2017-02-18 11:13:56Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2000  Psyon.Org!
@@ -7,7 +7,7 @@
 //    Donald Becker
 //    http://www.psyon.org
 //
-//  Copyright (C) 2001-2015  The Bochs Project
+//  Copyright (C) 2001-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -52,11 +52,7 @@
 
 #include "icon_bochs.h"
 #include "font/vga.bitmap.h"
-#if (BX_WITH_SDL || BX_WITH_SDL2 || BX_WITH_RFB) && !BX_PLUGINS
-extern unsigned char sdl_font8x8[256][8];
-#else
 #include "sdl.h" // 8x8 font for status text
-#endif
 
 #define HAVE_LIBVNCSERVER
 
@@ -68,6 +64,7 @@ public:
   bx_vncsrv_gui_c(void) : screen(NULL) {}
   DECLARE_GUI_VIRTUAL_METHODS()
   DECLARE_GUI_NEW_VIRTUAL_METHODS()
+  virtual void set_display_mode(disp_mode_t newmode);
   void get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp);
   void statusbar_setitem_specific(int element, bx_bool active, bx_bool w);
   virtual void set_mouse_mode_absxy(bx_bool mode);
@@ -75,6 +72,9 @@ public:
   void show_ips(Bit32u ips_count);
 #endif
   rfbScreenInfoPtr screen;
+private:
+  void vncMouseMove(int x, int y, int z, int bmask);
+  void vncKeyPressed(Bit32u key, int press_release);
 };
 
 // declare one instance of the gui object and call macro to insert the
@@ -120,16 +120,7 @@ static struct _rfbBitmaps {
     unsigned ydim;
 } rfbBitmaps[BX_MAX_PIXMAPS];
 
-static unsigned rfbHeaderbarBitmapCount = 0;
-static struct _rfbHeaderbarBitmaps {
-    unsigned int index;
-    unsigned int xorigin;
-    unsigned int yorigin;
-    unsigned int alignment;
-    void (*f)(void);
-} rfbHeaderbarBitmaps[BX_MAX_HEADERBAR_ENTRIES];
-
-//Keyboard stuff
+// Keyboard/Mouse stuff
 #define KEYBOARD 1
 #define MOUSE    0
 #define MAX_KEY_EVENTS 512
@@ -210,8 +201,6 @@ void UpdateScreen(rfbPixel *newBits, int x, int y, int width, int height);
 void SendUpdate(int x, int y, int width, int height);
 void vncSetStatusText(int element, const char *text, bx_bool active, bx_bool w = 0);
 static Bit32u convertStringToRfbKey(const char *string);
-void vncKeyPressed(Bit32u key, int press_release);
-void vncMouseMove(int x, int y, int z, int bmask);
 
 void clientgone(rfbClientPtr cl);
 enum rfbNewClientAction newclient(rfbClientPtr cl);
@@ -224,24 +213,10 @@ DWORD WINAPI vncShowIPSthread(LPVOID);
 #endif
 
 
-// VNCSRV implementation of the bx_gui_c methods
-
-// ::SPECIFIC_INIT()
-//
-// Called from gui.cc, once upon program startup, to allow for the
-// specific GUI code (X11, Win32, ...) to be initialized.
-//
-// argc, argv: used to pass display library specific options to the init code
-//     (X11 options, Win32 options,...)
-//
-// headerbar_y:  A headerbar (toolbar) is display on the top of the
-//     VGA window, showing floppy status, and other information.  It
-//     always assumes the width of the current VGA mode width, but
-//     it's height is defined by this parameter.
+// VNCSRV implementation of the bx_gui_c methods (see nogui.cc for details)
 
 void bx_vncsrv_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 {
-  unsigned char fc, vc;
   int i, timeout = 30;
 
   put("VNCSRV");
@@ -260,13 +235,7 @@ void bx_vncsrv_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 
   for (i = 0; i < 256; i++) {
     for (int j = 0; j < 16; j++) {
-      vc = bx_vgafont[i].data[j];
-      fc = 0;
-      for (int b = 0; b < 8; b++) {
-        fc |= (vc & 0x01) << (7 - b);
-        vc >>= 1;
-      }
-      vga_charmap[i * 32 + j] = fc;
+      vga_charmap[i * 32 + j] = reverse_bitorder(bx_vgafont[i].data[j]);
     }
   }
 
@@ -300,7 +269,7 @@ void bx_vncsrv_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
     BX_PANIC(("Create VNC screen failed!"));
 
   screen->desktopName = "Bochs VNC Screen";
-  screen->frameBuffer = (char *) malloc(rfbWindowX * rfbWindowY * sizeof(rfbPixel));
+  screen->frameBuffer = new char[rfbWindowX * rfbWindowY * sizeof(rfbPixel)];
   screen->alwaysShared = TRUE;
   screen->ptrAddEvent = doptr;
   screen->kbdAddEvent = dokey;
@@ -354,15 +323,9 @@ void bx_vncsrv_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 #endif
 
   new_gfx_api = 1;
-  dialog_caps = 0;
+  console.present = 1;
 }
 
-// ::HANDLE_EVENTS()
-//
-// Called periodically (vga_update_interval in .bochsrc) so the
-// the gui code can poll for keyboard, mouse, and other
-// relevant events.
-
 void bx_vncsrv_gui_c::handle_events(void)
 {
   BX_LOCK(bKeyboardInUse);
@@ -387,19 +350,10 @@ void bx_vncsrv_gui_c::handle_events(void)
 #endif
 }
 
-// ::FLUSH()
-//
-// Called periodically, requesting that the gui code flush all pending
-// screen update requests.
-
 void bx_vncsrv_gui_c::flush(void)
 {
 }
 
-// ::CLEAR_SCREEN()
-//
-// Called to request that the VGA region is cleared.  Don't
-// clear the area that defines the headerbar.
 void bx_vncsrv_gui_c::clear_screen(void)
 {
   memset(&(screen->frameBuffer[rfbWindowX * rfbHeaderbarY * sizeof(rfbPixel)]),
@@ -407,25 +361,6 @@ void bx_vncsrv_gui_c::clear_screen(void)
   SendUpdate(0, rfbHeaderbarY, rfbDimensionX, rfbDimensionY);
 }
 
-// ::TEXT_UPDATE()
-//
-// Called in a VGA text mode, to update the screen with
-// new content.
-//
-// old_text: array of character/attributes making up the contents
-//           of the screen from the last call.  See below
-// new_text: array of character/attributes making up the current
-//           contents, which should now be displayed.  See below
-//
-// format of old_text & new_text: each is tm_info->line_offset*text_rows
-//     bytes long. Each character consists of 2 bytes.  The first by is
-//     the character value, the second is the attribute byte.
-//
-// cursor_x: new x location of cursor
-// cursor_y: new y location of cursor
-// tm_info:  this structure contains information for additional
-//           features in text mode (cursor shape, line offset,...)
-
 void bx_vncsrv_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
         unsigned long cursor_x, unsigned long cursor_y,
         bx_vga_tminfo_t *tm_info)
@@ -529,13 +464,6 @@ int bx_vncsrv_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
   return 0;
 }
 
-// ::PALETTE_CHANGE()
-//
-// Allocate a color in the native GUI, for this color, and put
-// it in the colormap location 'index'.
-// returns: 0=no screen update needed (color map change has direct effect)
-//          1=screen updated needed (redraw using current colormap)
-
 bx_bool bx_vncsrv_gui_c::palette_change(U8 index, U8 red,
         U8 green, U8 blue)
 {
@@ -543,20 +471,6 @@ bx_bool bx_vncsrv_gui_c::palette_change(U8 index, U8 red,
   return 1;
 }
 
-// ::GRAPHICS_TILE_UPDATE()
-//
-// Called to request that a tile of graphics be drawn to the
-// screen, since info in this region has changed.
-//
-// tile: array of 8bit values representing a block of pixels with
-//       dimension equal to the 'x_tilesize' & 'y_tilesize' members.
-//       Each value specifies an index into the
-//       array of colors you allocated for ::palette_change()
-// x0: x origin of tile
-// y0: y origin of tile
-//
-// note: origin of tile and of window based on (0,0) being in the upper
-//       left of the window.
 void bx_vncsrv_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 {
   rfbPixel *buf;
@@ -592,18 +506,6 @@ void bx_vncsrv_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0
   SendUpdate(x0, y0 + rfbHeaderbarY, rfbTileX, rfbTileY);
 }
 
-// ::DIMENSION_UPDATE()
-//
-// Called when the VGA mode changes it's X,Y dimensions.
-// Resize the window to this size, but you need to add on
-// the height of the headerbar to the Y value.
-//
-// x: new VGA x size
-// y: new VGA y size (add headerbar_y parameter from ::specific_init().
-// fheight: new VGA character height in text mode
-// fwidth : new VGA character width in text mode
-// bpp : bits per pixel in graphics mode
-
 void bx_vncsrv_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
         unsigned fwidth, unsigned bpp)
 {
@@ -613,6 +515,7 @@ void bx_vncsrv_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
     BX_PANIC(("%d bpp graphics mode not supported", bpp));
   }
   guest_textmode = (fheight > 0);
+  guest_fsize = (fheight << 4) | fwidth;
   guest_xres = x;
   guest_yres = y;
   if (guest_textmode) {
@@ -645,17 +548,6 @@ void bx_vncsrv_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
   }
 }
 
-// ::CREATE_BITMAP()
-//
-// Create a monochrome bitmap of size 'xdim' by 'ydim', which will
-// be drawn in the headerbar.  Return an integer ID to the bitmap,
-// with which the bitmap can be referenced later.
-//
-// bmap: packed 8 pixels-per-byte bitmap.  The pixel order is:
-//       bit0 is the left most pixel, bit7 is the right most pixel.
-// xdim: x dimension of bitmap
-// ydim: y dimension of bitmap
-
 unsigned bx_vncsrv_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim,
         unsigned ydim)
 {
@@ -663,7 +555,7 @@ unsigned bx_vncsrv_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim
     BX_ERROR(("too many pixmaps."));
     return 0;
   }
-  rfbBitmaps[rfbBitmapCount].bmap = (char *) malloc((xdim * ydim) / 8);
+  rfbBitmaps[rfbBitmapCount].bmap = new char[(xdim * ydim) / 8];
   rfbBitmaps[rfbBitmapCount].xdim = xdim;
   rfbBitmaps[rfbBitmapCount].ydim = ydim;
   memcpy(rfbBitmaps[rfbBitmapCount].bmap, bmap, (xdim * ydim) / 8);
@@ -672,73 +564,52 @@ unsigned bx_vncsrv_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim
   return (rfbBitmapCount - 1);
 }
 
-// ::HEADERBAR_BITMAP()
-//
-// Called to install a bitmap in the bochs headerbar (toolbar).
-//
-// bmap_id: will correspond to an ID returned from
-//     ::create_bitmap().  'alignment' is either BX_GRAVITY_LEFT
-//     or BX_GRAVITY_RIGHT, meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// alignment: is either BX_GRAVITY_LEFT or BX_GRAVITY_RIGHT,
-//     meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// f: a 'C' function pointer to callback when the mouse is clicked in
-//     the boundaries of this bitmap.
-
 unsigned bx_vncsrv_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment,
         void (*f)(void))
 {
   int hb_index;
 
-  if ((rfbHeaderbarBitmapCount + 1) > BX_MAX_HEADERBAR_ENTRIES) {
+  if ((bx_headerbar_entries + 1) > BX_MAX_HEADERBAR_ENTRIES) {
     return 0;
   }
 
-  rfbHeaderbarBitmapCount++;
-  hb_index = rfbHeaderbarBitmapCount - 1;
-  rfbHeaderbarBitmaps[hb_index].index = bmap_id;
-  rfbHeaderbarBitmaps[hb_index].alignment = alignment;
-  rfbHeaderbarBitmaps[hb_index].f = f;
+  hb_index = bx_headerbar_entries++;
+  bx_headerbar_entry[hb_index].bmap_id = bmap_id;
+  bx_headerbar_entry[hb_index].xdim = rfbBitmaps[bmap_id].xdim;
+  bx_headerbar_entry[hb_index].ydim = rfbBitmaps[bmap_id].ydim;
+  bx_headerbar_entry[hb_index].alignment = alignment;
+  bx_headerbar_entry[hb_index].f = f;
   if (alignment == BX_GRAVITY_LEFT) {
-    rfbHeaderbarBitmaps[hb_index].xorigin = rfbOriginLeft;
-    rfbHeaderbarBitmaps[hb_index].yorigin = 0;
+    bx_headerbar_entry[hb_index].xorigin = rfbOriginLeft;
     rfbOriginLeft += rfbBitmaps[bmap_id].xdim;
   } else { // BX_GRAVITY_RIGHT
     rfbOriginRight += rfbBitmaps[bmap_id].xdim;
-    rfbHeaderbarBitmaps[hb_index].xorigin = rfbOriginRight;
-    rfbHeaderbarBitmaps[hb_index].yorigin = 0;
+    bx_headerbar_entry[hb_index].xorigin = rfbOriginRight;
   }
   return hb_index;
 }
 
-// ::SHOW_HEADERBAR()
-//
-// Show (redraw) the current headerbar, which is composed of
-// currently installed bitmaps.
-
 void bx_vncsrv_gui_c::show_headerbar(void)
 {
   char *newBits, value;
-  unsigned int i, xorigin, addr;
+  unsigned int i, xorigin, addr, bmap_id;
 
-  newBits = (char *) malloc(rfbWindowX * rfbHeaderbarY);
+  newBits = new char[rfbWindowX * rfbHeaderbarY];
   memset(newBits, 0, (rfbWindowX * rfbHeaderbarY));
   DrawBitmap(0, 0, rfbWindowX, rfbHeaderbarY, newBits, headerbar_fg,
              headerbar_bg);
-  for (i = 0; i < rfbHeaderbarBitmapCount; i++) {
-    if (rfbHeaderbarBitmaps[i].alignment == BX_GRAVITY_LEFT) {
-      xorigin = rfbHeaderbarBitmaps[i].xorigin;
+  for (i = 0; i < bx_headerbar_entries; i++) {
+    if (bx_headerbar_entry[i].alignment == BX_GRAVITY_LEFT) {
+      xorigin = bx_headerbar_entry[i].xorigin;
     } else {
-      xorigin = rfbWindowX - rfbHeaderbarBitmaps[i].xorigin;
+      xorigin = rfbWindowX - bx_headerbar_entry[i].xorigin;
     }
-    DrawBitmap(xorigin, 0, rfbBitmaps[rfbHeaderbarBitmaps[i].index].xdim,
-               rfbBitmaps[rfbHeaderbarBitmaps[i].index].ydim,
-               rfbBitmaps[rfbHeaderbarBitmaps[i].index].bmap, headerbar_fg,
-               headerbar_bg);
+    bmap_id = bx_headerbar_entry[i].bmap_id;
+    DrawBitmap(xorigin, 0, rfbBitmaps[bmap_id].xdim, rfbBitmaps[bmap_id].ydim,
+               rfbBitmaps[bmap_id].bmap, headerbar_fg, headerbar_bg);
   }
-  free(newBits);
-  newBits = (char *) malloc(rfbWindowX * rfbStatusbarY / 8);
+  delete [] newBits;
+  newBits = new char[rfbWindowX * rfbStatusbarY / 8];
   memset(newBits, 0, (rfbWindowX * rfbStatusbarY / 8));
   for (i = 1; i < 12; i++) {
     addr = rfbStatusitemPos[i] / 8;
@@ -749,47 +620,28 @@ void bx_vncsrv_gui_c::show_headerbar(void)
   }
   DrawBitmap(0, rfbWindowY - rfbStatusbarY, rfbWindowX, rfbStatusbarY,
              newBits, headerbar_fg, headerbar_bg);
-  free(newBits);
+  delete [] newBits;
   for (i = 1; i <= statusitem_count; i++) {
     vncSetStatusText(i, statusitem[i - 1].text, rfbStatusitemActive[i]);
   }
 }
 
-// ::REPLACE_BITMAP()
-//
-// Replace the bitmap installed in the headerbar ID slot 'hbar_id',
-// with the one specified by 'bmap_id'.  'bmap_id' will have
-// been generated by ::create_bitmap().  The old and new bitmap
-// must be of the same size.  This allows the bitmap the user
-// sees to change, when some action occurs.  For example when
-// the user presses on the floppy icon, it then displays
-// the ejected status.
-//
-// hbar_id: headerbar slot ID
-// bmap_id: bitmap ID
-
 void bx_vncsrv_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 {
   unsigned int xorigin;
 
-  if (bmap_id == rfbHeaderbarBitmaps[hbar_id].index)
+  if (bmap_id == bx_headerbar_entry[hbar_id].bmap_id)
     return;
-  rfbHeaderbarBitmaps[hbar_id].index = bmap_id;
-  if (rfbHeaderbarBitmaps[hbar_id].alignment == BX_GRAVITY_LEFT) {
-    xorigin = rfbHeaderbarBitmaps[hbar_id].xorigin;
+  bx_headerbar_entry[hbar_id].bmap_id = bmap_id;
+  if (bx_headerbar_entry[hbar_id].alignment == BX_GRAVITY_LEFT) {
+    xorigin = bx_headerbar_entry[hbar_id].xorigin;
   } else {
-    xorigin = rfbWindowX - rfbHeaderbarBitmaps[hbar_id].xorigin;
+    xorigin = rfbWindowX - bx_headerbar_entry[hbar_id].xorigin;
   }
-  DrawBitmap(xorigin, 0, rfbBitmaps[rfbHeaderbarBitmaps[hbar_id].index].xdim,
-             rfbBitmaps[rfbHeaderbarBitmaps[hbar_id].index].ydim,
-             rfbBitmaps[rfbHeaderbarBitmaps[hbar_id].index].bmap, headerbar_fg,
-             headerbar_bg);
+  DrawBitmap(xorigin, 0, rfbBitmaps[bmap_id].xdim, rfbBitmaps[bmap_id].ydim,
+             rfbBitmaps[bmap_id].bmap, headerbar_fg, headerbar_bg);
 }
 
-// ::EXIT()
-//
-// Called before bochs terminates, to allow for a graceful
-// exit from the native GUI mechanism.
 void bx_vncsrv_gui_c::exit(void)
 {
   unsigned int i;
@@ -802,7 +654,7 @@ void bx_vncsrv_gui_c::exit(void)
   BX_FINI_MUTEX(bKeyboardInUse);
 
   for (i = 0; i < rfbBitmapCount; i++) {
-    free(rfbBitmaps[i].bmap);
+    delete [] rfbBitmaps[i].bmap;
   }
 
   BX_DEBUG(("bx_vncsrv_gui_c::exit()"));
@@ -891,134 +743,49 @@ void bx_vncsrv_gui_c::show_ips(Bit32u ips_count)
 }
 #endif
 
-// VNCSRV specific functions
-
-BX_THREAD_FUNC(vncServerThreadInit, indata)
+void bx_vncsrv_gui_c::set_display_mode(disp_mode_t newmode)
 {
-  /* this is the blocking event loop, i.e. it never returns */
-  /* 40000 are the microseconds to wait on select(), i.e. 0.04 seconds*/
-  rfbRunEventLoop(theGui->screen, 40000, FALSE);
-
-  free(theGui->screen->frameBuffer);
-  rfbScreenCleanup(theGui->screen);
-
-  rfbServerDown = true;
-
-  BX_THREAD_EXIT;
-}
-
-void vncStartThread()
-{
-  BX_THREAD_ID(threadID);
-
-  BX_THREAD_CREATE(vncServerThreadInit, NULL, threadID);
-}
-
-void DrawBitmap(int x, int y, int width, int height, char *bmap,
-        rfbPixel fgcolor, rfbPixel bgcolor)
-{
-  rfbPixel *newBits;
-
-  newBits = (rfbPixel *) malloc(width * height * sizeof(rfbPixel));
-  memset(newBits, 0, (width * height * sizeof(rfbPixel)));
-  for (int i = 0; i < (width * height) / 8; i++) {
-    newBits[i * 8 + 0] = (bmap[i] & 0x01) ? fgcolor : bgcolor;
-    newBits[i * 8 + 1] = (bmap[i] & 0x02) ? fgcolor : bgcolor;
-    newBits[i * 8 + 2] = (bmap[i] & 0x04) ? fgcolor : bgcolor;
-    newBits[i * 8 + 3] = (bmap[i] & 0x08) ? fgcolor : bgcolor;
-    newBits[i * 8 + 4] = (bmap[i] & 0x10) ? fgcolor : bgcolor;
-    newBits[i * 8 + 5] = (bmap[i] & 0x20) ? fgcolor : bgcolor;
-    newBits[i * 8 + 6] = (bmap[i] & 0x40) ? fgcolor : bgcolor;
-    newBits[i * 8 + 7] = (bmap[i] & 0x80) ? fgcolor : bgcolor;
+  // if no mode change, do nothing.
+  if (disp_mode == newmode) return;
+  // remember the display mode for next time
+  disp_mode = newmode;
+  if ((newmode == DISP_MODE_SIM) && console_running()) {
+    console_cleanup();
   }
-  UpdateScreen(newBits, x, y, width, height);
-  free(newBits);
 }
 
-void DrawChar(int x, int y, int width, int height, int fonty, char *bmap,
-        rfbPixel fgcolor, rfbPixel bgcolor, bx_bool gfxchar)
+void bx_vncsrv_gui_c::vncMouseMove(int x, int y, int z, int bmask)
 {
-  static rfbPixel newBits[9 * 32];
-  unsigned char mask;
-  int bytes = width * height;
+  static int oldx = -1;
+  static int oldy = -1;
+  int dx, dy;
 
-  for (int i = 0; i < bytes; i += width) {
-    mask = 0x80;
-    for (int j = 0; j < width; j++) {
-      if (mask > 0) {
-        newBits[i + j] = (bmap[fonty] & mask) ? fgcolor : bgcolor;
-      } else {
-        if (gfxchar) {
-          newBits[i + j] = (bmap[fonty] & 0x01) ? fgcolor : bgcolor;
-        } else {
-          newBits[i + j] = bgcolor;
-        }
+  if ((oldx == 1) && (oldy == -1)) {
+    oldx = x;
+    oldy = y;
+    return;
+  }
+  if (y > rfbHeaderbarY) {
+    if (console_running())
+      return;
+    if (rfbMouseModeAbsXY) {
+      if ((y >= rfbHeaderbarY) && (y < (int)(rfbDimensionY + rfbHeaderbarY))) {
+        dx = x * 0x7fff / rfbDimensionX;
+        dy = (y - rfbHeaderbarY) * 0x7fff / rfbDimensionY;
+        DEV_mouse_motion(dx, dy, z, bmask, 1);
       }
-      mask >>= 1;
+    } else {
+      DEV_mouse_motion(x - oldx, oldy - y, z, bmask, 0);
     }
-    fonty++;
-  }
-  UpdateScreen(newBits, x, y, width, height);
-}
-
-void UpdateScreen(rfbPixel *newBits, int x, int y, int width, int height)
-{
-  int i, x0, y0;
-  x0 = x;
-  y0 = y;
-  if ((unsigned)(x + width - 1) >= rfbWindowX) {
-    width = rfbWindowX - x + 1;
-  }
-  if ((unsigned)(y + height - 1) >= rfbWindowY) {
-    height = rfbWindowY - y + 1;
-  }
-  for (i = 0; i < height; i++) {
-    memcpy(&(theGui->screen->frameBuffer[y * rfbWindowX * sizeof(rfbPixel)
-             + x * sizeof(rfbPixel)]),
-           &newBits[i * width], width * sizeof(rfbPixel));
-    y++;
-  }
-  SendUpdate(x0, y0, width, height);
-}
-
-void SendUpdate(int x, int y, int width, int height)
-{
-  rfbMarkRectAsModified(theGui->screen, x, y, x + width, y + height);
-}
-
-void vncSetStatusText(int element, const char *text, bx_bool active, bx_bool w)
-{
-  char *newBits;
-  unsigned xleft, xsize, i, len;
-
-  rfbStatusitemActive[element] = active;
-  xleft = rfbStatusitemPos[element] + 2;
-  xsize = rfbStatusitemPos[element + 1] - xleft - 1;
-  newBits = (char *) malloc(((xsize / 8) + 1) * (rfbStatusbarY - 2));
-  memset(newBits, 0, ((xsize / 8) + 1) * (rfbStatusbarY - 2));
-  for (i = 0; i < (rfbStatusbarY - 2); i++) {
-    newBits[((xsize / 8) + 1) * i] = 0;
-  }
-
-  rfbPixel fgcolor = active ? headerbar_fg : status_gray_text;
-  rfbPixel bgcolor = 0;
-  if (element > 0) {
-    bgcolor = active ? (w ? status_led_red : status_led_green) : headerbar_bg;
+    oldx = x;
+    oldy = y;
   } else {
-    bgcolor = headerbar_bg;
+    if (bmask == 1) {
+      rfbKeyboardEvents = 0;
+      BX_UNLOCK(bKeyboardInUse);
+      headerbar_click(x);
+    }
   }
-  DrawBitmap(xleft, rfbWindowY - rfbStatusbarY + 1, xsize, rfbStatusbarY - 2,
-             newBits, fgcolor, bgcolor);
-
-  free(newBits);
-  len = ((element > 0) && (strlen(text) > 4)) ? 4 : strlen(text);
-  for (i = 0; i < len; i++) {
-    DrawChar(xleft + i * 8 + 2, rfbWindowY - rfbStatusbarY + 5, 8, 8, 0,
-             (char *) &sdl_font8x8[(unsigned) text[i]][0], fgcolor, bgcolor, 0);
-  }
-
-  rfbMarkRectAsModified(theGui->screen, xleft, rfbWindowY - rfbStatusbarY + 1,
-                        xleft + xsize, rfbWindowY - 2);
 }
 
 // function to convert key names into rfb key values.
@@ -1085,10 +852,17 @@ static Bit32u rfb_ascii_to_key_event[0x5f] = {
         BX_KEY_RIGHT_BRACKET, BX_KEY_GRAVE
 };
 
-void vncKeyPressed(Bit32u key, int press_release)
+void bx_vncsrv_gui_c::vncKeyPressed(Bit32u key, int press_release)
 {
   Bit32u key_event;
 
+  if (console_running() && press_release) {
+    if (((key >= XK_space) && (key <= XK_asciitilde)) ||
+        (key == XK_Return) || (key == XK_BackSpace)) {
+      console_key_enq((Bit8u)(key & 0xff));
+    }
+    return;
+  }
   if (!SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
     if ((key >= XK_space) && (key <= XK_asciitilde)) {
       key_event = rfb_ascii_to_key_event[key - XK_space];
@@ -1339,44 +1113,135 @@ void vncKeyPressed(Bit32u key, int press_release)
   DEV_kbd_gen_scancode(key_event);
 }
 
-void vncMouseMove(int x, int y, int z, int bmask)
-{
-  static int oldx = -1;
-  static int oldy = -1;
-  int dx, dy, xorigin;
+// VNCSRV specific functions
 
-  if ((oldx == 1) && (oldy == -1)) {
-    oldx = x;
-    oldy = y;
-    return;
+BX_THREAD_FUNC(vncServerThreadInit, indata)
+{
+  /* this is the blocking event loop, i.e. it never returns */
+  /* 40000 are the microseconds to wait on select(), i.e. 0.04 seconds*/
+  rfbRunEventLoop(theGui->screen, 40000, FALSE);
+
+  delete [] theGui->screen->frameBuffer;
+  rfbScreenCleanup(theGui->screen);
+
+  rfbServerDown = true;
+
+  BX_THREAD_EXIT;
+}
+
+void vncStartThread()
+{
+  BX_THREAD_ID(threadID);
+
+  BX_THREAD_CREATE(vncServerThreadInit, NULL, threadID);
+}
+
+void DrawBitmap(int x, int y, int width, int height, char *bmap,
+        rfbPixel fgcolor, rfbPixel bgcolor)
+{
+  rfbPixel *newBits;
+
+  newBits = new rfbPixel[width * height];
+  memset(newBits, 0, (width * height * sizeof(rfbPixel)));
+  for (int i = 0; i < (width * height) / 8; i++) {
+    newBits[i * 8 + 0] = (bmap[i] & 0x01) ? fgcolor : bgcolor;
+    newBits[i * 8 + 1] = (bmap[i] & 0x02) ? fgcolor : bgcolor;
+    newBits[i * 8 + 2] = (bmap[i] & 0x04) ? fgcolor : bgcolor;
+    newBits[i * 8 + 3] = (bmap[i] & 0x08) ? fgcolor : bgcolor;
+    newBits[i * 8 + 4] = (bmap[i] & 0x10) ? fgcolor : bgcolor;
+    newBits[i * 8 + 5] = (bmap[i] & 0x20) ? fgcolor : bgcolor;
+    newBits[i * 8 + 6] = (bmap[i] & 0x40) ? fgcolor : bgcolor;
+    newBits[i * 8 + 7] = (bmap[i] & 0x80) ? fgcolor : bgcolor;
   }
-  if (y > rfbHeaderbarY) {
-    if (rfbMouseModeAbsXY) {
-      if ((y >= rfbHeaderbarY) && (y < (int)(rfbDimensionY + rfbHeaderbarY))) {
-        dx = x * 0x7fff / rfbDimensionX;
-        dy = (y - rfbHeaderbarY) * 0x7fff / rfbDimensionY;
-        DEV_mouse_motion(dx, dy, z, bmask, 1);
-      }
-    } else {
-      DEV_mouse_motion(x - oldx, oldy - y, z, bmask, 0);
-    }
-    oldx = x;
-    oldy = y;
-  } else {
-    if (bmask == 1) {
-      for (unsigned i = 0; i < rfbHeaderbarBitmapCount; i++) {
-        if (rfbHeaderbarBitmaps[i].alignment == BX_GRAVITY_LEFT)
-          xorigin = rfbHeaderbarBitmaps[i].xorigin;
-        else
-          xorigin = rfbWindowX - rfbHeaderbarBitmaps[i].xorigin;
-        if ((x >= xorigin) &&
-            (x < (xorigin + int(rfbBitmaps[rfbHeaderbarBitmaps[i].index].xdim)))) {
-          rfbHeaderbarBitmaps[i].f();
-          return;
+  UpdateScreen(newBits, x, y, width, height);
+  delete [] newBits;
+}
+
+void DrawChar(int x, int y, int width, int height, int fonty, char *bmap,
+        rfbPixel fgcolor, rfbPixel bgcolor, bx_bool gfxchar)
+{
+  static rfbPixel newBits[18 * 32];
+  unsigned char mask;
+  int bytes = width * height;
+  bx_bool dwidth = (width > 9);
+
+  for (int i = 0; i < bytes; i += width) {
+    mask = 0x80;
+    for (int j = 0; j < width; j++) {
+      if (mask > 0) {
+        newBits[i + j] = (bmap[fonty] & mask) ? fgcolor : bgcolor;
+      } else {
+        if (gfxchar) {
+          newBits[i + j] = (bmap[fonty] & 0x01) ? fgcolor : bgcolor;
+        } else {
+          newBits[i + j] = bgcolor;
         }
       }
+      if (!dwidth || (j & 1)) mask >>= 1;
     }
+    fonty++;
   }
+  UpdateScreen(newBits, x, y, width, height);
+}
+
+void UpdateScreen(rfbPixel *newBits, int x, int y, int width, int height)
+{
+  int i, x0, y0;
+  x0 = x;
+  y0 = y;
+  if ((unsigned)(x + width - 1) >= rfbWindowX) {
+    width = rfbWindowX - x + 1;
+  }
+  if ((unsigned)(y + height - 1) >= rfbWindowY) {
+    height = rfbWindowY - y + 1;
+  }
+  for (i = 0; i < height; i++) {
+    memcpy(&(theGui->screen->frameBuffer[y * rfbWindowX * sizeof(rfbPixel)
+             + x * sizeof(rfbPixel)]),
+           &newBits[i * width], width * sizeof(rfbPixel));
+    y++;
+  }
+  SendUpdate(x0, y0, width, height);
+}
+
+void SendUpdate(int x, int y, int width, int height)
+{
+  rfbMarkRectAsModified(theGui->screen, x, y, x + width, y + height);
+}
+
+void vncSetStatusText(int element, const char *text, bx_bool active, bx_bool w)
+{
+  char *newBits;
+  unsigned xleft, xsize, i, len;
+
+  rfbStatusitemActive[element] = active;
+  xleft = rfbStatusitemPos[element] + 2;
+  xsize = rfbStatusitemPos[element + 1] - xleft - 1;
+  newBits = new char[((xsize / 8) + 1) * (rfbStatusbarY - 2)];
+  memset(newBits, 0, ((xsize / 8) + 1) * (rfbStatusbarY - 2));
+  for (i = 0; i < (rfbStatusbarY - 2); i++) {
+    newBits[((xsize / 8) + 1) * i] = 0;
+  }
+
+  rfbPixel fgcolor = active ? headerbar_fg : status_gray_text;
+  rfbPixel bgcolor = 0;
+  if (element > 0) {
+    bgcolor = active ? (w ? status_led_red : status_led_green) : headerbar_bg;
+  } else {
+    bgcolor = headerbar_bg;
+  }
+  DrawBitmap(xleft, rfbWindowY - rfbStatusbarY + 1, xsize, rfbStatusbarY - 2,
+             newBits, fgcolor, bgcolor);
+
+  delete [] newBits;
+  len = ((element > 0) && (strlen(text) > 4)) ? 4 : strlen(text);
+  for (i = 0; i < len; i++) {
+    DrawChar(xleft + i * 8 + 2, rfbWindowY - rfbStatusbarY + 5, 8, 8, 0,
+             (char *) &sdl_font8x8[(unsigned) text[i]][0], fgcolor, bgcolor, 0);
+  }
+
+  rfbMarkRectAsModified(theGui->screen, xleft, rfbWindowY - rfbStatusbarY + 1,
+                        xleft + xsize, rfbWindowY - 2);
 }
 
 void newframebuffer(rfbScreenInfoPtr screen, int width, int height)
@@ -1384,10 +1249,10 @@ void newframebuffer(rfbScreenInfoPtr screen, int width, int height)
     unsigned char *oldfb, *newfb;
 
     oldfb = (unsigned char*) screen->frameBuffer;
-    newfb = (unsigned char*) malloc(width * height * sizeof(rfbPixel));
+    newfb = new unsigned char[width * height * sizeof(rfbPixel)];
     rfbNewFramebuffer(screen, (char*) newfb, width, height, 8, 3,
             sizeof(rfbPixel));
-    free(oldfb);
+    delete [] oldfb;
 
     /*** FIXME: Re-install cursor. ***/
 }
