@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: win32.cc 12588 2014-12-30 16:31:17Z vruppert $
+// $Id: win32.cc 13144 2017-03-23 19:09:37Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2014  The Bochs Project
+//  Copyright (C) 2002-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -75,7 +75,7 @@ IMPLEMENT_GUI_PLUGIN_CODE(win32)
 // Keyboard/mouse stuff
 #define SCANCODE_BUFSIZE    20
 #define MOUSE_PRESSED       0x20000000
-#define HEADERBAR_CLICKED   0x08000000
+#define TOOLBAR_CLICKED     0x08000000
 #define MOUSE_MOTION        0x22000000
 #define FOCUS_CHANGED       0x44000000
 #define BX_SYSKEY           (KF_UP|KF_REPEAT|KF_ALTDOWN)
@@ -143,9 +143,9 @@ static struct {
   unsigned bmap_id;
   void (*f)(void);
   const char *tooltip;
-} bx_headerbar_entry[BX_MAX_HEADERBAR_ENTRIES];
+} win32_toolbar_entry[BX_MAX_HEADERBAR_ENTRIES];
 
-static int bx_headerbar_entries;
+static int win32_toolbar_entries;
 static unsigned bx_hb_separator;
 
 // Status Bar stuff
@@ -203,10 +203,9 @@ DWORD WINAPI UIThread(PVOID);
 void SetStatusText(unsigned Num, const char *Text, bx_bool active, bx_bool w=0);
 void terminateEmul(int);
 void create_vga_font(void);
-static unsigned char reverse_bitorder(unsigned char);
 void DrawBitmap(HDC, HBITMAP, int, int, int, int, int, int, DWORD, unsigned char);
 void updateUpdated(int,int,int,int);
-static void headerbar_click(int x);
+static void win32_toolbar_click(int x);
 
 
 Bit32u win32_to_bx_key[2][0x100] =
@@ -540,27 +539,25 @@ void terminateEmul(int reason)
   if (MemoryDC) DeleteDC (MemoryDC);
   if (MemoryBitmap) DeleteObject (MemoryBitmap);
 
-  if (bitmap_info) delete[] (char*)bitmap_info;
+  delete[] (char*)bitmap_info;
 
   for (unsigned b=0; b<bx_bitmap_entries; b++)
     if (bx_bitmaps[b].bmap) DeleteObject(bx_bitmaps[b].bmap);
   for (unsigned c=0; c<256; c++)
     if (vgafont[c]) DeleteObject(vgafont[c]);
 
-  LOG_THIS setonoff(LOGLEV_PANIC, ACT_FATAL);
-
   switch (reason) {
   case EXIT_GUI_SHUTDOWN:
-    BX_PANIC(("Window closed, exiting!"));
+    BX_FATAL(("Window closed, exiting!"));
     break;
   case EXIT_GMH_FAILURE:
-    BX_PANIC(("GetModuleHandle failure!"));
+    BX_FATAL(("GetModuleHandle failure!"));
     break;
   case EXIT_FONT_BITMAP_ERROR:
-    BX_PANIC(("Font bitmap creation failure!"));
+    BX_FATAL(("Font bitmap creation failure!"));
     break;
   case EXIT_HEADER_BITMAP_ERROR:
-    BX_PANIC(("Header bitmap creation failure!"));
+    BX_FATAL(("Header bitmap creation failure!"));
     break;
   case EXIT_NORMAL:
     break;
@@ -568,6 +565,8 @@ void terminateEmul(int reason)
 }
 
 
+// WIN32 implementation of the bx_gui_c methods (see nogui.cc for details)
+
 bx_win32_gui_c::bx_win32_gui_c()
 {
   // prepare for possible fullscreen mode
@@ -578,19 +577,6 @@ bx_win32_gui_c::bx_win32_gui_c()
 }
 
 
-// ::SPECIFIC_INIT()
-//
-// Called from gui.cc, once upon program startup, to allow for the
-// specific GUI code (X11, Win32, ...) to be initialized.
-//
-// argc, argv: used to pass display library specific options to the init code
-//     (X11 options, Win32 options,...)
-//
-// headerbar_y:  A headerbar (toolbar) is display on the top of the
-//     VGA window, showing floppy status, and other information.  It
-//     always assumes the width of the current VGA mode width, but
-//     it's height is defined by this parameter.
-
 void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 {
   int i;
@@ -615,7 +601,7 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
   win32_max_yres = this->max_yres;
 
   bx_bitmap_entries = 0;
-  bx_headerbar_entries = 0;
+  win32_toolbar_entries = 0;
   bx_hb_separator = 0;
   mouseCaptureMode = FALSE;
   mouseCaptureNew = FALSE;
@@ -628,8 +614,6 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
       if (!strcmp(argv[i], "nokeyrepeat")) {
         BX_INFO(("disabled host keyboard repeat"));
         win32_nokeyrepeat = 1;
-      } else if (!strcmp(argv[i], "legacyF12")) {
-        BX_PANIC(("The option 'legacyF12' is now deprecated - use 'mouse: toggle=f12' instead"));
 #if BX_DEBUGGER && BX_DEBUGGER_GUI
       } else if (!strcmp(argv[i], "gui_debug")) {
         if (gui_ci) {
@@ -677,8 +661,7 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
   for(unsigned c=0; c<256; c++) vgafont[c] = NULL;
   create_vga_font();
 
-  bitmap_info=(BITMAPINFO*)new char[sizeof(BITMAPINFOHEADER)+
-    259*sizeof(RGBQUAD)]; // 256 + 3 entries for 16 bpp mode
+  bitmap_info=(BITMAPINFO*)new char[sizeof(BITMAPINFOHEADER)+259*sizeof(RGBQUAD)]; // 256 + 3 entries for 16 bpp mode
   bitmap_info->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
   bitmap_info->bmiHeader.biWidth=x_tilesize;
   // Height is negative for top-down bitmap
@@ -1021,7 +1004,7 @@ LRESULT CALLBACK mainWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
   case WM_COMMAND:
     if (LOWORD(wParam) >= 101) {
       EnterCriticalSection(&stInfo.keyCS);
-      enq_key_event(LOWORD(wParam)-101, HEADERBAR_CLICKED);
+      enq_key_event(LOWORD(wParam)-101, TOOLBAR_CLICKED);
       LeaveCriticalSection(&stInfo.keyCS);
     }
     break;
@@ -1090,8 +1073,8 @@ LRESULT CALLBACK mainWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       lpttt = (LPTOOLTIPTEXT)lParam;
       idTT = (int)wParam;
       hbar_id = idTT - 101;
-      if (SendMessage(hwndTB, TB_GETSTATE, idTT, 0) && bx_headerbar_entry[hbar_id].tooltip != NULL) {
-        lstrcpy(lpttt->szText, bx_headerbar_entry[hbar_id].tooltip);
+      if (SendMessage(hwndTB, TB_GETSTATE, idTT, 0) && win32_toolbar_entry[hbar_id].tooltip != NULL) {
+        lstrcpy(lpttt->szText, win32_toolbar_entry[hbar_id].tooltip);
       }
     }
     return FALSE;
@@ -1479,12 +1462,6 @@ QueueEvent* deq_key_event(void)
 }
 
 
-// ::HANDLE_EVENTS()
-//
-// Called periodically (vga_update_interval in .bochsrc) so the
-// the gui code can poll for keyboard, mouse, and other
-// relevant events.
-
 void bx_win32_gui_c::handle_events(void)
 {
   Bit32u key;
@@ -1519,8 +1496,8 @@ void bx_win32_gui_c::handle_events(void)
     else if (key & MOUSE_PRESSED) {
       DEV_mouse_motion(0, 0, 0, LOWORD(key), 0);
     }
-    else if (key & HEADERBAR_CLICKED) {
-      headerbar_click(LOWORD(key));
+    else if (key & TOOLBAR_CLICKED) {
+      win32_toolbar_click(LOWORD(key));
     }
     else {
       key_event = win32_to_bx_key[(key & 0x100) ? 1 : 0][key & 0xff];
@@ -1538,11 +1515,6 @@ void bx_win32_gui_c::handle_events(void)
 }
 
 
-// ::FLUSH()
-//
-// Called periodically, requesting that the gui code flush all pending
-// screen update requests.
-
 void bx_win32_gui_c::flush(void)
 {
   EnterCriticalSection(&stInfo.drawCS);
@@ -1556,11 +1528,6 @@ void bx_win32_gui_c::flush(void)
   LeaveCriticalSection(&stInfo.drawCS);
 }
 
-// ::CLEAR_SCREEN()
-//
-// Called to request that the VGA region is cleared.  Don't
-// clear the area that defines the headerbar.
-
 void bx_win32_gui_c::clear_screen(void)
 {
   HGDIOBJ oldObj;
@@ -1579,25 +1546,6 @@ void bx_win32_gui_c::clear_screen(void)
 }
 
 
-// ::TEXT_UPDATE()
-//
-// Called in a VGA text mode, to update the screen with
-// new content.
-//
-// old_text: array of character/attributes making up the contents
-//           of the screen from the last call.  See below
-// new_text: array of character/attributes making up the current
-//           contents, which should now be displayed.  See below
-//
-// format of old_text & new_text: each is tm_info->line_offset*text_rows
-//     bytes long. Each character consists of 2 bytes.  The first by is
-//     the character value, the second is the attribute byte.
-//
-// cursor_x: new x location of cursor
-// cursor_y: new y location of cursor
-// tm_info:  this structure contains information for additional
-//           features in text mode (cursor shape, line offset,...)
-
 void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
                                  unsigned long cursor_x, unsigned long cursor_y,
                                  bx_vga_tminfo_t *tm_info)
@@ -1843,13 +1791,6 @@ int bx_win32_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
 }
 
 
-// ::PALETTE_CHANGE()
-//
-// Allocate a color in the native GUI, for this color, and put
-// it in the colormap location 'index'.
-// returns: 0=no screen update needed (color map change has direct effect)
-//          1=screen updated needed (redraw using current colormap)
-
 bx_bool bx_win32_gui_c::palette_change(Bit8u index, Bit8u red, Bit8u green,
                                        Bit8u blue) {
   if ((current_bpp == 16) && (index < 3)) {
@@ -1866,21 +1807,6 @@ bx_bool bx_win32_gui_c::palette_change(Bit8u index, Bit8u red, Bit8u green,
 }
 
 
-// ::GRAPHICS_TILE_UPDATE()
-//
-// Called to request that a tile of graphics be drawn to the
-// screen, since info in this region has changed.
-//
-// tile: array of 8bit values representing a block of pixels with
-//       dimension equal to the 'x_tilesize' & 'y_tilesize' members.
-//       Each value specifies an index into the
-//       array of colors you allocated for ::palette_change()
-// x0: x origin of tile
-// y0: y origin of tile
-//
-// note: origin of tile and of window based on (0,0) being in the upper
-//       left of the window.
-
 void bx_win32_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 {
   HDC hdc;
@@ -1904,21 +1830,14 @@ void bx_win32_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 
 
 
-// ::DIMENSION_UPDATE()
-//
-// Called when the VGA mode changes it's X,Y dimensions.
-// Resize the window to this size, but you need to add on
-// the height of the headerbar to the Y value.
-//
-// x: new VGA x size
-// y: new VGA y size (add headerbar_y parameter from ::specific_init().
-// fheight: new VGA character height in text mode
-// fwidth : new VGA character width in text mode
-// bpp : bits per pixel in graphics mode
-
 void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, unsigned fwidth, unsigned bpp)
 {
   guest_textmode = (fheight > 0);
+  if (guest_textmode && (fwidth > 9)) {
+    // use existing stretching feature for text mode CO40
+    x >>= 1;
+    fwidth >>= 1;
+  }
   xChar = fwidth;
   yChar = fheight;
   guest_xres = x;
@@ -1984,19 +1903,7 @@ void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
 }
 
 
-// ::CREATE_BITMAP()
-//
-// Create a monochrome bitmap of size 'xdim' by 'ydim', which will
-// be drawn in the headerbar.  Return an integer ID to the bitmap,
-// with which the bitmap can be referenced later.
-//
-// bmap: packed 8 pixels-per-byte bitmap.  The pixel order is:
-//       bit0 is the left most pixel, bit7 is the right most pixel.
-// xdim: x dimension of bitmap
-// ydim: y dimension of bitmap
-
-unsigned bx_win32_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim,
-				 unsigned ydim)
+unsigned bx_win32_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim, unsigned ydim)
 {
   unsigned char *data;
   TBADDBITMAP tbab;
@@ -2027,31 +1934,15 @@ unsigned bx_win32_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim,
 }
 
 
-// ::HEADERBAR_BITMAP()
-//
-// Called to install a bitmap in the bochs headerbar (toolbar).
-//
-// bmap_id: will correspond to an ID returned from
-//     ::create_bitmap().  'alignment' is either BX_GRAVITY_LEFT
-//     or BX_GRAVITY_RIGHT, meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// alignment: is either BX_GRAVITY_LEFT or BX_GRAVITY_RIGHT,
-//     meaning install the bitmap in the next
-//     available leftmost or rightmost space.
-// f: a 'C' function pointer to callback when the mouse is clicked in
-//     the boundaries of this bitmap.
-
-unsigned bx_win32_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment,
-				    void (*f)(void))
+unsigned bx_win32_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment, void (*f)(void))
 {
   unsigned hb_index;
   TBBUTTON tbb[1];
 
-  if ((bx_headerbar_entries+1) > BX_MAX_HEADERBAR_ENTRIES)
+  if ((win32_toolbar_entries+1) > BX_MAX_HEADERBAR_ENTRIES)
     terminateEmul(EXIT_HEADER_BITMAP_ERROR);
 
-  bx_headerbar_entries++;
-  hb_index = bx_headerbar_entries - 1;
+  hb_index = win32_toolbar_entries++;
 
   memset(tbb,0,sizeof(tbb));
   if (bx_hb_separator==0) {
@@ -2072,19 +1963,14 @@ unsigned bx_win32_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment,
     SendMessage(hwndTB, TB_INSERTBUTTON, bx_hb_separator+1, (LPARAM)(LPTBBUTTON)&tbb);
   }
 
-  bx_headerbar_entry[hb_index].bmap_id = bmap_id;
-  bx_headerbar_entry[hb_index].f = f;
-  bx_headerbar_entry[hb_index].tooltip = NULL;
+  win32_toolbar_entry[hb_index].bmap_id = bmap_id;
+  win32_toolbar_entry[hb_index].f = f;
+  win32_toolbar_entry[hb_index].tooltip = NULL;
 
   return(hb_index);
 }
 
 
-// ::SHOW_HEADERBAR()
-//
-// Show (redraw) the current headerbar, which is composed of
-// currently installed bitmaps.
-
 void bx_win32_gui_c::show_headerbar(void)
 {
   if (!IsWindowVisible(hwndTB)) {
@@ -2096,23 +1982,10 @@ void bx_win32_gui_c::show_headerbar(void)
 }
 
 
-// ::REPLACE_BITMAP()
-//
-// Replace the bitmap installed in the headerbar ID slot 'hbar_id',
-// with the one specified by 'bmap_id'.  'bmap_id' will have
-// been generated by ::create_bitmap().  The old and new bitmap
-// must be of the same size.  This allows the bitmap the user
-// sees to change, when some action occurs.  For example when
-// the user presses on the floppy icon, it then displays
-// the ejected status.
-//
-// hbar_id: headerbar slot ID
-// bmap_id: bitmap ID
-
 void bx_win32_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 {
-  if (bmap_id != bx_headerbar_entry[hbar_id].bmap_id) {
-    bx_headerbar_entry[hbar_id].bmap_id = bmap_id;
+  if (bmap_id != win32_toolbar_entry[hbar_id].bmap_id) {
+    win32_toolbar_entry[hbar_id].bmap_id = bmap_id;
     bx_bool is_visible = IsWindowVisible(hwndTB);
     if (is_visible) {
       ShowWindow(hwndTB, SW_HIDE);
@@ -2127,10 +2000,6 @@ void bx_win32_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 }
 
 
-// ::EXIT()
-//
-// Called before bochs terminates, to allow for a graceful
-// exit from the native GUI mechanism.
 void bx_win32_gui_c::exit(void)
 {
 #if BX_DEBUGGER && BX_DEBUGGER_GUI
@@ -2165,19 +2034,6 @@ void create_vga_font(void)
 }
 
 
-unsigned char reverse_bitorder(unsigned char b)
-{
-  unsigned char ret=0;
-
-  for (unsigned i=0; i<8; i++) {
-    ret |= (b & 0x01) << (7-i);
-    b >>= 1;
-  }
-
-  return(ret);
-}
-
-
 COLORREF GetColorRef(unsigned char attr)
 {
   Bit8u pal_idx = text_pal_idx[attr];
@@ -2248,10 +2104,10 @@ void updateUpdated(int x1, int y1, int x2, int y2)
 }
 
 
-void headerbar_click(int x)
+void win32_toolbar_click(int x)
 {
-  if (x < bx_headerbar_entries) {
-    bx_headerbar_entry[x].f();
+  if (x < win32_toolbar_entries) {
+    win32_toolbar_entry[x].f();
   }
 }
 
@@ -2278,7 +2134,7 @@ void bx_win32_gui_c::get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp)
 
 void bx_win32_gui_c::set_tooltip(unsigned hbar_id, const char *tip)
 {
-  bx_headerbar_entry[hbar_id].tooltip = tip;
+  win32_toolbar_entry[hbar_id].tooltip = tip;
 }
 
 void bx_win32_gui_c::set_mouse_mode_absxy(bx_bool mode)
