@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: win32dialog.cc 11629 2013-02-14 21:06:20Z vruppert $
+// $Id: win32dialog.cc 13030 2017-01-10 21:40:05Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2003-2013  The Bochs Project
+//  Copyright (C) 2003-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,25 +20,15 @@
 
 #include "win32dialog.h"
 
-#if BX_USE_TEXTCONFIG && defined(WIN32) && (BX_WITH_WIN32 || BX_WITH_SDL)
+#if BX_USE_WIN32CONFIG
 
 #include "bochs.h"
 #include "param_names.h"
 #include "win32res.h"
 #include "win32paramdlg.h"
+#include "textconfig.h"
 
-const char log_choices[5][16] = {"ignore", "log", "ask user", "end simulation", "no change"};
-
-char *backslashes(char *s)
-{
-  if (s != NULL) {
-    while (*s != 0) {
-       if (*s == '/') *s = '\\';
-       s++;
-    }
-  }
-  return s;
-}
+const char log_choices[N_ACT+1][16] = {"ignore", "log", "warn user", "ask user", "end simulation", "no change"};
 
 HWND GetBochsWindow()
 {
@@ -51,32 +41,6 @@ HWND GetBochsWindow()
   return hwnd;
 }
 
-BOOL CreateImage(HWND hDlg, int sectors, const char *filename)
-{
-  if (sectors < 1) {
-    MessageBox(hDlg, "The disk size is invalid.", "Invalid size", MB_ICONERROR);
-    return FALSE;
-  }
-  if (lstrlen(filename) < 1) {
-    MessageBox(hDlg, "You must type a file name for the new disk image.", "Bad filename", MB_ICONERROR);
-    return FALSE;
-  }
-  int ret = SIM->create_disk_image (filename, sectors, 0);
-  if (ret == -1) {  // already exists
-    int answer = MessageBox(hDlg, "File exists.  Do you want to overwrite it?",
-                            "File exists", MB_YESNO);
-    if (answer == IDYES)
-      ret = SIM->create_disk_image (filename, sectors, 1);
-    else
-      return FALSE;
-  }
-  if (ret == -2) {
-    MessageBox(hDlg, "I could not create the disk image. Check for permission problems or available disk space.", "Failed", MB_ICONERROR);
-    return FALSE;
-  }
-  return TRUE;
-}
-
 int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
 {
   char path[MAX_PATH];
@@ -134,12 +98,16 @@ static BOOL CALLBACK LogAskProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lPara
       SetWindowText(GetDlgItem(hDlg, IDASKMSG), event->u.logmsg.msg);
       SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue");
       SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue and don't ask again");
-      SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Kill simulation");
-      SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Abort (dump core)");
+      if (event->u.logmsg.mode == BX_LOG_DLG_ASK) {
+        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Kill simulation");
+        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Abort (dump core)");
 #if BX_DEBUGGER
-      SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue and return to debugger");
+        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue and return to debugger");
 #endif
-      SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_SETCURSEL, 2, 0);
+        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_SETCURSEL, 2, 0);
+      } else {
+        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_SETCURSEL, 0, 0);
+      }
       SetFocus(GetDlgItem(hDlg, IDASKLIST));
       return FALSE;
     case WM_CLOSE:
@@ -193,107 +161,6 @@ static BOOL CALLBACK StringParamProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
   return FALSE;
 }
 
-static BOOL CALLBACK FloppyDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
-{
-  static bx_param_filename_c *param;
-  static bx_param_bool_c *readonly;
-  static bx_param_enum_c *devtype, *status;
-  static bx_param_enum_c *mediatype;
-  bx_list_c *list;
-  char mesg[MAX_PATH];
-  char path[MAX_PATH];
-  const char *title;
-  int i, cap;
-
-  switch (msg) {
-    case WM_INITDIALOG:
-      param = (bx_param_filename_c *)lParam;
-      list = (bx_list_c *)param->get_parent();
-      status = SIM->get_param_enum("status", list);
-      readonly = SIM->get_param_bool("readonly", list);
-      devtype = SIM->get_param_enum("devtype", list);
-      mediatype = SIM->get_param_enum("type", list);
-      cap = devtype->get() - (int)devtype->get_min();
-      SetWindowText(GetDlgItem(hDlg, IDDEVTYPE), floppy_devtype_names[cap]);
-      i = 0;
-      while (floppy_type_names[i] != NULL) {
-        SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_ADDSTRING, 0, (LPARAM)floppy_type_names[i]);
-        SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_SETITEMDATA, i, (LPARAM)(mediatype->get_min() + i));
-        i++;
-      }
-      cap = mediatype->get() - (int)mediatype->get_min();
-      SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_SETCURSEL, cap, 0);
-      if (status->get() == BX_INSERTED) {
-        SendMessage(GetDlgItem(hDlg, IDSTATUS), BM_SETCHECK, BST_CHECKED, 0);
-      }
-      if (readonly->get()) {
-        SendMessage(GetDlgItem(hDlg, IDREADONLY), BM_SETCHECK, BST_CHECKED, 0);
-      }
-      lstrcpy(path, param->getptr());
-      title = param->get_label();
-      if (!title) title = param->get_name();
-      SetWindowText(hDlg, title);
-      if (lstrlen(path) && lstrcmp(path, "none")) {
-        SetWindowText(GetDlgItem(hDlg, IDPATH), path);
-      }
-      return TRUE;
-    case WM_CLOSE:
-      EndDialog(hDlg, -1);
-      return TRUE;
-    case WM_COMMAND:
-      switch (LOWORD(wParam)) {
-        case IDBROWSE:
-          GetDlgItemText(hDlg, IDPATH, path, MAX_PATH);
-          if (AskFilename(hDlg, param, path) > 0) {
-            SetWindowText(GetDlgItem(hDlg, IDPATH), path);
-            SendMessage(GetDlgItem(hDlg, IDSTATUS), BM_SETCHECK, BST_CHECKED, 0);
-            SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_SELECTSTRING, (WPARAM)-1, (LPARAM)"auto");
-            EnableWindow(GetDlgItem(hDlg, IDCREATE), FALSE);
-          }
-          return TRUE;
-        case IDOK:
-          status->set(BX_EJECTED);
-          if (SendMessage(GetDlgItem(hDlg, IDSTATUS), BM_GETCHECK, 0, 0) == BST_CHECKED) {
-            GetDlgItemText(hDlg, IDPATH, path, MAX_PATH);
-            if (lstrlen(path) == 0) {
-              lstrcpy(path, "none");
-            }
-          } else {
-            lstrcpy(path, "none");
-          }
-          readonly->set(SendMessage(GetDlgItem(hDlg, IDREADONLY), BM_GETCHECK, 0, 0) == BST_CHECKED);
-          param->set(path);
-          i = SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_GETCURSEL, 0, 0);
-          cap = SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_GETITEMDATA, i, 0);
-          mediatype->set(cap);
-          if (lstrcmp(path, "none")) {
-            status->set(BX_INSERTED);
-          }
-          EndDialog(hDlg, 1);
-          return TRUE;
-        case IDCANCEL:
-          EndDialog(hDlg, -1);
-          return TRUE;
-        case IDMEDIATYPE:
-          if (HIWORD(wParam) == CBN_SELCHANGE) {
-            i = SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_GETCURSEL, 0, 0);
-            EnableWindow(GetDlgItem(hDlg, IDCREATE), (floppy_type_n_sectors[i] > 0));
-          }
-          break;
-        case IDCREATE:
-          GetDlgItemText(hDlg, IDPATH, path, MAX_PATH);
-          backslashes(path);
-          i = SendMessage(GetDlgItem(hDlg, IDMEDIATYPE), CB_GETCURSEL, 0, 0);
-          if (CreateImage(hDlg, floppy_type_n_sectors[i], path)) {
-            wsprintf(mesg, "Created a %s disk image called %s", floppy_type_names[i], path);
-            MessageBox(hDlg, mesg, "Image created", MB_OK);
-          }
-          return TRUE;
-      }
-  }
-  return FALSE;
-}
-
 void SetStandardLogOptions(HWND hDlg)
 {
   int level, idx;
@@ -725,25 +592,19 @@ static BOOL CALLBACK MainMenuDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM
 
 void LogAskDialog(BxEvent *event)
 {
-  event->retcode = DialogBoxParam(NULL, MAKEINTRESOURCE(ASK_DLG), GetBochsWindow(),
+  event->retcode = (Bit32s) DialogBoxParam(NULL, MAKEINTRESOURCE(ASK_DLG), GetBochsWindow(),
                                   (DLGPROC)LogAskProc, (LPARAM)event);
 }
 
 int AskString(bx_param_string_c *param)
 {
-  return DialogBoxParam(NULL, MAKEINTRESOURCE(STRING_DLG), GetBochsWindow(),
+  return (int) DialogBoxParam(NULL, MAKEINTRESOURCE(STRING_DLG), GetBochsWindow(),
                         (DLGPROC)StringParamProc, (LPARAM)param);
 }
 
-int FloppyDialog(bx_param_filename_c *param)
-{
-  return DialogBoxParam(NULL, MAKEINTRESOURCE(FLOPPY_DLG), GetBochsWindow(),
-                        (DLGPROC)FloppyDlgProc, (LPARAM)param);
-}
-
 int MainMenuDialog(HWND hwnd, bx_bool runtime)
 {
-  return DialogBoxParam(NULL, MAKEINTRESOURCE(MAINMENU_DLG), hwnd,
+  return (int) DialogBoxParam(NULL, MAKEINTRESOURCE(MAINMENU_DLG), hwnd,
                         (DLGPROC)MainMenuDlgProc, (LPARAM)runtime);
 }
 
@@ -757,7 +618,7 @@ BxEvent* win32_notify_callback(void *unused, BxEvent *event)
   event->retcode = -1;
   switch (event->type)
   {
-    case BX_SYNC_EVT_LOG_ASK:
+    case BX_SYNC_EVT_LOG_DLG:
       LogAskDialog(event);
       return event;
     case BX_SYNC_EVT_ASK_PARAM:
@@ -768,10 +629,8 @@ BxEvent* win32_notify_callback(void *unused, BxEvent *event)
         if (opts & sparam->IS_FILENAME) {
           if (opts & sparam->SELECT_FOLDER_DLG) {
             event->retcode = BrowseDir(sparam->get_label(), sparam->getptr());
-          } else if (param->get_parent() == NULL) {
-            event->retcode = AskFilename(GetBochsWindow(), (bx_param_filename_c *)sparam, NULL);
           } else {
-            event->retcode = FloppyDialog((bx_param_filename_c *)sparam);
+            event->retcode = AskFilename(GetBochsWindow(), (bx_param_filename_c *)sparam, NULL);
           }
           return event;
         } else {
@@ -779,8 +638,12 @@ BxEvent* win32_notify_callback(void *unused, BxEvent *event)
           return event;
         }
       } else if (param->get_type() == BXT_LIST) {
-        SIM->get_first_cdrom()->get_param_path(pname, BX_PATHNAME_LEN);
-        event->retcode = win32ParamDialog(GetBochsWindow(), pname);
+        param->get_param_path(pname, BX_PATHNAME_LEN);
+        if (!strncmp(pname, "floppy", 6)) {
+          event->retcode = (Bit32s) win32FloppyParamDialog(GetBochsWindow(), pname);
+        } else {
+          event->retcode = (Bit32s) win32ParamDialog(GetBochsWindow(), pname);
+        }
         return event;
       } else if (param->get_type() == BXT_PARAM_BOOL) {
         UINT flag = MB_YESNO | MB_SETFOREGROUND;
@@ -817,15 +680,19 @@ static int win32_ci_callback(void *userdata, ci_command_t command)
       }
       break;
     case CI_RUNTIME_CONFIG:
-      if (MainMenuDialog(GetBochsWindow(), 1) < 0) {
-        bx_user_quit = 1;
+      if (!bx_gui->has_gui_console()) {
+        if (MainMenuDialog(GetBochsWindow(), 1) < 0) {
+          bx_user_quit = 1;
 #if !BX_DEBUGGER
-        bx_atexit();
-        SIM->quit_sim(1);
+          bx_atexit();
+          SIM->quit_sim(1);
 #else
-        bx_dbg_exit(1);
+          bx_dbg_exit(1);
 #endif
-        return -1;
+          return -1;
+        }
+      } else {
+        bx_text_config_interface(BX_CI_RUNTIME);
       }
       break;
     case CI_SHUTDOWN:
@@ -840,4 +707,4 @@ int init_win32_config_interface()
   return 0;  // success
 }
 
-#endif // BX_USE_TEXTCONFIG && defined(WIN32)
+#endif // BX_USE_WIN32CONFIG
