@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wxmain.h 12583 2014-12-27 15:57:30Z vruppert $
+// $Id: wxmain.h 13014 2016-12-29 09:47:07Z vruppert $
 /////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2014  The Bochs Project
+//  Copyright (C) 2002-2016  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -185,7 +185,7 @@ public:
   bx_bool SimThreadControl(bx_bool resume);
   void OnKillSim(wxCommandEvent& event);
   void OnSim2CIEvent(wxCommandEvent& event);
-  void OnLogAsk(BxEvent *be);
+  void OnLogDlg(BxEvent *be);
   void OnEditPluginCtrl(wxCommandEvent& event);
   void OnEditCPU(wxCommandEvent& event);
   void OnEditCPUID(wxCommandEvent& event);
