@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: hdimage.h 11649 2013-03-08 18:25:32Z vruppert $
+// $Id: hdimage.h 13055 2017-01-30 19:08:37Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2005-2013  The Bochs Project
+//  Copyright (C) 2005-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,11 @@
 #ifndef BX_HDIMAGE_H
 #define BX_HDIMAGE_H
 
+// required for access() checks
+#ifndef F_OK
+#define F_OK 0
+#endif
+
 // hdimage capabilities
 #define HDIMAGE_READONLY      1
 #define HDIMAGE_HAS_GEOMETRY  2
@@ -118,28 +123,41 @@
 // htod : convert host to disk (little) endianness
 // dtoh : convert disk (little) to host endianness
 #if defined (BX_LITTLE_ENDIAN)
+#define htod16(val) (val)
+#define dtoh16(val) (val)
 #define htod32(val) (val)
 #define dtoh32(val) (val)
 #define htod64(val) (val)
 #define dtoh64(val) (val)
 #else
+#define htod16(val) ((((val)&0xff00)>>8) | (((val)&0xff)<<8))
+#define dtoh16(val) htod16(val)
 #define htod32(val) bx_bswap32(val)
 #define dtoh32(val) htod32(val)
 #define htod64(val) bx_bswap64(val)
 #define dtoh64(val) htod64(val)
 #endif
 
-#ifndef HDIMAGE_HEADERS_ONLY
+class device_image_t;
+class redolog_t;
 
 int bx_read_image(int fd, Bit64s offset, void *buf, int count);
 int bx_write_image(int fd, Bit64s offset, void *buf, int count);
+int bx_close_image(int fd, const char *pathname);
 #ifndef WIN32
 int hdimage_open_file(const char *pathname, int flags, Bit64u *fsize, time_t *mtime);
 #else
 int hdimage_open_file(const char *pathname, int flags, Bit64u *fsize, FILETIME *mtime);
 #endif
+int hdimage_detect_image_mode(const char *pathname);
 bx_bool hdimage_backup_file(int fd, const char *backup_fname);
 bx_bool hdimage_copy_file(const char *src, const char *dst);
+bx_bool coherency_check(device_image_t *ro_disk, redolog_t *redolog);
+#ifndef WIN32
+Bit16u fat_datetime(time_t time, int return_time);
+#else
+Bit16u fat_datetime(FILETIME time, int return_time);
+#endif
 
 // base class
 class device_image_t
@@ -174,15 +192,17 @@ class device_image_t
       virtual Bit32u get_capabilities();
 
       // Get modification time in FAT format
-      Bit32u get_timestamp();
+      virtual Bit32u get_timestamp();
 
       // Check image format
       static int check_format(int fd, Bit64u imgsize) {return HDIMAGE_NO_SIGNATURE;}
 
+#ifndef BXIMAGE
       // Save/restore support
       virtual void register_state(bx_list_c *parent);
       virtual bx_bool save_state(const char *backup_fname) {return 0;}
       virtual void restore_state(const char *backup_fname) {}
+#endif
 
       unsigned cylinders;
       unsigned heads;
@@ -197,7 +217,7 @@ class device_image_t
 };
 
 // FLAT MODE
-class default_image_t : public device_image_t
+class flat_image_t : public device_image_t
 {
   public:
       // Open an image with specific flags. Returns non-negative if successful.
@@ -221,9 +241,11 @@ class default_image_t : public device_image_t
       // Check image format
       static int check_format(int fd, Bit64u imgsize);
 
+#ifndef BXIMAGE
       // Save/restore support
       bx_bool save_state(const char *backup_fname);
       void restore_state(const char *backup_fname);
+#endif
 
   private:
       int fd;
@@ -255,15 +277,17 @@ class concat_image_t : public device_image_t
       // written (count).
       ssize_t write(const void* buf, size_t count);
 
+#ifndef BXIMAGE
       // Save/restore support
       bx_bool save_state(const char *backup_fname);
       void restore_state(const char *backup_fname);
+#endif
 
   private:
 #define BX_CONCAT_MAX_IMAGES 8
       int fd_table[BX_CONCAT_MAX_IMAGES];
-      Bit64s start_offset_table[BX_CONCAT_MAX_IMAGES];
-      Bit64s length_table[BX_CONCAT_MAX_IMAGES];
+      Bit64u start_offset_table[BX_CONCAT_MAX_IMAGES];
+      Bit64u length_table[BX_CONCAT_MAX_IMAGES];
       void increment_string(char *str);
       int maxfd;  // number of entries in tables that are valid
 
@@ -276,8 +300,9 @@ class concat_image_t : public device_image_t
       // the following variables tell which partial image file to use for
       // the next read and write.
       int index;  // index into table
-      int fd;     // fd to use for reads and writes
-      Bit64s thismin, thismax; // byte offset boundary of this image
+      int curr_fd;     // fd to use for reads and writes
+      Bit64u curr_min, curr_max; // byte offset boundary of this image
+      Bit64u total_offset;     // current byte offset
       const char *pathname0;
 };
 
@@ -315,9 +340,11 @@ class sparse_image_t : public device_image_t
     // Check image format
     static int check_format(int fd, Bit64u imgsize);
 
+#ifndef BXIMAGE
     // Save/restore support
     bx_bool save_state(const char *backup_fname);
     void restore_state(const char *backup_fname);
+#endif
 
   private:
     int fd;
@@ -418,10 +445,15 @@ class redolog_t
 
       static int check_format(int fd, const char *subtype);
 
+#ifdef BXIMAGE
+      int commit(device_image_t *base_image);
+#else
       bx_bool save_state(const char *backup_fname);
+#endif
 
   private:
       void             print_header();
+      char            *pathname;
       int              fd;
       redolog_header_t header;     // Header is kept in x86 (little) endianness
       Bit32u          *catalog;
@@ -463,12 +495,17 @@ class growing_image_t : public device_image_t
       // written (count).
       ssize_t write(const void* buf, size_t count);
 
+      // Get modification time in FAT format
+      virtual Bit32u get_timestamp();
+
       // Check image format
       static int check_format(int fd, Bit64u imgsize);
 
+#ifndef BXIMAGE
       // Save/restore support
       bx_bool save_state(const char *backup_fname);
       void restore_state(const char *backup_fname);
+#endif
 
   private:
       redolog_t *redolog;
@@ -501,9 +538,11 @@ class undoable_image_t : public device_image_t
       // written (count).
       ssize_t write(const void* buf, size_t count);
 
+#ifndef BXIMAGE
       // Save/restore support
       bx_bool save_state(const char *backup_fname);
       void restore_state(const char *backup_fname);
+#endif
 
   private:
       redolog_t       *redolog;       // Redolog instance
@@ -538,9 +577,11 @@ class volatile_image_t : public device_image_t
       // written (count).
       ssize_t write(const void* buf, size_t count);
 
+#ifndef BXIMAGE
       // Save/restore support
       bx_bool save_state(const char *backup_fname);
       void restore_state(const char *backup_fname);
+#endif
 
   private:
       redolog_t       *redolog;       // Redolog instance
@@ -550,16 +591,14 @@ class volatile_image_t : public device_image_t
 };
 
 
+#ifndef BXIMAGE
 class bx_hdimage_ctl_c : public bx_hdimage_ctl_stub_c {
 public:
   bx_hdimage_ctl_c();
   virtual ~bx_hdimage_ctl_c() {}
   virtual device_image_t *init_image(Bit8u image_mode, Bit64u disk_size, const char *journal);
-#ifdef LOWLEVEL_CDROM
-  virtual LOWLEVEL_CDROM* init_cdrom(const char *dev);
-#endif
+  virtual cdrom_base_c *init_cdrom(const char *dev);
 };
-
-#endif // HDIMAGE_HEADERS_ONLY
+#endif // BXIMAGE
 
 #endif
