@@ -1,12 +1,12 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vpc-img.cc 11528 2012-11-01 15:43:12Z vruppert $
+// $Id: vpc-img.cc 13055 2017-01-30 19:08:37Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // Block driver for Connectix / Microsoft Virtual PC images (ported from QEMU)
 //
 // Copyright (c) 2005  Alex Beregszaszi
 // Copyright (c) 2009  Kevin Wolf <kwolf@suse.de>
-// Copyright (C) 2012  The Bochs Project
+// Copyright (C) 2012-2017  The Bochs Project
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -33,24 +33,29 @@
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
 
+#ifdef BXIMAGE
+#include "config.h"
+#include "misc/bxcompat.h"
+#include "osdep.h"
+#include "misc/bswap.h"
+#else
 #include "iodev.h"
+#endif
 #include "hdimage.h"
 #include "vpc-img.h"
 
 #define LOG_THIS bx_devices.pluginHDImageCtl->
 
-// be*_to_cpu : convert disk (big) to host endianness
-#if defined (BX_LITTLE_ENDIAN)
-#define be16_to_cpu(val) bx_bswap16(val)
-#define be32_to_cpu(val) bx_bswap32(val)
-#define be64_to_cpu(val) bx_bswap64(val)
-#define cpu_to_be32(val) bx_bswap32(val)
-#else
-#define be16_to_cpu(val) (val)
-#define be32_to_cpu(val) (val)
-#define be64_to_cpu(val) (val)
-#define cpu_to_be32(val) (val)
-#endif
+Bit32u vpc_checksum(Bit8u *buf, size_t size)
+{
+  Bit32u res = 0;
+  unsigned i;
+
+  for (i = 0; i < size; i++)
+    res += buf[i];
+
+  return ~res;
+}
 
 int vpc_image_t::check_format(int fd, Bit64u imgsize)
 {
@@ -86,7 +91,7 @@ int vpc_image_t::open(const char* _pathname, int flags)
   vhd_dyndisk_header_t *dyndisk_header;
   Bit8u buf[HEADER_SIZE];
   Bit32u checksum;
-  Bit64u imgsize = 0;
+  Bit64u offset = 0, imgsize = 0;
   int disk_type;
 
   pathname = _pathname;
@@ -105,16 +110,20 @@ int vpc_image_t::open(const char* _pathname, int flags)
         BX_ERROR(("VPC: signature missed in file '%s'", _pathname));
         return -1;
     }
+  } else if (disk_type == VHD_FIXED) {
+    offset = imgsize - HEADER_SIZE;
   }
-  if (bx_read_image(fd, 0, (char*)footer_buf, HEADER_SIZE) != HEADER_SIZE) {
+  if (bx_read_image(fd, offset, (char*)footer_buf, HEADER_SIZE) != HEADER_SIZE) {
     return -1;
   }
   footer = (vhd_footer_t*)footer_buf;
 
   checksum = be32_to_cpu(footer->checksum);
   footer->checksum = 0;
-  if (vpc_checksum(footer_buf, HEADER_SIZE) != checksum)
+  if (vpc_checksum(footer_buf, HEADER_SIZE) != checksum) {
     BX_ERROR(("The header checksum of '%s' is incorrect", pathname));
+    return -1;
+  }
 
   // Write 'checksum' back to footer, or else will leave it with zero.
   footer->checksum = be32_to_cpu(checksum);
@@ -129,20 +138,20 @@ int vpc_image_t::open(const char* _pathname, int flags)
   hd_size = sector_count * 512;
 
   if (sector_count >= 65535 * 16 * 255) {
-    ::close(fd);
+    bx_close_image(fd, pathname);
     return -EFBIG;
   }
 
   if (disk_type == VHD_DYNAMIC) {
     if (bx_read_image(fd, be64_to_cpu(footer->data_offset), buf, HEADER_SIZE) != HEADER_SIZE) {
-      ::close(fd);
+      bx_close_image(fd, pathname);
       return -1;
     }
 
     dyndisk_header = (vhd_dyndisk_header_t*)buf;
 
     if (strncmp((char*)dyndisk_header->magic, "cxsparse", 8)) {
-      ::close(fd);
+      bx_close_image(fd, pathname);
       return -1;
     }
 
@@ -154,7 +163,7 @@ int vpc_image_t::open(const char* _pathname, int flags)
 
     bat_offset = be64_to_cpu(dyndisk_header->table_offset);
     if (bx_read_image(fd, bat_offset, (void*)pagetable, max_table_entries * 4) != (max_table_entries * 4)) {
-      ::close(fd);
+      bx_close_image(fd, pathname);
       return -1;
     }
 
@@ -184,7 +193,7 @@ void vpc_image_t::close(void)
 {
   if (fd > -1) {
     delete [] pagetable;
-    ::close(fd);
+    bx_close_image(fd, pathname);
   }
 }
 
@@ -283,6 +292,7 @@ Bit32u vpc_image_t::get_capabilities(void)
   return HDIMAGE_HAS_GEOMETRY;
 }
 
+#ifndef BXIMAGE
 bx_bool vpc_image_t::save_state(const char *backup_fname)
 {
   return hdimage_backup_file(fd, backup_fname);
@@ -311,17 +321,7 @@ void vpc_image_t::restore_state(const char *backup_fname)
   }
   device_image_t::open(pathname);
 }
-
-Bit32u vpc_image_t::vpc_checksum(Bit8u *buf, size_t size)
-{
-  Bit32u res = 0;
-  unsigned i;
-
-  for (i = 0; i < size; i++)
-    res += buf[i];
-
-  return ~res;
-}
+#endif
 
 /*
  * Returns the absolute byte offset of the given sector in the image file.
