@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: soundsdl.cc 12699 2015-03-29 15:53:56Z vruppert $
+// $Id: soundsdl.cc 13160 2017-03-30 18:08:15Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2012-2015  The Bochs Project
+//  Copyright (C) 2012-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,14 @@
 
 // Lowlevel sound output support for SDL written by Volker Ruppert
 
+// Define BX_PLUGGABLE in files that can be compiled into plugins.  For
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
+// is used to know when we are exporting symbols and when we are importing.
+#define BX_PLUGGABLE
+
 #include "iodev.h"
 #include "soundlow.h"
+#include "soundmod.h"
 #include "soundsdl.h"
 
 #if BX_HAVE_SOUND_SDL && BX_SUPPORT_SOUNDLOW
@@ -30,6 +36,19 @@
 
 #include <SDL.h>
 
+// sound driver plugin entry points
+
+int CDECL libsdl_sound_plugin_init(plugin_t *plugin, plugintype_t type)
+{
+  // Nothing here yet
+  return 0; // Success
+}
+
+void CDECL libsdl_sound_plugin_fini(void)
+{
+  // Nothing here yet
+}
+
 // SDL audio callback
 
 void sdl_callback(void *thisptr, Bit8u *stream, int len)
@@ -38,12 +57,12 @@ void sdl_callback(void *thisptr, Bit8u *stream, int len)
   ((bx_soundlow_waveout_sdl_c*)thisptr)->mixer_common(stream, len);
 }
 
-// bx_soundlow_waveout_sdl_c class implemenzation
+// bx_soundlow_waveout_sdl_c class implementation
 
 bx_soundlow_waveout_sdl_c::bx_soundlow_waveout_sdl_c()
     :bx_soundlow_waveout_c()
 {
-  WaveOpen = 0;
+  WaveOutOpen = 0;
   if (SDL_InitSubSystem(SDL_INIT_AUDIO)) {
     BX_PANIC(("Initialization of sound lowlevel module 'sdl' failed"));
   } else {
@@ -57,7 +76,7 @@ bx_soundlow_waveout_sdl_c::~bx_soundlow_waveout_sdl_c()
     unregister_wave_callback(pcm_callback_id);
     pcm_callback_id = -1;
   }
-  WaveOpen = 0;
+  WaveOutOpen = 0;
   mixer_control = 0;
   SDL_CloseAudio();
   SDL_QuitSubSystem(SDL_INIT_AUDIO);
@@ -66,6 +85,7 @@ bx_soundlow_waveout_sdl_c::~bx_soundlow_waveout_sdl_c()
 int bx_soundlow_waveout_sdl_c::openwaveoutput(const char *wavedev)
 {
   set_pcm_params(&real_pcm_param);
+  start_resampler_thread();
   return BX_SOUNDLOW_OK;
 }
 
@@ -94,55 +114,50 @@ int bx_soundlow_waveout_sdl_c::set_pcm_params(bx_pcm_param_t *param)
   fmt.samples = fmt.freq / 10;
   fmt.callback = sdl_callback;
   fmt.userdata = this;
-  if (WaveOpen) {
+  if (WaveOutOpen) {
     SDL_CloseAudio();
   } else {
     pcm_callback_id = register_wave_callback(this, pcm_callback);
   }
   if (SDL_OpenAudio(&fmt, NULL) < 0) {
     BX_PANIC(("SDL_OpenAudio() failed"));
-    WaveOpen = 0;
+    WaveOutOpen = 0;
     return BX_SOUNDLOW_ERR;
   } else {
-    WaveOpen = 1;
+    if (fmt.freq != param->samplerate) {
+      param->samplerate = fmt.freq;
+      BX_INFO(("changed sample rate to %d", fmt.freq));
+    }
+    WaveOutOpen = 1;
     mixer_control = 1;
   }
   SDL_PauseAudio(0);
   return BX_SOUNDLOW_OK;
 }
 
-int bx_soundlow_waveout_sdl_c::sendwavepacket(int length, Bit8u data[], bx_pcm_param_t *src_param)
+void bx_soundlow_waveout_sdl_c::resampler(audio_buffer_t *inbuffer, audio_buffer_t *outbuffer)
 {
-  int ret = BX_SOUNDLOW_OK;
-  int len2;
+  Bit32u fcount;
+  float *fbuffer = NULL;
 
-  if (memcmp(src_param, &emu_pcm_param, sizeof(bx_pcm_param_t)) != 0) {
-    emu_pcm_param = *src_param;
-    cvt_mult = (src_param->bits == 8) ? 2 : 1;
-    if (src_param->channels == 1) cvt_mult <<= 1;
-    if (src_param->samplerate != real_pcm_param.samplerate) {
-      real_pcm_param.samplerate = src_param->samplerate;
-      set_pcm_params(&real_pcm_param);
-    }
-  }
-  len2 = length * cvt_mult;
+  UNUSED(outbuffer);
+  fcount = resampler_common(inbuffer, &fbuffer);
   SDL_LockAudio();
-  if (WaveOpen) {
-    audio_buffer_t *newbuffer = new_audio_buffer(len2);
-    convert_pcm_data(data, length, newbuffer->data, len2, src_param);
-  } else {
-    BX_ERROR(("SDL: audio not open"));
-    ret = BX_SOUNDLOW_ERR;
+  if (WaveOutOpen) {
+    audio_buffer_t *newbuffer = audio_buffers[1]->new_buffer(fcount << 1);
+    convert_float_to_s16le(fbuffer, fcount, newbuffer->data);
   }
   SDL_UnlockAudio();
-  return ret;
+  if (fbuffer != NULL) {
+    delete [] fbuffer;
+  }
 }
 
 bx_bool bx_soundlow_waveout_sdl_c::mixer_common(Bit8u *buffer, int len)
 {
   Bit32u len2 = 0;
 
-  Bit8u *tmpbuffer = (Bit8u*)malloc(len);
+  Bit8u *tmpbuffer = new Bit8u[len];
   for (int i = 0; i < cb_count; i++) {
     if (get_wave[i].cb != NULL) {
       memset(tmpbuffer, 0, len);
@@ -152,7 +167,7 @@ bx_bool bx_soundlow_waveout_sdl_c::mixer_common(Bit8u *buffer, int len)
       }
     }
   }
-  free(tmpbuffer);
+  delete [] tmpbuffer;
   return 1;
 }
 
@@ -166,14 +181,118 @@ void bx_soundlow_waveout_sdl_c::unregister_wave_callback(int callback_id)
   SDL_UnlockAudio();
 }
 
-// bx_sound_sdl_c class implemenzation
+// bx_soundlow_wavein_sdl2_c class implementation
 
-bx_sound_sdl_c::bx_sound_sdl_c()
-    :bx_sound_lowlevel_c()
+#if BX_HAVE_SDL2_AUDIO_CAPTURE
+bx_soundlow_wavein_sdl2_c::bx_soundlow_wavein_sdl2_c()
+    :bx_soundlow_wavein_c()
 {
-  // nothing here yet
+  WaveInOpen = 0;
+  devID = 0;
 }
 
+bx_soundlow_wavein_sdl2_c::~bx_soundlow_wavein_sdl2_c()
+{
+  if (WaveInOpen) {
+    SDL_CloseAudioDevice(devID);
+  }
+}
+
+int bx_soundlow_wavein_sdl2_c::openwaveinput(const char *wavedev, sound_record_handler_t rh)
+{
+  UNUSED(wavedev);
+  record_handler = rh;
+  if (rh != NULL) {
+    record_timer_index = DEV_register_timer(this, record_timer_handler, 1, 1, 0, "wavein");
+    // record timer: inactive, continuous, frequency variable
+  }
+  return BX_SOUNDLOW_OK;
+}
+
+int bx_soundlow_wavein_sdl2_c::startwaverecord(bx_pcm_param_t *param)
+{
+  int signeddata = param->format & 1;
+  Bit64u timer_val;
+  Bit8u shift = 0;
+
+  if (record_timer_index != BX_NULL_TIMER_HANDLE) {
+    if (param->bits == 16) shift++;
+    if (param->channels == 2) shift++;
+    record_packet_size = (param->samplerate / 10) << shift; // 0.1 sec
+    if (record_packet_size > BX_SOUNDLOW_WAVEPACKETSIZE) {
+      record_packet_size = BX_SOUNDLOW_WAVEPACKETSIZE;
+    }
+    timer_val = (Bit64u)record_packet_size * 1000000 / (param->samplerate << shift);
+    bx_pc_system.activate_timer(record_timer_index, (Bit32u)timer_val, 1);
+  }
+  fmt.freq = param->samplerate;
+
+  if (param->bits == 16) {
+    if (signeddata == 1)
+      fmt.format = AUDIO_S16;
+    else
+      fmt.format = AUDIO_U16;
+  } else if (param->bits == 8) {
+    if (signeddata == 1)
+      fmt.format = AUDIO_S8;
+    else
+      fmt.format = AUDIO_U8;
+  } else
+    return BX_SOUNDLOW_ERR;
+
+  fmt.channels = param->channels;
+  fmt.samples = fmt.freq / 10;
+  fmt.callback = NULL;
+  fmt.userdata = NULL;
+  if (WaveInOpen) {
+    SDL_CloseAudioDevice(devID);
+  }
+  devID = SDL_OpenAudioDevice(NULL, 1, &fmt, NULL, SDL_AUDIO_ALLOW_ANY_CHANGE);
+  if (devID <= 0) {
+    BX_PANIC(("SDL_OpenAudioDevive() failed"));
+    WaveInOpen = 0;
+    return BX_SOUNDLOW_ERR;
+  } else {
+    if (fmt.freq != param->samplerate) {
+      param->samplerate = fmt.freq;
+      BX_INFO(("changed sample rate to %d", fmt.freq));
+    }
+    WaveInOpen = 1;
+  }
+  SDL_PauseAudioDevice(devID, 0);
+  return BX_SOUNDLOW_OK;
+}
+
+int bx_soundlow_wavein_sdl2_c::getwavepacket(int length, Bit8u data[])
+{
+  SDL_DequeueAudio(devID, data, length);
+  return BX_SOUNDLOW_OK;
+}
+
+int bx_soundlow_wavein_sdl2_c::stopwaverecord()
+{
+  SDL_PauseAudioDevice(devID, 1);
+  if (record_timer_index != BX_NULL_TIMER_HANDLE) {
+    bx_pc_system.deactivate_timer(record_timer_index);
+  }
+  return BX_SOUNDLOW_OK;
+}
+
+void bx_soundlow_wavein_sdl2_c::record_timer_handler(void *this_ptr)
+{
+  bx_soundlow_wavein_sdl2_c *class_ptr = (bx_soundlow_wavein_sdl2_c *) this_ptr;
+
+  class_ptr->record_timer();
+}
+
+void bx_soundlow_wavein_sdl2_c::record_timer(void)
+{
+  record_handler(this, record_packet_size);
+}
+#endif
+
+// bx_sound_sdl_c class implementation
+
 bx_soundlow_waveout_c* bx_sound_sdl_c::get_waveout()
 {
   if (waveout == NULL) {
@@ -182,4 +301,14 @@ bx_soundlow_waveout_c* bx_sound_sdl_c::get_waveout()
   return waveout;
 }
 
+#if BX_HAVE_SDL2_AUDIO_CAPTURE
+bx_soundlow_wavein_c* bx_sound_sdl_c::get_wavein()
+{
+  if (wavein == NULL) {
+    wavein = new bx_soundlow_wavein_sdl2_c();
+  }
+  return wavein;
+}
+#endif
+
 #endif  // BX_HAVE_SOUND_SDL
