@@ -1,12 +1,12 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_common.h 12488 2014-09-22 19:49:39Z vruppert $
+// $Id: usb_common.h 13169 2017-04-01 10:35:38Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // Generic USB emulation code
 //
 // Copyright (c) 2005       Fabrice Bellard
-// Copyright (C) 2009       Benjamin D Lunt (fys at frontiernet net)
-//               2009-2014  The Bochs Project
+// Copyright (C) 2009-2016  Benjamin D Lunt (fys [at] fysnet [dot] net)
+//               2009-2017  The Bochs Project
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -38,11 +38,12 @@
 #define USB_MSG_DETACH   0x101
 #define USB_MSG_RESET    0x102
 
-#define USB_RET_NODEV  (-1)
-#define USB_RET_NAK    (-2)
-#define USB_RET_STALL  (-3)
-#define USB_RET_BABBLE (-4)
-#define USB_RET_ASYNC  (-5)
+#define USB_RET_NODEV   (-1)
+#define USB_RET_NAK     (-2)
+#define USB_RET_STALL   (-3)
+#define USB_RET_BABBLE  (-4)
+#define USB_RET_IOERROR (-5)
+#define USB_RET_ASYNC   (-6)
 
 #define USB_SPEED_LOW   0
 #define USB_SPEED_FULL  1
@@ -117,6 +118,13 @@
 // USB 3.0
 #define USB_DT_BIN_DEV_OBJ_STORE        0x0F
 
+typedef struct USBPacket USBPacket;
+
+#define USB_EVENT_WAKEUP 0
+#define USB_EVENT_ASYNC  1
+
+typedef void USBCallback(int event, USBPacket *packet, void *dev, int port);
+
 class usb_device_c;
 
 struct USBPacket {
@@ -125,9 +133,19 @@ struct USBPacket {
   Bit8u devep;
   Bit8u *data;
   int len;
+  USBCallback *complete_cb;
+  void *complete_dev;
   usb_device_c *dev;
 };
 
+typedef struct USBAsync {
+  USBPacket packet;
+  Bit64u    td_addr;
+  bx_bool done;
+  Bit16u  slot_ep;
+  struct USBAsync *next;
+} USBAsync;
+
 enum usbdev_type {
   USB_DEV_TYPE_NONE=0,
   USB_DEV_TYPE_MOUSE,
@@ -136,7 +154,8 @@ enum usbdev_type {
   USB_DEV_TYPE_DISK,
   USB_DEV_TYPE_CDROM,
   USB_DEV_TYPE_HUB,
-  USB_DEV_TYPE_PRINTER
+  USB_DEV_TYPE_PRINTER,
+  USB_DEV_TYPE_FLOPPY
 };
 
 class bx_usb_devctl_c : public bx_usb_devctl_stub_c {
@@ -156,10 +175,11 @@ public:
 
   virtual bx_bool init() {return 1;}
   virtual const char* get_info() {return NULL;}
+  virtual usb_device_c* find_device(Bit8u addr);
 
   virtual int handle_packet(USBPacket *p);
   virtual void handle_reset() {}
-  virtual int handle_control(int request, int value, int index, int length, Bit8u *data) {return 0;}
+  virtual int handle_control(int request, int value, int index, int length, Bit8u *data) {return -1;}
   virtual int handle_data(USBPacket *p) {return 0;}
   void register_state(bx_list_c *parent);
   virtual void register_state_specific(bx_list_c *parent) {}
@@ -170,10 +190,26 @@ public:
 
   bx_bool get_connected() {return d.connected;}
   usbdev_type get_type() {return d.type;}
-  int get_maxspeed() {return d.maxspeed;}
   int get_speed() {return d.speed;}
-  void set_speed(int speed) {d.speed = speed;}
+  bx_bool set_speed(int speed)
+  {
+    if ((speed >= d.minspeed) && (speed <= d.maxspeed)) {
+      d.speed = speed;
+      return 1;
+    } else {
+      return 0;
+    }
+  }
+
   Bit8u get_address() {return d.addr;}
+  void set_async_mode(bx_bool async) {d.async_mode = async;}
+  void set_event_handler(void *dev, USBCallback *cb, int port)
+  {
+    d.event.dev = dev;
+    d.event.cb = cb;
+    d.event.port = port;
+  }
+  void set_debug_mode();
 
   void usb_send_msg(int msg);
 
@@ -181,12 +217,22 @@ protected:
   struct {
     enum usbdev_type type;
     bx_bool connected;
+    int minspeed;
     int maxspeed;
     int speed;
     Bit8u addr;
     Bit8u config;
+    Bit8u interface;
     char devname[32];
 
+    const Bit8u *dev_descriptor;
+    const Bit8u *config_descriptor;
+    int device_desc_size;
+    int config_desc_size;
+    const char *vendor_desc;
+    const char *product_desc;
+    const char *serial_num;
+
     int state;
     Bit8u setup_buf[8];
     Bit8u data_buf[1024];
@@ -195,12 +241,40 @@ protected:
     int setup_len;
     int setup_index;
     bx_bool stall;
+    bx_bool async_mode;
+    struct {
+      USBCallback *cb;
+      void *dev;
+      int port;
+    } event;
+    bx_list_c *sr;
   } d;
 
+  int handle_control_common(int request, int value, int index, int length, Bit8u *data);
   void usb_dump_packet(Bit8u *data, unsigned size);
   int set_usb_string(Bit8u *buf, const char *str);
 };
 
+static BX_CPP_INLINE void usb_packet_init(USBPacket *p, int size)
+{
+  memset(p, 0, sizeof(USBPacket));
+  if (size > 0) {
+    p->data = new Bit8u[size];
+    if (!p->data) {
+      return;
+    }
+  }
+  p->len = size;
+}
+
+static BX_CPP_INLINE void usb_packet_cleanup(USBPacket *p)
+{
+  if (p->data) {
+    delete [] p->data;
+    p->data = NULL;
+  }
+}
+
 static BX_CPP_INLINE void usb_defer_packet(USBPacket *p, usb_device_c *dev)
 {
   p->dev = dev;
@@ -211,4 +285,83 @@ static BX_CPP_INLINE void usb_cancel_packet(USBPacket *p)
   p->dev->cancel_packet(p);
 }
 
+static BX_CPP_INLINE void usb_packet_complete(USBPacket *p)
+{
+    p->complete_cb(USB_EVENT_ASYNC, p, p->complete_dev, 0);
+}
+
+// Async packet support
+
+static BX_CPP_INLINE USBAsync* create_async_packet(USBAsync **base, Bit64u addr, int maxlen)
+{
+  USBAsync *p;
+
+  p = new USBAsync;
+  usb_packet_init(&p->packet, maxlen);
+  p->td_addr = addr;
+  p->done = 0;
+  p->next = *base;
+  *base = p;
+  return p;
+}
+
+static BX_CPP_INLINE void remove_async_packet(USBAsync **base, USBAsync *p)
+{
+  USBAsync *last;
+
+  if (*base == p) {
+    *base = p->next;
+  } else {
+    last = *base;
+    while (last != NULL) {
+      if (last->next != p)
+        last = last->next;
+      else
+        break;
+    }
+    if (last) {
+      last->next = p->next;
+    } else {
+      return;
+    }
+  }
+  usb_packet_cleanup(&p->packet);
+  delete p;
+}
+
+static BX_CPP_INLINE USBAsync* find_async_packet(USBAsync **base, Bit64u addr)
+{
+  USBAsync *p = *base;
+
+  while (p != NULL) {
+    if (p->td_addr != addr)
+      p = p->next;
+    else
+      break;
+  }
+  return p;
+}
+
+static BX_CPP_INLINE struct USBAsync *container_of_usb_packet(void *ptr)
+{
+  return reinterpret_cast<struct USBAsync*>(static_cast<char*>(ptr) -
+    reinterpret_cast<size_t>(&(static_cast<struct USBAsync*>(0)->packet)));
+}
+
+// dword read / write helper functions
+
+static BX_CPP_INLINE void get_dwords(bx_phy_address addr, Bit32u *buf, int num)
+{
+  for (int i = 0; i < num; i++, buf++, addr += sizeof(*buf)) {
+    DEV_MEM_READ_PHYSICAL(addr, 4, (Bit8u*)buf);
+  }
+}
+
+static BX_CPP_INLINE void put_dwords(bx_phy_address addr, Bit32u *buf, int num)
+{
+  for (int i = 0; i < num; i++, buf++, addr += sizeof(*buf)) {
+    DEV_MEM_WRITE_PHYSICAL(addr, 4, (Bit8u*)buf);
+  }
+}
+
 #endif
