@@ -1,13 +1,13 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_hid.cc 12488 2014-09-22 19:49:39Z vruppert $
+// $Id: usb_hid.cc 13170 2017-04-02 08:47:14Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // USB HID emulation support (mouse and tablet) ported from QEMU
-// USB keypad emulation based on code by Benjamin D Lunt (fys at frontiernet net)
+// USB keypad emulation based on code by Benjamin D Lunt (fys [at] fysnet [dot] net)
 //
 // Copyright (c) 2005       Fabrice Bellard
 // Copyright (c) 2007       OpenMoko, Inc.  (andrew@openedhand.com)
-// Copyright (C) 2009-2014  The Bochs Project
+// Copyright (C) 2009-2017  The Bochs Project
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@
 /////////////////////////////////////////////////////////////////////////
 //  Portions of this file contain code released under the LGPL.
 //
-// Copyright (C) 2004/2005  Benjamin D Lunt (fys at frontiernet net)
+// Copyright (C) 2004-2016  Benjamin D Lunt (fys [at] fysnet [dot] net)
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -67,9 +67,9 @@
 #define SET_IDLE     0x210a
 #define SET_PROTOCOL 0x210b
 
-#define KEYPAD_LEN   128
+#define KEYPAD_LEN   16
 struct KEYPAD {
-  Bit8u  scan_code[8];
+  Bit32u bxkey;
   Bit8u  keypad_packet[8];
 };
 
@@ -87,9 +87,29 @@ static const Bit8u bx_mouse_dev_descriptor[] = {
   0x01, 0x00, /*  u16 idProduct; */
   0x00, 0x00, /*  u16 bcdDevice */
 
-  0x03,       /*  u8  iManufacturer; */
+  0x01,       /*  u8  iManufacturer; */
   0x02,       /*  u8  iProduct; */
-  0x01,       /*  u8  iSerialNumber; */
+  0x03,       /*  u8  iSerialNumber; */
+  0x01        /*  u8  bNumConfigurations; */
+};
+
+static const Bit8u bx_mouse_dev_descriptor2[] = {
+  0x12,       /*  u8 bLength; */
+  0x01,       /*  u8 bDescriptorType; Device */
+  0x00, 0x02, /*  u16 bcdUSB; v2.0 */
+
+  0x00,       /*  u8  bDeviceClass; */
+  0x00,       /*  u8  bDeviceSubClass; */
+  0x00,       /*  u8  bDeviceProtocol; [ low/full speeds only ] */
+  0x40,       /*  u8  bMaxPacketSize0; 64 Bytes */
+
+  0x27, 0x06, /*  u16 idVendor; */
+  0x01, 0x00, /*  u16 idProduct; */
+  0x00, 0x00, /*  u16 bcdDevice */
+
+  0x01,       /*  u8  iManufacturer; */
+  0x02,       /*  u8  iProduct; */
+  0x03,       /*  u8  iSerialNumber; */
   0x01        /*  u8  bNumConfigurations; */
 };
 
@@ -148,6 +168,61 @@ static const Bit8u bx_mouse_config_descriptor[] = {
   0x0a,       /*  u8  ep_bInterval; (0 - 255ms -- usb 2.0 spec) */
 };
 
+static const Bit8u bx_mouse_config_descriptor2[] = {
+  /* one configuration */
+  0x09,       /*  u8  bLength; */
+  0x02,       /*  u8  bDescriptorType; Configuration */
+  0x22, 0x00, /*  u16 wTotalLength; */
+  0x01,       /*  u8  bNumInterfaces; (1) */
+  0x01,       /*  u8  bConfigurationValue; */
+  0x04,       /*  u8  iConfiguration; */
+  0xa0,       /*  u8  bmAttributes;
+			 Bit 7: must be set,
+			     6: Self-powered,
+			     5: Remote wakeup,
+			     4..0: resvd */
+  50,         /*  u8  MaxPower; */
+
+  /* USB 1.1:
+   * USB 2.0, single TT organization (mandatory):
+   *	one interface, protocol 0
+   *
+   * USB 2.0, multiple TT organization (optional):
+   *	two interfaces, protocols 1 (like single TT)
+   *	and 2 (multiple TT mode) ... config is
+   *	sometimes settable
+   *	NOT IMPLEMENTED
+   */
+
+  /* one interface */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x00,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x01,       /*  u8  if_bNumEndpoints; */
+  0x03,       /*  u8  if_bInterfaceClass; */
+  0x01,       /*  u8  if_bInterfaceSubClass; */
+  0x02,       /*  u8  if_bInterfaceProtocol; [usb1.1 or single tt] */
+  0x05,       /*  u8  if_iInterface; */
+
+  /* HID descriptor */
+  0x09,        /*  u8  bLength; */
+  0x21,        /*  u8 bDescriptorType; */
+  0x00, 0x01,  /*  u16 HID_class */
+  0x00,        /*  u8 country_code */
+  0x01,        /*  u8 num_descriptors */
+  0x22,        /*  u8 type; Report */
+  50, 0,       /*  u16 len */
+
+  /* one endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
+  0x03,       /*  u8  ep_bmAttributes; Interrupt */
+  0x03, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x07,       /*  u8  ep_bInterval; (2 ^ (8-1) * 125 usecs = 8 ms) */
+};
+
 static const Bit8u bx_mouse_hid_descriptor[] = {
   /* HID descriptor */
   0x09,        /*  u8  bLength; */
@@ -213,6 +288,50 @@ static const Bit8u bx_tablet_config_descriptor[] = {
   0x0a,       /*  u8  ep_bInterval; (0 - 255ms -- usb 2.0 spec) */
 };
 
+static const Bit8u bx_tablet_config_descriptor2[] = {
+  /* one configuration */
+  0x09,       /*  u8  bLength; */
+  0x02,       /*  u8  bDescriptorType; Configuration */
+  0x22, 0x00, /*  u16 wTotalLength; */
+  0x01,       /*  u8  bNumInterfaces; (1) */
+  0x01,       /*  u8  bConfigurationValue; */
+  0x04,       /*  u8  iConfiguration; */
+  0xa0,       /*  u8  bmAttributes;
+			 Bit 7: must be set,
+			     6: Self-powered,
+			     5: Remote wakeup,
+			     4..0: resvd */
+  50,         /*  u8  MaxPower; */
+
+  /* one interface */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x00,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x01,       /*  u8  if_bNumEndpoints; */
+  0x03,       /*  u8  if_bInterfaceClass; */
+  0x01,       /*  u8  if_bInterfaceSubClass; */
+  0x02,       /*  u8  if_bInterfaceProtocol; [usb1.1 or single tt] */
+  0x05,       /*  u8  if_iInterface; */
+
+  /* HID descriptor */
+  0x09,        /*  u8  bLength; */
+  0x21,        /*  u8 bDescriptorType; */
+  0x00, 0x01,  /*  u16 HID_class */
+  0x00,        /*  u8 country_code */
+  0x01,        /*  u8 num_descriptors */
+  0x22,        /*  u8 type; Report */
+  74, 0,       /*  u16 len */
+
+  /* one endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
+  0x03,       /*  u8  ep_bmAttributes; Interrupt */
+  0x08, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x04,       /*  u8  ep_bInterval; (2 ^ (4-1) * 125 usecs = 1 ms) */
+};
+
 static const Bit8u bx_tablet_hid_descriptor[] = {
   /* HID descriptor */
   0x09,        /*  u8  bLength; */
@@ -278,9 +397,29 @@ static const Bit8u bx_keypad_dev_descriptor[] = {
   0x01, 0x01, /*  u16 idProduct; */
   0x01, 0x00, /*  u16 bcdDevice */
 
-  0x03,       /*  u8  iManufacturer; */
+  0x01,       /*  u8  iManufacturer; */
   0x02,       /*  u8  iProduct; */
-  0x01,       /*  u8  iSerialNumber; */
+  0x03,       /*  u8  iSerialNumber; */
+  0x01        /*  u8  bNumConfigurations; */
+};
+
+static const Bit8u bx_keypad_dev_descriptor2[] = {
+  0x12,       /*  u8 bLength; */
+  0x01,       /*  u8 bDescriptorType; Device */
+  0x00, 0x02, /*  u16 bcdUSB; v2.0 */
+
+  0x00,       /*  u8  bDeviceClass; */
+  0x00,       /*  u8  bDeviceSubClass; */
+  0x00,       /*  u8  bDeviceProtocol; [ low/full speeds only ] */
+  0x40,       /*  u8  bMaxPacketSize0; 64 Bytes */
+
+  0xB4, 0x04, /*  u16 idVendor; */
+  0x01, 0x01, /*  u16 idProduct; */
+  0x01, 0x00, /*  u16 bcdDevice */
+
+  0x01,       /*  u8  iManufacturer; */
+  0x02,       /*  u8  iProduct; */
+  0x03,       /*  u8  iSerialNumber; */
   0x01        /*  u8  bNumConfigurations; */
 };
 
@@ -356,6 +495,78 @@ static const Bit8u bx_keypad_config_descriptor[] = {
   0x0a,       /*  u8  ep_bInterval; (0 - 255ms -- usb 2.0 spec) */
 };
 
+static const Bit8u bx_keypad_config_descriptor2[] = {
+  /* one configuration */
+  0x09,       /*  u8  bLength; */
+  0x02,       /*  u8  bDescriptorType; Configuration */
+  0x3B, 0x00, /*  u16 wTotalLength; */
+  0x02,       /*  u8  bNumInterfaces; (2) */
+  0x01,       /*  u8  bConfigurationValue; */
+  0x04,       /*  u8  iConfiguration; */
+  0xa0,       /*  u8  bmAttributes;
+			 Bit 7: must be set,
+			     6: Self-powered,
+			     5: Remote wakeup,
+			     4..0: resvd */
+  50,         /*  u8  MaxPower; */
+
+  /* interface #1 of 2 */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x00,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x01,       /*  u8  if_bNumEndpoints; */
+  0x03,       /*  u8  if_bInterfaceClass; */
+  0x01,       /*  u8  if_bInterfaceSubClass; */
+  0x01,       /*  u8  if_bInterfaceProtocol; [usb1.1 or single tt] */
+  0x05,       /*  u8  if_iInterface; */
+
+  /* HID descriptor */
+  0x09,        /*  u8  bLength; */
+  0x21,        /*  u8 bDescriptorType; */
+  0x00, 0x01,  /*  u16 HID_class */
+  0x00,        /*  u8 country_code */
+  0x01,        /*  u8 num_descriptors */
+  0x22,        /*  u8 type; Report */
+  65, 0,       /*  u16 len */
+
+  /* one endpoint (status change endpoint) */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
+  0x03,       /*  u8  ep_bmAttributes; Interrupt */
+  0x08, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x07,       /*  u8  ep_bInterval; (2 ^ (8-1) * 125 usecs = 8 ms) */
+
+  /* interface #2 of 2 */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x01,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x01,       /*  u8  if_bNumEndpoints; */
+  0x03,       /*  u8  if_bInterfaceClass; */
+  0x01,       /*  u8  if_bInterfaceSubClass; */
+  0x02,       /*  u8  if_bInterfaceProtocol; [usb1.1 or single tt] */
+  0x06,       /*  u8  if_iInterface; */
+
+  /* HID descriptor */
+  0x09,        /*  u8  bLength; */
+  0x21,        /*  u8 bDescriptorType; */
+  0x00, 0x01,  /*  u16 HID_class */
+  0x00,        /*  u8 country_code */
+  0x01,        /*  u8 num_descriptors */
+  0x22,        /*  u8 type; Report */
+  50, 0,       /*  u16 len */
+
+  /* one endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x82,       /*  u8  ep_bEndpointAddress; IN Endpoint 2 */
+  0x03,       /*  u8  ep_bmAttributes; Interrupt */
+  0x08, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x07,       /*  u8  ep_bInterval; (2 ^ (8-1) * 125 usecs = 8 ms) */
+};
+
 static const Bit8u bx_keypad_hid_descriptor[] = {
   /* HID descriptor */
   0x09,        /*  u8  bLength; */
@@ -389,82 +600,117 @@ static const Bit8u bx_keypad_hid_report_descriptor2[] = {
   0x01, 0xC0
 };
 
-// this interface has a key conversion table of len = 18
+// this interface has a key conversion table of len = 16
 struct KEYPAD keypad_lookup[KEYPAD_LEN] = {
-  { { 0x6C,       }, { 0x00, 0x00, 0x5F, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 7
-  { { 0x6B,       }, { 0x00, 0x00, 0x5C, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 4
-  { { 0x69,       }, { 0x00, 0x00, 0x59, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 1
-  { { 0x70,       }, { 0x00, 0x00, 0x62, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 0
-  { { 0xE0, 0x4A, }, { 0x00, 0x00, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // /
-  { { 0x75,       }, { 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 8
-  { { 0x73,       }, { 0x00, 0x00, 0x5D, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 5
-  { { 0x72,       }, { 0x00, 0x00, 0x5A, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 2
-  { { 0x7C,       }, { 0x00, 0x00, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // *
-  { { 0x7D,       }, { 0x00, 0x00, 0x61, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 9
-  { { 0x74,       }, { 0x00, 0x00, 0x5E, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 6
-  { { 0x7A,       }, { 0x00, 0x00, 0x5B, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 3
-  { { 0x71,       }, { 0x00, 0x00, 0x63, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // -
-  { { 0x7B,       }, { 0x00, 0x00, 0x56, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // +
-  { { 0x79,       }, { 0x00, 0x00, 0x57, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // .
-  { { 0xE0, 0x5A  }, { 0x00, 0x00, 0x58, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // Enter
+  { BX_KEY_KP_HOME,      { 0x00, 0x00, 0x5F, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 7
+  { BX_KEY_KP_LEFT,      { 0x00, 0x00, 0x5C, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 4
+  { BX_KEY_KP_END,       { 0x00, 0x00, 0x59, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 1
+  { BX_KEY_KP_INSERT,    { 0x00, 0x00, 0x62, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 0
+  { BX_KEY_KP_DIVIDE,    { 0x00, 0x00, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // /
+  { BX_KEY_KP_UP,        { 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 8
+  { BX_KEY_KP_5,         { 0x00, 0x00, 0x5D, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 5
+  { BX_KEY_KP_DOWN,      { 0x00, 0x00, 0x5A, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 2
+  { BX_KEY_KP_MULTIPLY,  { 0x00, 0x00, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // *
+  { BX_KEY_KP_PAGE_UP,   { 0x00, 0x00, 0x61, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 9
+  { BX_KEY_KP_RIGHT,     { 0x00, 0x00, 0x5E, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 6
+  { BX_KEY_KP_PAGE_DOWN, { 0x00, 0x00, 0x5B, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // 3
+  { BX_KEY_KP_SUBTRACT,  { 0x00, 0x00, 0x56, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // -
+  { BX_KEY_KP_ADD,       { 0x00, 0x00, 0x57, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // +
+  { BX_KEY_KP_DELETE,    { 0x00, 0x00, 0x63, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // .
+  { BX_KEY_KP_ENTER,     { 0x00, 0x00, 0x58, 0x00, 0x00, 0x00, 0x00, 0x00 } },  // Enter
 };
 
 usb_hid_device_c::usb_hid_device_c(usbdev_type type)
 {
   d.type = type;
-  d.maxspeed = USB_SPEED_LOW;
-  d.speed = d.maxspeed;
+  d.minspeed = USB_SPEED_LOW;
+  d.maxspeed = USB_SPEED_HIGH;
+  d.speed = d.minspeed;
   if (d.type == USB_DEV_TYPE_MOUSE) {
     strcpy(d.devname, "USB Mouse");
+    if (d.speed == USB_SPEED_HIGH) {
+      d.dev_descriptor = bx_mouse_dev_descriptor2;
+      d.config_descriptor = bx_mouse_config_descriptor2;
+      d.device_desc_size = sizeof(bx_mouse_dev_descriptor2);
+      d.config_desc_size = sizeof(bx_mouse_config_descriptor2);
+    } else {
+      d.dev_descriptor = bx_mouse_dev_descriptor;
+      d.config_descriptor = bx_mouse_config_descriptor;
+      d.device_desc_size = sizeof(bx_mouse_dev_descriptor);
+      d.config_desc_size = sizeof(bx_mouse_config_descriptor);
+    }
     DEV_register_removable_mouse((void*)this, mouse_enq_static, mouse_enabled_changed);
   } else if (d.type == USB_DEV_TYPE_TABLET) {
     strcpy(d.devname, "USB Tablet");
+    if (d.speed == USB_SPEED_HIGH) {
+      d.dev_descriptor = bx_mouse_dev_descriptor2;
+      d.config_descriptor = bx_tablet_config_descriptor2;
+      d.device_desc_size = sizeof(bx_mouse_dev_descriptor2);
+      d.config_desc_size = sizeof(bx_tablet_config_descriptor2);
+    } else {
+      d.dev_descriptor = bx_mouse_dev_descriptor;
+      d.config_descriptor = bx_tablet_config_descriptor;
+      d.device_desc_size = sizeof(bx_mouse_dev_descriptor);
+      d.config_desc_size = sizeof(bx_tablet_config_descriptor);
+    }
     DEV_register_removable_mouse((void*)this, mouse_enq_static, mouse_enabled_changed);
     bx_gui->set_mouse_mode_absxy(1);
   } else if (d.type == USB_DEV_TYPE_KEYPAD) {
     strcpy(d.devname, "USB/PS2 Keypad");
-    DEV_register_removable_keyboard((void*)this, key_enq_static);
+    if (d.speed == USB_SPEED_HIGH) {
+      d.dev_descriptor = bx_keypad_dev_descriptor2;
+      d.config_descriptor = bx_keypad_config_descriptor2;
+      d.device_desc_size = sizeof(bx_keypad_dev_descriptor2);
+      d.config_desc_size = sizeof(bx_keypad_config_descriptor2);
+    } else {
+      d.dev_descriptor = bx_keypad_dev_descriptor;
+      d.config_descriptor = bx_keypad_config_descriptor;
+      d.device_desc_size = sizeof(bx_keypad_dev_descriptor);
+      d.config_desc_size = sizeof(bx_keypad_config_descriptor);
+    }
+    DEV_register_removable_keyboard((void*)this, gen_scancode_static);
+//    DEV_register_removable_mouse((void*)this, mouse_enq_static, mouse_enabled_changed);
   }
+  d.vendor_desc = "BOCHS";
+  d.product_desc = d.devname;
+  d.serial_num = "1";
   d.connected = 1;
   memset((void*)&s, 0, sizeof(s));
+  if (d.type == USB_DEV_TYPE_KEYPAD) {
+    s.saved_key = BX_KEY_UNHANDLED;
+  }
 
   put("usb_hid", "USBHID");
 }
 
 usb_hid_device_c::~usb_hid_device_c(void)
 {
+  d.sr->clear();
   if ((d.type == USB_DEV_TYPE_MOUSE) ||
       (d.type == USB_DEV_TYPE_TABLET)) {
     bx_gui->set_mouse_mode_absxy(0);
     DEV_unregister_removable_mouse((void*)this);
   } else if (d.type == USB_DEV_TYPE_KEYPAD) {
     DEV_unregister_removable_keyboard((void*)this);
+//    DEV_unregister_removable_mouse((void*)this);
   }
 }
 
 void usb_hid_device_c::register_state_specific(bx_list_c *parent)
 {
-  bx_list_c *key;
-  Bit8u i;
-  char name[6];
-
   bx_list_c *list = new bx_list_c(parent, "s", "USB HID Device State");
-  new bx_shadow_num_c(list, "mouse_delayed_dx", &s.mouse_delayed_dx);
-  new bx_shadow_num_c(list, "mouse_delayed_dy", &s.mouse_delayed_dy);
-  new bx_shadow_num_c(list, "mouse_delayed_dz", &s.mouse_delayed_dz);
-  new bx_shadow_num_c(list, "mouse_x", &s.mouse_x);
-  new bx_shadow_num_c(list, "mouse_y", &s.mouse_y);
-  new bx_shadow_num_c(list, "mouse_z", &s.mouse_z);
-  new bx_shadow_num_c(list, "b_state", &s.b_state, BASE_HEX);
-  key = new bx_list_c(list, "saved_key");
-  for (i=0; i<8; i++) {
-    sprintf(name, "0x%02x", i);
-    new bx_shadow_num_c(key, name, &s.saved_key[i], BASE_HEX);
-  }
-  key = new bx_list_c(list, "key_pad_packet");
-  for (i=0; i<8; i++) {
-    sprintf(name, "0x%02x", i);
-    new bx_shadow_num_c(key, name, &s.key_pad_packet[i], BASE_HEX);
+  BXRS_DEC_PARAM_FIELD(list, mouse_delayed_dx, s.mouse_delayed_dx);
+  BXRS_DEC_PARAM_FIELD(list, mouse_delayed_dy, s.mouse_delayed_dy);
+  BXRS_DEC_PARAM_FIELD(list, mouse_delayed_dz, s.mouse_delayed_dz);
+  BXRS_DEC_PARAM_FIELD(list, mouse_x, s.mouse_x);
+  BXRS_DEC_PARAM_FIELD(list, mouse_y, s.mouse_y);
+  BXRS_DEC_PARAM_FIELD(list, mouse_z, s.mouse_z);
+  BXRS_HEX_PARAM_FIELD(list, b_state, s.b_state);
+  BXRS_HEX_PARAM_FIELD(list, idle, s.idle);
+  BXRS_PARAM_BOOL(list, has_events, s.has_events);
+  if (d.type == USB_DEV_TYPE_KEYPAD) {
+    BXRS_DEC_PARAM_FIELD(list, saved_key, s.saved_key);
+    new bx_shadow_data_c(list, "key_pad_packet", s.key_pad_packet, 8, 1);
   }
 }
 
@@ -476,99 +722,25 @@ void usb_hid_device_c::handle_reset()
 
 int usb_hid_device_c::handle_control(int request, int value, int index, int length, Bit8u *data)
 {
-  int ret = 0;
+  int ret;
 
+  ret = handle_control_common(request, value, index, length, data);
+  if (ret >= 0) {
+    return ret;
+  }
+
+  ret = 0;
   switch(request) {
-    case DeviceRequest | USB_REQ_GET_STATUS:
-      if (d.state == USB_STATE_DEFAULT)
-        goto fail;
-      else {
-        data[0] = (1 << USB_DEVICE_SELF_POWERED) |
-          (d.remote_wakeup << USB_DEVICE_REMOTE_WAKEUP);
-        data[1] = 0x00;
-        ret = 2;
-      }
-      break;
     case DeviceOutRequest | USB_REQ_CLEAR_FEATURE:
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 0;
-      } else {
-        goto fail;
-      }
-      ret = 0;
+      goto fail;
       break;
     case DeviceOutRequest | USB_REQ_SET_FEATURE:
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 1;
-      } else {
-        goto fail;
-      }
-      ret = 0;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_ADDRESS:
-      d.state = USB_STATE_ADDRESS;
-      d.addr = value;
-      ret = 0;
+      goto fail;
       break;
     case DeviceRequest | USB_REQ_GET_DESCRIPTOR:
       switch(value >> 8) {
-        case USB_DT_DEVICE:
-          if ((d.type == USB_DEV_TYPE_MOUSE) ||
-              (d.type == USB_DEV_TYPE_TABLET)) {
-            memcpy(data, bx_mouse_dev_descriptor,
-                   sizeof(bx_mouse_dev_descriptor));
-            ret = sizeof(bx_mouse_dev_descriptor);
-          } else if (d.type == USB_DEV_TYPE_KEYPAD) {
-            memcpy(data, bx_keypad_dev_descriptor,
-                   sizeof(bx_keypad_dev_descriptor));
-            ret = sizeof(bx_keypad_dev_descriptor);
-          } else {
-            goto fail;
-          }
-          break;
-        case USB_DT_CONFIG:
-          if (d.type == USB_DEV_TYPE_MOUSE) {
-            memcpy(data, bx_mouse_config_descriptor,
-                   sizeof(bx_mouse_config_descriptor));
-            ret = sizeof(bx_mouse_config_descriptor);
-          } else if (d.type == USB_DEV_TYPE_TABLET) {
-            memcpy(data, bx_tablet_config_descriptor,
-                   sizeof(bx_tablet_config_descriptor));
-            ret = sizeof(bx_tablet_config_descriptor);
-          } else if (d.type == USB_DEV_TYPE_KEYPAD) {
-            memcpy(data, bx_keypad_config_descriptor,
-                   sizeof(bx_keypad_config_descriptor));
-            ret = sizeof(bx_keypad_config_descriptor);
-          } else {
-            goto fail;
-          }
-          break;
         case USB_DT_STRING:
           switch(value & 0xff) {
-            case 0:
-              /* language ids */
-              data[0] = 4;
-              data[1] = 3;
-              data[2] = 0x09;
-              data[3] = 0x04;
-              ret = 4;
-              break;
-            case 1:
-              /* serial number */
-              ret = set_usb_string(data, "1");
-              break;
-            case 2:
-              /* product description */
-              if (strlen(d.devname) > 0) {
-                ret = set_usb_string(data, d.devname);
-              } else {
-                goto fail;
-              }
-              break;
-            case 3:
-              /* vendor description */
-              ret = set_usb_string(data, "BOCHS");
-              break;
             case 4:
               ret = set_usb_string(data, "HID Mouse");
               break;
@@ -585,21 +757,6 @@ int usb_hid_device_c::handle_control(int request, int value, int index, int leng
           goto fail;
       }
       break;
-    case DeviceRequest | USB_REQ_GET_CONFIGURATION:
-      data[0] = 1;
-      ret = 1;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_CONFIGURATION:
-      d.state = USB_STATE_CONFIGURED;
-      ret = 0;
-      break;
-    case DeviceRequest | USB_REQ_GET_INTERFACE:
-      data[0] = 0;
-      ret = 1;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_INTERFACE:
-      ret = 0;
-      break;
       /* hid specific requests */
     case InterfaceRequest | USB_REQ_GET_DESCRIPTOR:
       switch(value >> 8) {
@@ -651,12 +808,21 @@ int usb_hid_device_c::handle_control(int request, int value, int index, int leng
           goto fail;
         }
         break;
+    case EndpointOutRequest | USB_REQ_CLEAR_FEATURE:
+      if ((value == 0) && (index != 0x81)) { /* clear EP halt */
+        goto fail;
+      }
+      break;
     case GET_REPORT:
       if ((d.type == USB_DEV_TYPE_MOUSE) ||
           (d.type == USB_DEV_TYPE_TABLET)) {
-        ret = mouse_poll(data, length);
+        ret = mouse_poll(data, length, 1);
       } else if (d.type == USB_DEV_TYPE_KEYPAD) {
-        ret = keypad_poll(data, length);
+        if (index == 0) {
+          ret = keypad_poll(data, length, 1);
+        } else {
+          ret = mouse_poll(data, length, 1);
+        }
       } else {
         goto fail;
       }
@@ -669,7 +835,14 @@ int usb_hid_device_c::handle_control(int request, int value, int index, int leng
         goto fail;
       }
       break;
+    case GET_IDLE:
+      data[0] = s.idle;
+      ret = 1;
+      break;
     case SET_IDLE:
+      s.idle = (value >> 8);
+      ret = 0;
+      break;
     case SET_PROTOCOL:
       ret = 0;
       break;
@@ -692,15 +865,15 @@ int usb_hid_device_c::handle_data(USBPacket *p)
       if (p->devep == 1) {
         if ((d.type == USB_DEV_TYPE_MOUSE) ||
             (d.type == USB_DEV_TYPE_TABLET)) {
-          ret = mouse_poll(p->data, p->len);
+          ret = mouse_poll(p->data, p->len, 0);
         } else if (d.type == USB_DEV_TYPE_KEYPAD) {
-          ret = keypad_poll(p->data, p->len);
+          ret = keypad_poll(p->data, p->len, 0);
         } else {
           goto fail;
         }
       } else if (p->devep == 2) {
         if (d.type == USB_DEV_TYPE_KEYPAD) {
-          ret = mouse_poll(p->data, p->len);
+          ret = mouse_poll(p->data, p->len, 0);
         } else {
           goto fail;
         }
@@ -719,36 +892,42 @@ int usb_hid_device_c::handle_data(USBPacket *p)
   return ret;
 }
 
-int usb_hid_device_c::mouse_poll(Bit8u *buf, int len)
+int usb_hid_device_c::mouse_poll(Bit8u *buf, int len, bx_bool force)
 {
-  int l = 0;
+  int l = USB_RET_NAK;
 
   if ((d.type == USB_DEV_TYPE_MOUSE) ||
       (d.type == USB_DEV_TYPE_KEYPAD)) {
-    if (!s.mouse_x && !s.mouse_y) {
+    if (!s.has_events) {
       // if there's no new movement, handle delayed one
       mouse_enq(0, 0, s.mouse_z, s.b_state, 0);
     }
-    buf[0] = (Bit8u) s.b_state;
-    buf[1] = (Bit8s) s.mouse_x;
-    buf[2] = (Bit8s) s.mouse_y;
-    s.mouse_x = 0;
-    s.mouse_y = 0;
-    l = 3;
-    if (len >= 4) {
-      buf[3] = (Bit8s) s.mouse_z; // if wheel mouse
-      s.mouse_z = 0;
-      l = 4;
+    if (s.has_events || (s.idle != 0) || force) {
+      buf[0] = (Bit8u) s.b_state;
+      buf[1] = (Bit8s) s.mouse_x;
+      buf[2] = (Bit8s) s.mouse_y;
+      s.mouse_x = 0;
+      s.mouse_y = 0;
+      l = 3;
+      if (len >= 4) {
+        buf[3] = (Bit8s) s.mouse_z; // if wheel mouse
+        s.mouse_z = 0;
+        l = 4;
+      }
+      s.has_events = 0;
     }
   } else if (d.type == USB_DEV_TYPE_TABLET) {
-    buf[0] = (Bit8u) s.b_state;
-    buf[1] = (Bit8u)(s.mouse_x & 0xff);
-    buf[2] = (Bit8u)(s.mouse_x >> 8);
-    buf[3] = (Bit8u)(s.mouse_y & 0xff);
-    buf[4] = (Bit8u)(s.mouse_y >> 8);
-    buf[5] = (Bit8s) s.mouse_z;
-    s.mouse_z = 0;
-    l = 6;
+    if (s.has_events || (s.idle != 0) || force) {
+      buf[0] = (Bit8u) s.b_state;
+      buf[1] = (Bit8u)(s.mouse_x & 0xff);
+      buf[2] = (Bit8u)(s.mouse_x >> 8);
+      buf[3] = (Bit8u)(s.mouse_y & 0xff);
+      buf[4] = (Bit8u)(s.mouse_y >> 8);
+      buf[5] = (Bit8s) s.mouse_z;
+      s.mouse_z = 0;
+      l = 6;
+      s.has_events = 0;
+    }
   }
   return l;
 }
@@ -765,6 +944,8 @@ void usb_hid_device_c::mouse_enq_static(void *dev, int delta_x, int delta_y, int
 
 void usb_hid_device_c::mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state, bx_bool absxy)
 {
+  Bit16s prev_x, prev_y;
+
   if (d.type == USB_DEV_TYPE_MOUSE) {
     // scale down the motion
     if ((delta_x < -1) || (delta_x > 1))
@@ -803,7 +984,12 @@ void usb_hid_device_c::mouse_enq(int delta_x, int delta_y, int delta_z, unsigned
 
     s.mouse_x = (Bit8s) delta_x;
     s.mouse_y = (Bit8s) delta_y;
+    if ((s.mouse_x != 0) || (s.mouse_y != 0) || (button_state != s.b_state)) {
+      s.has_events = 1;
+    }
   } else if (d.type == USB_DEV_TYPE_TABLET) {
+    prev_x = s.mouse_x;
+    prev_y = s.mouse_y;
     if (absxy) {
       s.mouse_x = delta_x;
       s.mouse_y = delta_y;
@@ -815,72 +1001,61 @@ void usb_hid_device_c::mouse_enq(int delta_x, int delta_y, int delta_z, unsigned
       s.mouse_x = 0;
     if (s.mouse_y < 0)
       s.mouse_y = 0;
+    if ((s.mouse_x != prev_x) || (s.mouse_y != prev_y) || (button_state != s.b_state)) {
+      s.has_events = 1;
+    }
   }
   s.mouse_z = (Bit8s) delta_z;
   s.b_state = (Bit8u) button_state;
 }
 
-int usb_hid_device_c::keypad_poll(Bit8u *buf, int len)
+int usb_hid_device_c::keypad_poll(Bit8u *buf, int len, bx_bool force)
 {
-  int l = 0;
+  int l = USB_RET_NAK;
 
   if (d.type == USB_DEV_TYPE_KEYPAD) {
-    memcpy(buf, s.key_pad_packet, len);
-    l = 8;
+    if (s.has_events || (s.idle != 0) || force) {
+      memcpy(buf, s.key_pad_packet, len);
+      l = 8;
+      s.has_events = 0;
+    }
   }
   return l;
 }
 
-bx_bool usb_hid_device_c::key_enq_static(void *dev, Bit8u *scan_code)
+bx_bool usb_hid_device_c::gen_scancode_static(void *dev, Bit32u key)
 {
-  return ((usb_hid_device_c*)dev)->key_enq(scan_code);
+  return ((usb_hid_device_c*)dev)->gen_scancode(key);
 }
 
-bx_bool usb_hid_device_c::key_enq(Bit8u *scan_code)
+bx_bool usb_hid_device_c::gen_scancode(Bit32u key)
 {
-  bx_bool is_break_code = 0;
-  Bit8u our_scan_code[8];
-
-  memset(our_scan_code, 0, 8);
-  int os = 0;
-  for (int sc=0; sc<8; sc++) {
-    if ((scan_code[sc] == 0xF0) && ((sc == 0) || ((sc == 1) && (scan_code[0] == 0xE0)))) {
-      is_break_code = 1;
-    } else {
-      if (!(our_scan_code[os++] = scan_code[sc])) break;
-    }
-  }
-
   // if it is the break code of the saved key, then clear our packet key.
-  if (is_break_code && !memcmp(s.saved_key, our_scan_code, 8)) {
-    memset(s.saved_key, 0, 8);
-    memset(s.key_pad_packet, 0, 8);
-    return 1; // tell the keyboard handler that we used it, and to return with out processing key
+  if (key & BX_KEY_RELEASED) {
+    key &= ~BX_KEY_RELEASED;
+    if (key == s.saved_key) {
+      s.saved_key = BX_KEY_UNHANDLED;
+      memset(s.key_pad_packet, 0, 8);
+      s.has_events = 1;
+      BX_DEBUG(("Routing Bochs key release (%d) to USB keypad", key));
+      return 1; // tell the keyboard handler that we used it, and to return with out processing key
+    }
+    return 0;
   }
 
   bx_bool fnd = 0;
-  for (int m = 0; m < 18; m++) {
-    if (!memcmp(keypad_lookup[m].scan_code, our_scan_code, 8)) {
+  for (int m = 0; m < KEYPAD_LEN; m++) {
+    if (key == keypad_lookup[m].bxkey) {
       memcpy(s.key_pad_packet, keypad_lookup[m].keypad_packet, 8);
       fnd = 1;
       break;
     }
   }
 
-  if (!fnd) {
-    memset(s.key_pad_packet, 0, 8);
-    memset(s.saved_key, 0, 8);
-  } else {
-    memcpy(s.saved_key, our_scan_code, 8);
-    // print a debug line to the log file
-    char bx_debug_code[128] = "";
-    char value[8];
-    for (unsigned i=0; i<strlen((char *) our_scan_code); i++) {
-      sprintf(value, "0x%02x", our_scan_code[i]);
-      if (i) strcat(bx_debug_code, "  ");
-      strcat(bx_debug_code, value);
-    }
-    BX_DEBUG(("Re-routing scan code (%s) to USB keypad", bx_debug_code));
+  if (fnd) {
+    s.saved_key = key;
+    s.has_events = 1;
+    BX_DEBUG(("Routing Bochs key press (%d) to USB keypad", key));
   }
 
   // tell the keyboard handler whether we used it or not.  (0 = no, 1 = yes and keyboard.cc ignores keystoke)
