@@ -1,12 +1,12 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_msd.cc 12494 2014-09-29 17:48:30Z vruppert $
+// $Id: usb_msd.cc 13138 2017-03-19 12:22:27Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  USB mass storage device support (ported from QEMU)
 //
 //  Copyright (c) 2006 CodeSourcery.
 //  Written by Paul Brook
-//  Copyright (C) 2009-2014  The Bochs Project
+//  Copyright (C) 2009-2016  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -67,7 +67,7 @@ struct usb_msd_csw {
 #define MassStorageReset  0xff
 #define GetMaxLun         0xfe
 
-// Low-, Full-, and High-speed
+// Full-speed
 static const Bit8u bx_msd_dev_descriptor[] = {
   0x12,       /*  u8 bLength; */
   0x01,       /*  u8 bDescriptorType; Device */
@@ -76,7 +76,7 @@ static const Bit8u bx_msd_dev_descriptor[] = {
   0x00,       /*  u8  bDeviceClass; */
   0x00,       /*  u8  bDeviceSubClass; */
   0x00,       /*  u8  bDeviceProtocol; [ low/full speeds only ] */
-  0x40,       /*  u8  bMaxPacketSize0; 64 Bytes */
+  0x08,       /*  u8  bMaxPacketSize0; 8 Bytes */
 
   /* Vendor and product id are arbitrary.  */
   0x00, 0x00, /*  u16 idVendor; */
@@ -89,7 +89,7 @@ static const Bit8u bx_msd_dev_descriptor[] = {
   0x01        /*  u8  bNumConfigurations; */
 };
 
-// Low-, Full-, and High-speed
+// Full-speed
 static const Bit8u bx_msd_config_descriptor[] = {
 
   /* one configuration */
@@ -122,7 +122,7 @@ static const Bit8u bx_msd_config_descriptor[] = {
   0x05,       /*  u8  ep_bDescriptorType; Endpoint */
   0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
   0x02,       /*  u8  ep_bmAttributes; Bulk */
-  0x40, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x40, 0x00, /*  u16 ep_wMaxPacketSize; 64 */
   0x00,       /*  u8  ep_bInterval; */
 
   /* Bulk-Out endpoint */
@@ -130,7 +130,74 @@ static const Bit8u bx_msd_config_descriptor[] = {
   0x05,       /*  u8  ep_bDescriptorType; Endpoint */
   0x02,       /*  u8  ep_bEndpointAddress; OUT Endpoint 2 */
   0x02,       /*  u8  ep_bmAttributes; Bulk */
-  0x40, 0x00, /*  u16 ep_wMaxPacketSize; */
+  0x40, 0x00, /*  u16 ep_wMaxPacketSize; 64 */
+  0x00        /*  u8  ep_bInterval; */
+};
+
+// High-speed
+static const Bit8u bx_msd_dev_descriptor2[] = {
+  0x12,       /*  u8 bLength; */
+  0x01,       /*  u8 bDescriptorType; Device */
+  0x00, 0x02, /*  u16 bcdUSB; v2.0 */
+
+  0x00,       /*  u8  bDeviceClass; */
+  0x00,       /*  u8  bDeviceSubClass; */
+  0x00,       /*  u8  bDeviceProtocol; [ low/full speeds only ] */
+  0x40,       /*  u8  bMaxPacketSize0; 64 Bytes */
+
+  /* Vendor and product id are arbitrary.  */
+  0x00, 0x00, /*  u16 idVendor; */
+  0x00, 0x00, /*  u16 idProduct; */
+  0x00, 0x01, /*  u16 bcdDevice */
+
+  0x01,       /*  u8  iManufacturer; */
+  0x02,       /*  u8  iProduct; */
+  0x03,       /*  u8  iSerialNumber; */
+  0x01        /*  u8  bNumConfigurations; */
+};
+
+// High-speed
+static const Bit8u bx_msd_config_descriptor2[] = {
+
+  /* one configuration */
+  0x09,       /*  u8  bLength; */
+  0x02,       /*  u8  bDescriptorType; Configuration */
+  0x20, 0x00, /*  u16 wTotalLength; */
+  0x01,       /*  u8  bNumInterfaces; (1) */
+  0x01,       /*  u8  bConfigurationValue; */
+  0x00,       /*  u8  iConfiguration; */
+  0xc0,       /*  u8  bmAttributes;
+                        Bit 7: must be set,
+                            6: Self-powered,
+                            5: Remote wakeup,
+                            4..0: resvd */
+  0x00,       /*  u8  MaxPower; */
+
+  /* one interface */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x00,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x02,       /*  u8  if_bNumEndpoints; */
+  0x08,       /*  u8  if_bInterfaceClass; MASS STORAGE */
+  0x06,       /*  u8  if_bInterfaceSubClass; SCSI */
+  0x50,       /*  u8  if_bInterfaceProtocol; Bulk Only */
+  0x00,       /*  u8  if_iInterface; */
+
+  /* Bulk-In endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
+  0x02,       /*  u8  ep_bmAttributes; Bulk */
+  0x00, 0x02, /*  u16 ep_wMaxPacketSize; 512 */
+  0x00,       /*  u8  ep_bInterval; */
+
+  /* Bulk-Out endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x02,       /*  u8  ep_bEndpointAddress; OUT Endpoint 2 */
+  0x02,       /*  u8  ep_bmAttributes; Bulk */
+  0x00, 0x02, /*  u16 ep_wMaxPacketSize; 512 */
   0x00        /*  u8  ep_bInterval; */
 };
 
@@ -187,7 +254,7 @@ static const Bit8u bx_msd_config_descriptor3[] = {
   0x05,       /*  u8  ep_bDescriptorType; Endpoint */
   0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
   0x02,       /*  u8  ep_bmAttributes; Bulk */
-  0x00, 0x04, /*  u16 ep_wMaxPacketSize; */
+  0x00, 0x04, /*  u16 ep_wMaxPacketSize; 1024 */
   0x00,       /*  u8  ep_bInterval; */
 
   /* Bulk-In companion descriptor */
@@ -202,7 +269,7 @@ static const Bit8u bx_msd_config_descriptor3[] = {
   0x05,       /*  u8  ep_bDescriptorType; Endpoint */
   0x02,       /*  u8  ep_bEndpointAddress; OUT Endpoint 2 */
   0x02,       /*  u8  ep_bmAttributes; Bulk */
-  0x00, 0x04, /*  u16 ep_wMaxPacketSize; */
+  0x00, 0x04, /*  u16 ep_wMaxPacketSize; 1024 */
   0x00,       /*  u8  ep_bInterval; */
 
   /* Bulk-Out companion descriptor */
@@ -239,6 +306,8 @@ static const Bit8u bx_msd_bos_descriptor3[] = {
   0x20, 0x00  /* u16  bss_wU2DevExitLat; */
 };
 
+void usb_msd_restore_handler(void *dev, bx_list_c *conf);
+
 static int usb_cdrom_count = 0;
 
 usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
@@ -251,7 +320,9 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
   bx_param_enum_c *status;
 
   d.type = type;
+  d.minspeed = USB_SPEED_FULL;
   d.maxspeed = USB_SPEED_SUPER;
+  d.speed = d.minspeed;
   memset((void*)&s, 0, sizeof(s));
   if (d.type == USB_DEV_TYPE_DISK) {
     strcpy(d.devname, "BOCHS USB HARDDRIVE");
@@ -279,7 +350,7 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
     s.config->set_device_param(this);
     path = new bx_param_string_c(s.config, "path", "Path", "", "", BX_PATHNAME_LEN);
     path->set(s.fname);
-    path->set_handler(cd_param_string_handler);
+    path->set_handler(cdrom_path_handler);
     status = new bx_param_enum_c(s.config,
       "status",
       "Status",
@@ -287,22 +358,26 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
       media_status_names,
       BX_INSERTED,
       BX_EJECTED);
-    status->set_handler(cd_param_handler);
+    status->set_handler(cdrom_status_handler);
     status->set_ask_format("Is the device inserted or ejected? [%s] ");
     if (SIM->is_wx_selected()) {
       bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
       usb->add(s.config);
     }
   }
+  d.vendor_desc = "BOCHS";
+  d.product_desc = d.devname;
 
   put("usb_msd", "USBMSD");
 }
 
 usb_msd_device_c::~usb_msd_device_c(void)
 {
+  d.sr->clear();
   if (s.scsi_dev != NULL)
     delete s.scsi_dev;
   if (s.hdimage != NULL) {
+    s.hdimage->close();
     delete s.hdimage;
   } else if (s.cdrom != NULL) {
     delete s.cdrom;
@@ -355,7 +430,7 @@ bx_bool usb_msd_device_c::init()
     s.hdimage = DEV_hdimage_init_image(s.image_mode, 0, s.journal);
     if (s.image_mode == BX_HDIMAGE_MODE_VVFAT) {
       Bit64u hdsize = ((Bit64u)s.size) << 20;
-      s.hdimage->cylinders = (Bit64u)(hdsize/16.0/63.0/512.0);
+      s.hdimage->cylinders = (unsigned)(hdsize/16.0/63.0/512.0);
       s.hdimage->heads = 16;
       s.hdimage->spt = 63;
     }
@@ -376,8 +451,29 @@ bx_bool usb_msd_device_c::init()
     }
   }
   s.scsi_dev->register_state(s.sr_list, "scsidev");
+  if (getonoff(LOGLEV_DEBUG) == ACT_REPORT) {
+    s.scsi_dev->set_debug_mode();
+  }
+  if (get_speed() == USB_SPEED_SUPER) {
+    d.dev_descriptor = bx_msd_dev_descriptor3;
+    d.config_descriptor = bx_msd_config_descriptor3;
+    d.device_desc_size = sizeof(bx_msd_dev_descriptor3);
+    d.config_desc_size = sizeof(bx_msd_config_descriptor3);
+  } else if (get_speed() == USB_SPEED_HIGH) {
+    d.dev_descriptor = bx_msd_dev_descriptor2;
+    d.config_descriptor = bx_msd_config_descriptor2;
+    d.device_desc_size = sizeof(bx_msd_dev_descriptor2);
+    d.config_desc_size = sizeof(bx_msd_config_descriptor2);
+  } else {
+    d.dev_descriptor = bx_msd_dev_descriptor;
+    d.config_descriptor = bx_msd_config_descriptor;
+    d.device_desc_size = sizeof(bx_msd_dev_descriptor);
+    d.config_desc_size = sizeof(bx_msd_config_descriptor);
+  }
+  d.serial_num = s.scsi_dev->get_serial_number();
   s.mode = USB_MSDM_CBW;
   d.connected = 1;
+  s.status_changed = 0;
   return 1;
 }
 
@@ -389,16 +485,21 @@ const char* usb_msd_device_c::get_info()
 void usb_msd_device_c::register_state_specific(bx_list_c *parent)
 {
   s.sr_list = new bx_list_c(parent, "s", "USB MSD Device State");
-  if ((d.type == USB_DEV_TYPE_DISK) && (s.hdimage != NULL)) {
+  if (d.type == USB_DEV_TYPE_CDROM) {
+    bx_list_c *rt_config = new bx_list_c(s.sr_list, "rt_config");
+    rt_config->add(s.config->get_by_name("path"));
+    rt_config->add(s.config->get_by_name("status"));
+    rt_config->set_restore_handler(this, usb_msd_restore_handler);
+  } else if ((d.type == USB_DEV_TYPE_DISK) && (s.hdimage != NULL)) {
     s.hdimage->register_state(s.sr_list);
   }
-  new bx_shadow_num_c(s.sr_list, "mode", &s.mode);
-  new bx_shadow_num_c(s.sr_list, "scsi_len", &s.scsi_len);
-  new bx_shadow_num_c(s.sr_list, "usb_len", &s.usb_len);
-  new bx_shadow_num_c(s.sr_list, "data_len", &s.data_len);
-  new bx_shadow_num_c(s.sr_list, "residue", &s.residue);
-  new bx_shadow_num_c(s.sr_list, "tag", &s.tag);
-  new bx_shadow_num_c(s.sr_list, "result", &s.result);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, mode, s.mode);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, scsi_len, s.scsi_len);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, usb_len, s.usb_len);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, data_len, s.data_len);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, residue, s.residue);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, tag, s.tag);
+  BXRS_DEC_PARAM_FIELD(s.sr_list, result, s.result);
 }
 
 void usb_msd_device_c::handle_reset()
@@ -411,94 +512,35 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
 {
   int ret = 0;
 
+  ret = handle_control_common(request, value, index, length, data);
+  if (ret >= 0) {
+    return ret;
+  }
+
+  ret = 0;
   switch (request) {
-    case DeviceRequest | USB_REQ_GET_STATUS:
-    case EndpointRequest | USB_REQ_GET_STATUS:
-      BX_DEBUG(("USB_REQ_GET_STATUS:"));
-      data[0] = (1 << USB_DEVICE_SELF_POWERED) |
-        (d.remote_wakeup << USB_DEVICE_REMOTE_WAKEUP);
-      data[1] = 0x00;
-      ret = 2;
-      break;
     case DeviceOutRequest | USB_REQ_CLEAR_FEATURE:
-      BX_DEBUG(("USB_REQ_CLEAR_FEATURE:"));
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 0;
-      } else {
-        BX_DEBUG(("USB_REQ_CLEAR_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
-        goto fail;
-      }
-      ret = 0;
+      BX_DEBUG(("USB_REQ_CLEAR_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
+      goto fail;
       break;
     case DeviceOutRequest | USB_REQ_SET_FEATURE:
       BX_DEBUG(("USB_REQ_SET_FEATURE:"));
       switch (value) {
         case USB_DEVICE_REMOTE_WAKEUP:
-          d.remote_wakeup = 1;
-          break;
         case USB_DEVICE_U1_ENABLE:
         case USB_DEVICE_U2_ENABLE:
           break;
         default:
-        BX_DEBUG(("USB_REQ_SET_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
-        goto fail;
+          BX_DEBUG(("USB_REQ_SET_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
+          goto fail;
       }
       ret = 0;
       break;
-    case DeviceOutRequest | USB_REQ_SET_ADDRESS:
-      BX_DEBUG(("USB_REQ_SET_ADDRESS:"));
-      d.addr = value;
-      ret = 0;
-      break;
     case DeviceRequest | USB_REQ_GET_DESCRIPTOR:
       switch (value >> 8) {
-        case USB_DT_DEVICE:
-          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Device"));
-          if (get_speed() == USB_SPEED_SUPER) {
-            memcpy(data, bx_msd_dev_descriptor3, sizeof(bx_msd_dev_descriptor3));
-            ret = sizeof(bx_msd_dev_descriptor3);
-          } else {
-            memcpy(data, bx_msd_dev_descriptor, sizeof(bx_msd_dev_descriptor));
-            ret = sizeof(bx_msd_dev_descriptor);
-          }
-          break;
-        case USB_DT_CONFIG:
-          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Config"));
-          if (get_speed() == USB_SPEED_SUPER) {
-            memcpy(data, bx_msd_config_descriptor3, sizeof(bx_msd_config_descriptor3));
-            ret = sizeof(bx_msd_config_descriptor3);
-          } else {
-            memcpy(data, bx_msd_config_descriptor, sizeof(bx_msd_config_descriptor));
-            ret = sizeof(bx_msd_config_descriptor);
-          }
-          break;
         case USB_DT_STRING:
           BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: String"));
           switch(value & 0xff) {
-            case 0:
-              // language IDs
-              data[0] = 4;
-              data[1] = 3;
-              data[2] = 0x09;
-              data[3] = 0x04;
-              ret = 4;
-              break;
-            case 1:
-              // vendor description
-              ret = set_usb_string(data, "BOCHS");
-              break;
-            case 2:
-              // product description
-              if (strlen(d.devname) > 0) {
-                ret = set_usb_string(data, d.devname);
-              } else {
-                goto fail;
-              }
-              break;
-            case 3:
-              // serial number
-              ret = set_usb_string(data, s.scsi_dev->get_serial_number());
-              break;
             case 0xEE:
               // Microsoft OS Descriptor check
               // We don't support this check, so fail
@@ -511,12 +553,19 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
         case USB_DT_DEVICE_QUALIFIER:
           BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Device Qualifier"));
           // device qualifier
-          data[0] = 10;
-          data[1] = USB_DT_DEVICE_QUALIFIER;
-          memcpy(data+2, bx_msd_dev_descriptor+2, 6);
-          data[8] = 1;
-          data[9] = 0;
-          ret = 10;
+          if (d.speed >= USB_SPEED_HIGH) {
+            data[0] = 10;
+            data[1] = USB_DT_DEVICE_QUALIFIER;
+            memcpy(data+2, bx_msd_dev_descriptor+2, 6);
+            data[8] = 1;
+            data[9] = 0;
+            ret = 10;
+          } else {
+            // a low- or full-speed only device (i.e.: a non high-speed device) must return
+            //  request error on this function
+            BX_ERROR(("USB MSD handle_control: full-speed only device returning stall on Device Qualifier."));
+            goto fail;
+          }
           break;
         case USB_DT_BIN_DEV_OBJ_STORE:
           BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: BOS"));
@@ -531,25 +580,6 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
           goto fail;
       }
       break;
-    case DeviceRequest | USB_REQ_GET_CONFIGURATION:
-      BX_DEBUG(("USB_REQ_GET_CONFIGURATION:"));
-      data[0] = 1;
-      ret = 1;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_CONFIGURATION:
-      BX_DEBUG(("USB_REQ_SET_CONFIGURATION:"));
-      ret = 0;
-      break;
-    case DeviceRequest | USB_REQ_GET_INTERFACE:
-      BX_DEBUG(("USB_REQ_GET_INFTERFACE:"));
-      data[0] = 0;
-      ret = 1;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_INTERFACE:
-    case InterfaceOutRequest | USB_REQ_SET_INTERFACE:
-      BX_DEBUG(("USB_REQ_SET_INFTERFACE:"));
-      ret = 0;
-      break;
     case EndpointOutRequest | USB_REQ_CLEAR_FEATURE:
       BX_DEBUG(("USB_REQ_CLEAR_FEATURE:"));
       if (value == 0 && index != 0x81) { /* clear ep halt */
@@ -623,7 +653,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           BX_DEBUG(("command tag 0x%X flags %08X len %d data %d",
                    s.tag, cbw.flags, cbw.cmd_len, s.data_len));
           s.residue = 0;
-          s.scsi_dev->scsi_send_command(s.tag, cbw.cmd, cbw.lun);
+          s.scsi_dev->scsi_send_command(s.tag, cbw.cmd, cbw.lun, d.async_mode);
           if (s.residue == 0) {
             if (s.mode == USB_MSDM_DATAIN) {
               s.scsi_dev->scsi_read_data(s.tag);
@@ -651,7 +681,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
             s.usb_len = 0;
           }
           if (s.usb_len) {
-            BX_INFO(("deferring packet %p", p));
+            BX_DEBUG(("deferring packet %p", p));
             usb_defer_packet(p, this);
             s.packet = p;
             ret = USB_RET_ASYNC;
@@ -672,15 +702,9 @@ int usb_msd_device_c::handle_data(USBPacket *p)
 
       switch (s.mode) {
         case USB_MSDM_DATAOUT:
-          if ((s.data_len > 0) && (len == 13)) {
-            s.usb_len = len;
-            s.usb_buf = data;
-            send_status();
-            ret = 13;
-            break;
-          }
           if (s.data_len != 0 || len < 13)
             goto fail;
+          BX_DEBUG(("deferring packet %p", p));
           usb_defer_packet(p, this);
           s.packet = p;
           ret = USB_RET_ASYNC;
@@ -692,9 +716,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           if (len < 13)
             return ret;
 
-          s.usb_len = len;
-          s.usb_buf = data;
-          send_status();
+          send_status(p);
           s.mode = USB_MSDM_CBW;
           ret = 13;
           break;
@@ -716,7 +738,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
             s.usb_len = 0;
           }
           if (s.usb_len) {
-            BX_INFO(("deferring packet %p", p));
+            BX_DEBUG(("deferring packet %p", p));
             usb_defer_packet(p, this);
             s.packet = p;
             ret = USB_RET_ASYNC;
@@ -767,15 +789,15 @@ void usb_msd_device_c::copy_data()
   }
 }
 
-void usb_msd_device_c::send_status()
+void usb_msd_device_c::send_status(USBPacket *p)
 {
   struct usb_msd_csw csw;
 
   csw.sig = htod32(0x53425355);
   csw.tag = htod32(s.tag);
-  csw.residue = s.residue;
+  csw.residue = htod32(s.residue);
   csw.status = s.result;
-  memcpy(s.usb_buf, &csw, 13);
+  memcpy(p->data, &csw, BX_MIN(p->len, 13));
 }
 
 void usb_msd_device_c::usb_msd_command_complete(void *this_ptr, int reason, Bit32u tag, Bit32u arg)
@@ -796,8 +818,12 @@ void usb_msd_device_c::command_complete(int reason, Bit32u tag, Bit32u arg)
     s.residue = s.data_len;
     s.result = arg != 0;
     if (s.packet) {
-      if (s.data_len == 0 && s.mode == USB_MSDM_DATAOUT) {
-        send_status();
+      if ((s.data_len == 0) && (s.mode == USB_MSDM_DATAOUT)) {
+        send_status(p);
+        s.mode = USB_MSDM_CBW;
+        usb_dump_packet(p->data, p->len);
+      } else if (s.mode == USB_MSDM_CSW) {
+        send_status(p);
         s.mode = USB_MSDM_CBW;
       } else {
         if (s.data_len) {
@@ -810,6 +836,7 @@ void usb_msd_device_c::command_complete(int reason, Bit32u tag, Bit32u arg)
           s.mode = USB_MSDM_CSW;
       }
       s.packet = NULL;
+      usb_packet_complete(p);
     } else if (s.data_len == 0) {
       s.mode = USB_MSDM_CSW;
     }
@@ -818,10 +845,16 @@ void usb_msd_device_c::command_complete(int reason, Bit32u tag, Bit32u arg)
   s.scsi_len = arg;
   s.scsi_buf = s.scsi_dev->scsi_get_buf(tag);
   if (p) {
+    if ((s.scsi_len > 0) && (s.mode == USB_MSDM_DATAIN)) {
+      usb_dump_packet(s.scsi_buf, p->len);
+    }
     copy_data();
     if (s.usb_len == 0) {
-      BX_INFO(("packet complete %p", p));
-      s.packet = NULL;
+      BX_DEBUG(("packet complete %p", p));
+      if (s.packet != NULL) {
+        s.packet = NULL;
+        usb_packet_complete(p);
+      }
     }
   }
 }
@@ -839,9 +872,16 @@ bx_bool usb_msd_device_c::set_inserted(bx_bool value)
 
   if (value) {
     path = SIM->get_param_string("path", s.config)->getptr();
-    if (!s.cdrom->insert_cdrom(path)) {
+    if ((strlen(path) > 0) && (strcmp(path, "none"))) {
+      if (!s.cdrom->insert_cdrom(path)) {
+        value = 0;
+      }
+    } else {
+      value = 0;
+    }
+    if (!value) {
       SIM->get_param_enum("status", s.config)->set(BX_EJECTED);
-      return 0;
+      s.status_changed = 0;
     }
   } else {
     s.cdrom->eject_cdrom();
@@ -855,52 +895,80 @@ bx_bool usb_msd_device_c::get_inserted()
   return s.scsi_dev->get_inserted();
 }
 
+bx_bool usb_msd_device_c::get_locked()
+{
+  return s.scsi_dev->get_locked();
+}
+
+void usb_msd_device_c::runtime_config(void)
+{
+  if (d.type == USB_DEV_TYPE_CDROM) {
+    if (s.status_changed) {
+      set_inserted(0);
+      if (SIM->get_param_enum("status", s.config)->get() == BX_INSERTED) {
+        set_inserted(1);
+      }
+      s.status_changed = 0;
+    }
+  }
+}
+
 #undef LOG_THIS
 #define LOG_THIS cdrom->
 
-// USB hub runtime parameter handlers
-const char *usb_msd_device_c::cd_param_string_handler(bx_param_string_c *param, int set,
-                                                      const char *oldval, const char *val, int maxlen)
+// USB cdrom runtime parameter handlers
+const char *usb_msd_device_c::cdrom_path_handler(bx_param_string_c *param, int set,
+                                                 const char *oldval, const char *val, int maxlen)
 {
   usb_msd_device_c *cdrom;
 
   if (set) {
+    if (strlen(val) < 1) {
+      val = "none";
+    }
     cdrom = (usb_msd_device_c*) param->get_parent()->get_device_param();
     if (cdrom != NULL) {
-      bx_bool empty = ((strlen(val) == 0) || (!strcmp(val, "none")));
-      if (!empty) {
-        if (cdrom->get_inserted()) {
-          BX_ERROR(("direct path change not supported (setting to 'none')"));
-          param->set("none");
-        }
+      if (!cdrom->get_locked()) {
+        cdrom->s.status_changed = 1;
       } else {
-        SIM->get_param_enum("status", param->get_parent())->set(BX_EJECTED);
+        val = oldval;
+        BX_ERROR(("cdrom tray locked: path change failed"));
       }
     } else {
-      BX_PANIC(("cd_param_string_handler: cdrom not found"));
+      BX_PANIC(("cdrom_path_handler: cdrom not found"));
     }
   }
   return val;
 }
 
-Bit64s usb_msd_device_c::cd_param_handler(bx_param_c *param, int set, Bit64s val)
+Bit64s usb_msd_device_c::cdrom_status_handler(bx_param_c *param, int set, Bit64s val)
 {
   usb_msd_device_c *cdrom;
-  const char *path;
 
   if (set) {
     cdrom = (usb_msd_device_c*) param->get_parent()->get_device_param();
     if (cdrom != NULL) {
-      path = SIM->get_param_string("path", param->get_parent())->getptr();
-      val &= ((strlen(path) > 0) && (strcmp(path, "none")));
-      if (val != cdrom->get_inserted()) {
-        cdrom->set_inserted(val == BX_INSERTED);
+      if ((val == 1) || !cdrom->get_locked()) {
+        cdrom->s.status_changed = 1;
+      } else if (cdrom->get_locked()) {
+        BX_ERROR(("cdrom tray locked: eject failed"));
+        return BX_INSERTED;
       }
     } else {
-      BX_PANIC(("cd_param_string_handler: cdrom not found"));
+      BX_PANIC(("cdrom_status_handler: cdrom not found"));
     }
   }
   return val;
 }
 
+void usb_msd_restore_handler(void *dev, bx_list_c *conf)
+{
+  ((usb_msd_device_c*)dev)->restore_handler(conf);
+}
+
+void usb_msd_device_c::restore_handler(bx_list_c *conf)
+{
+  runtime_config();
+}
+
 #endif // BX_SUPPORT_PCI && BX_SUPPORT_PCIUSB
