@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_printer.cc 12488 2014-09-22 19:49:39Z vruppert $
+// $Id: usb_printer.cc 13054 2017-01-29 08:48:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009       Benjamin D Lunt (fys at frontiernet net)
-//                2009-2014  The Bochs Project
+//  Copyright (C) 2009-2016  Benjamin D Lunt (fys [at] fysnet [dot] net)
+//                2009-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -110,24 +110,56 @@ static const Bit8u bx_device_id_string[] =
   "VP:0800,FL,B0;"
   "VJ: ;";
 
+static int usb_printer_count = 0;
+
 usb_printer_device_c::usb_printer_device_c(usbdev_type type, const char *filename)
 {
+  char pname[10];
+  char label[32];
+  bx_param_string_c *fname;
+
   d.type = type;
-  d.maxspeed = USB_SPEED_FULL;
-  d.speed = d.maxspeed;
+  d.speed = d.minspeed = d.maxspeed = USB_SPEED_FULL;
   memset((void*)&s, 0, sizeof(s));
   strcpy(d.devname, "USB Printer");
+  d.dev_descriptor = bx_printer_dev_descriptor;
+  d.config_descriptor = bx_printer_config_descriptor;
+  d.device_desc_size = sizeof(bx_printer_dev_descriptor);
+  d.config_desc_size = sizeof(bx_printer_config_descriptor);
+  d.vendor_desc = "Hewlett-Packard";
+  d.product_desc = "Deskjet 920C";
+  d.serial_num = "HU18L6P2DNBI";
   s.fname = filename;
   s.fp = NULL;
-
+  // config options
+  bx_list_c *usb_rt = (bx_list_c*)SIM->get_param(BXPN_MENU_RUNTIME_USB);
+  sprintf(pname, "printer%d", ++usb_printer_count);
+  sprintf(label, "USB Printer #%d Configuration", usb_printer_count);
+  s.config = new bx_list_c(usb_rt, pname, label);
+  s.config->set_options(bx_list_c::SHOW_PARENT | bx_list_c::USE_BOX_TITLE);
+  s.config->set_device_param(this);
+  fname = new bx_param_filename_c(s.config, "file", "File", "", "", BX_PATHNAME_LEN);
+  fname->set(s.fname);
+  fname->set_handler(printfile_handler);
+  if (SIM->is_wx_selected()) {
+    bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
+    usb->add(s.config);
+  }
   put("usb_printer", "USBPRN");
 }
 
 usb_printer_device_c::~usb_printer_device_c(void)
 {
+  d.sr->clear();
   if (s.fp != NULL) {
     fclose(s.fp);
   }
+  if (SIM->is_wx_selected()) {
+    bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
+    usb->remove(s.config->get_name());
+  }
+  bx_list_c *usb_rt = (bx_list_c*)SIM->get_param(BXPN_MENU_RUNTIME_USB);
+  usb_rt->remove(s.config->get_name());
 }
 
 bx_bool usb_printer_device_c::init()
@@ -151,12 +183,11 @@ const char* usb_printer_device_c::get_info()
 void usb_printer_device_c::register_state_specific(bx_list_c *parent)
 {
   bx_list_c *list = new bx_list_c(parent, "s", "USB PRINTER Device State");
-  new bx_shadow_num_c(list, "printer_status", &s.printer_status);
+  BXRS_HEX_PARAM_FIELD(list, printer_status, s.printer_status);
 }
 
 void usb_printer_device_c::handle_reset()
 {
-  BX_INFO(("Opened %s for USB HP Deskjet 920C printer emulation.", s.fname));
   BX_DEBUG(("Reset"));
 }
 
@@ -165,95 +196,31 @@ int usb_printer_device_c::handle_control(int request, int value, int index, int
   int ret = 0;
 
   BX_DEBUG(("Printer: request: 0x%04X  value: 0x%04X  index: 0x%04X  len: %i", request, value, index, length));
+
+  ret = handle_control_common(request, value, index, length, data);
+  if (ret >= 0) {
+    return ret;
+  }
+
+  ret = 0;
   switch(request) {
-    case DeviceRequest | USB_REQ_GET_STATUS:
-      if (d.state == USB_STATE_DEFAULT)
-        goto fail;
-      else {
-        data[0] = (1 << USB_DEVICE_SELF_POWERED) |
-          (d.remote_wakeup << USB_DEVICE_REMOTE_WAKEUP);
-        data[1] = 0x00;
-        ret = 2;
-      }
-      break;
     case DeviceOutRequest | USB_REQ_CLEAR_FEATURE:
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 0;
-      } else {
-        goto fail;
-      }
-      ret = 0;
+      goto fail;
       break;
     case DeviceOutRequest | USB_REQ_SET_FEATURE:
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 1;
-      } else {
-        goto fail;
-      }
-      ret = 0;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_ADDRESS:
-      d.state = USB_STATE_ADDRESS;
-      d.addr = value;
-      ret = 0;
+      goto fail;
       break;
     case DeviceRequest | USB_REQ_GET_DESCRIPTOR:
       switch(value >> 8) {
-        case USB_DT_DEVICE:
-          memcpy(data, bx_printer_dev_descriptor, sizeof(bx_printer_dev_descriptor));
-          ret = sizeof(bx_printer_dev_descriptor);
-          break;
-        case USB_DT_CONFIG:
-          memcpy(data, bx_printer_config_descriptor, sizeof(bx_printer_config_descriptor));
-          ret = sizeof(bx_printer_config_descriptor);
-          break;
         case USB_DT_STRING:
-          switch(value & 0xff) {
-            case 0:
-              /* language ids */
-              data[0] = 4;
-              data[1] = 3;
-              data[2] = 0x09;
-              data[3] = 0x04;
-              ret = 4;
-              break;
-            case 1:
-              /* vendor description */
-              ret = set_usb_string(data, "Hewlett-Packard");
-              break;
-            case 2:
-              /* product description */
-              ret = set_usb_string(data, "Deskjet 920C");
-              break;
-            case 3:
-              /* serial number */
-              ret = set_usb_string(data, "HU18L6P2DNBI");
-              break;
-            default:
-              BX_ERROR(("USB Printer handle_control: unknown string descriptor 0x%02x", value & 0xff));
-              goto fail;
-          }
+          BX_ERROR(("USB Printer handle_control: unknown string descriptor 0x%02x", value & 0xff));
+          goto fail;
           break;
         default:
           BX_ERROR(("USB Printer handle_control: unknown descriptor type 0x%02x", value >> 8));
           goto fail;
       }
       break;
-    case DeviceRequest | USB_REQ_GET_CONFIGURATION:
-      data[0] = 1;
-      ret = 1;
-      break;
-    case DeviceOutRequest | USB_REQ_SET_CONFIGURATION:
-      d.state = USB_STATE_CONFIGURED;
-      ret = 0;
-      break;
-    case DeviceRequest | USB_REQ_GET_INTERFACE:
-      data[0] = 0;
-      ret = 1;
-      break;
-    case EndpointOutRequest | USB_REQ_SET_INTERFACE:
-      ret = 0;
-      break;
 
     /* printer specific requests */
     case InterfaceInClassRequest | 0x00:  // 1284 get device id string
@@ -299,7 +266,9 @@ int usb_printer_device_c::handle_data(USBPacket *p)
       if (p->devep == 2) {
         BX_DEBUG(("Sent %i bytes to the 'usb printer': %s", p->len, s.fname));
         usb_dump_packet(p->data, p->len);
-        fwrite(p->data, 1, p->len, s.fp);
+        if (s.fp != NULL) {
+          fwrite(p->data, 1, p->len, s.fp);
+        }
         ret = p->len;
       } else {
         goto fail;
@@ -314,4 +283,33 @@ fail:
   return ret;
 }
 
+#undef LOG_THIS
+#define LOG_THIS printer->
+
+// USB printer runtime parameter handlers
+const char *usb_printer_device_c::printfile_handler(bx_param_string_c *param, int set,
+                                                    const char *oldval, const char *val,
+                                                    int maxlen)
+{
+  usb_printer_device_c *printer;
+
+  if (set) {
+    if (strlen(val) < 1) {
+      val = "none";
+    }
+    printer = (usb_printer_device_c*) param->get_parent()->get_device_param();
+    if (printer != NULL) {
+      if (printer->s.fp != NULL) {
+        fclose(printer->s.fp);
+      }
+      printer->s.fp = fopen(val, "w+b");
+      if (printer->s.fp == NULL) {
+        BX_ERROR(("Could not create/open %s", val));
+      }
+    } else {
+      BX_PANIC(("printfile_handler: printer not found"));
+    }
+  }
+  return val;
+}
 #endif // BX_SUPPORT_PCI && BX_SUPPORT_PCIUSB
