@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_xhci.h 12493 2014-09-28 14:21:22Z vruppert $
+// $Id: usb_xhci.h 13150 2017-03-26 08:09:28Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2010-2014  Benjamin D Lunt (fys [at] fysnet [dot] net)
-//                2011-2014  The Bochs Project
+//  Copyright (C) 2010-2016  Benjamin D Lunt (fys [at] fysnet [dot] net)
+//                2011-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -147,13 +147,7 @@
 //  ie.: Each physical port (socket) has two defined port register sets.  One for USB3, one for USB2
 // Only one port type may be used at a time.  Port0 or Port1, not both.  If Port0 is used, then
 //  Port1 must be vacant.
-#ifndef BX_N_USB_XHCI_PORTS
-  #error "BX_N_USB_XHCI_PORTS was not defined in bochs.h"
-#else
-  #if (BX_N_USB_XHCI_PORTS < USB_XHCI_PORTS)
-    #error "BX_N_USB_XHCI_PORTS is less than USB_XHCI_PORTS."
-  #endif
-#endif
+#define BX_N_USB_XHCI_PORTS 4
 
 // xHCI speed values
 #define SPEED_FULL   1
@@ -224,9 +218,11 @@ struct HC_SLOT_CONTEXT {
   struct {
     struct EP_CONTEXT   ep_context;
     // our internal registers follow
-    Bit32u edtla;
-    Bit64u enqueue_pointer;
-    bx_bool  rcs;
+    Bit32u  edtla;
+    Bit64u  enqueue_pointer;
+    bx_bool rcs;
+    bx_bool retry;
+    int     retry_counter;
   } ep_context[32];  // first one is ignored by controller.
 };
 
@@ -272,7 +268,7 @@ enum { PLS_U0 = 0, PLS_U1, PLS_U2, PLS_U3_SUSPENDED, PLS_DISABLED, PLS_RXDETECT,
 // Slot State
 #define SLOT_STATE_DISABLED_ENABLED  0
 #define SLOT_STATE_DEFAULT           1
-#define SLOT_STATE_ADRESSED          2
+#define SLOT_STATE_ADDRESSED         2
 #define SLOT_STATE_CONFIGURED        3
 
 // EP State
@@ -522,11 +518,6 @@ typedef struct {
   struct HC_SLOT_CONTEXT slots[MAX_SLOTS];  // first one is ignored by controller.
 
   struct RING_MEMBERS ring_members;
-
-  Bit8u devfunc;
-
-  int statusbar_id; // ID of the status LEDs
-  Bit8u device_change;
 } bx_usb_xhci_t;
 
 // Version 3.0.23.0 of the Renesas uPD720202 driver, even though the card is
@@ -541,7 +532,7 @@ typedef struct {
   #error "ERSTABADD_MASK not defined"
 #endif
 
-class bx_usb_xhci_c : public bx_devmodel_c, public bx_pci_device_stub_c {
+class bx_usb_xhci_c : public bx_pci_device_c {
 public:
   bx_usb_xhci_c();
   virtual ~bx_usb_xhci_c();
@@ -549,15 +540,21 @@ public:
   virtual void reset(unsigned);
   virtual void register_state(void);
   virtual void after_restore_state(void);
-  virtual Bit32u  pci_read_handler(Bit8u address, unsigned io_len);
-  virtual void    pci_write_handler(Bit8u address, Bit32u value, unsigned io_len);
+
+  virtual void pci_write_handler(Bit8u address, Bit32u value, unsigned io_len);
+
+  void event_handler(int event, USBPacket *packet, int port);
 
   static const char *usb_param_handler(bx_param_string_c *param, int set,
                                        const char *oldval, const char *val, int maxlen);
 
 private:
   bx_usb_xhci_t hub;
-  Bit8u         *device_buffer;
+  Bit8u         devfunc;
+  Bit8u         device_change;
+  int           rt_conf_id;
+  int           xhci_timer_index;
+  USBAsync      *packets;
 
   static void reset_hc();
   static void reset_port(int);
@@ -571,8 +568,8 @@ private:
   static void usb_set_connect_status(Bit8u port, int type, bx_bool connected);
 
   static int  broadcast_packet(USBPacket *p, const int port);
-  //static void usb_frame_handler(void *);
-  //void usb_frame_timer(void);
+  static void xhci_timer_handler(void *);
+  void xhci_timer(void);
 
   static void process_transfer_ring(const int slot, const int ep);
   static void process_command_ring(void);
@@ -588,8 +585,8 @@ private:
   static void dump_ep_context(const Bit32u *context, const int slot, const int ep);
   static void copy_slot_from_buffer(struct SLOT_CONTEXT *slot_context, const Bit8u *buffer);
   static void copy_ep_from_buffer(struct EP_CONTEXT *ep_context, const Bit8u *buffer);
-  static void copy_slot_to_buffer(const Bit8u *buffer, const int slot);
-  static void copy_ep_to_buffer(const Bit8u *buffer, const int slot, const int ep);
+  static void copy_slot_to_buffer(Bit32u *buffer, const int slot);
+  static void copy_ep_to_buffer(Bit32u *buffer, const int slot, const int ep);
   static bx_bool validate_slot_context(const struct SLOT_CONTEXT *slot_context);
   static bx_bool validate_ep_context(const struct EP_CONTEXT *ep_context, int speed, int ep_num);
   static int  create_unique_address(const int slot);
