@@ -1,14 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: load32bitOShack.cc 11529 2012-11-02 07:46:50Z sshwarts $
+// $Id: load32bitOShack.cc 12590 2015-01-03 13:53:52Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001  MandrakeSoft S.A.
-//
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -71,6 +65,7 @@ void bx_load32bitOSimagehack(void)
       }
       if (feof(fp)) break;
     }
+    fclose(fp);
   } //if iolog file to load
 
   // Invoke proper hack depending on which OS image we're loading
