@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logio.cc 11637 2013-02-20 18:51:39Z vruppert $
+// $Id: logio.cc 13017 2016-12-30 10:04:06Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2013  The Bochs Project
+//  Copyright (C) 2001-2016  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,10 @@
 #include "cpu/cpu.h"
 #include <assert.h>
 
+#ifndef WIN32
+#include <pthread.h>
+#endif
+
 #if BX_WITH_CARBON
 #include <Carbon/Carbon.h>
 #endif
@@ -31,6 +35,7 @@
 // Just for the iofunctions
 
 static int Allocio=0;
+BX_MUTEX(logio_mutex);
 
 const char* iofunctions::getlevel(int i) const
 {
@@ -45,11 +50,25 @@ const char* iofunctions::getlevel(int i) const
   else return "?";
 }
 
+static const char *act_name[N_ACT] = { "ignore", "report", "warn", "ask", "fatal" };
+
 const char* iofunctions::getaction(int i) const
 {
-  static const char *name[] = { "ignore", "report", "ask", "fatal" };
   assert (i>=ACT_IGNORE && i<N_ACT);
-  return name[i];
+  return act_name[i];
+}
+
+int iofunctions::isaction(const char *val) const
+{
+  int action = -1;
+
+  for (int i = 0; i < N_ACT; i++) {
+    if (!strcmp(val, act_name[i])) {
+      action = ACT_IGNORE + i;
+      break;
+    }
+  }
+  return action;
 }
 
 void iofunctions::flush(void)
@@ -65,6 +84,8 @@ void iofunctions::init(void)
   // number is set.
   magic=MAGIC_LOGNUM;
 
+  BX_INIT_MUTEX(logio_mutex);
+
   // sets the default logprefix
   strcpy(logprefix,"%t%e%d");
   n_logfn = 0;
@@ -84,7 +105,7 @@ void iofunctions::remove_logfn(logfunc_t *fn)
 {
   assert(n_logfn > 0);
   int i = 0;
-  while ((fn != logfn_list[i]) && (i < n_logfn)) {
+  while ((i < n_logfn) && (fn != logfn_list[i])) {
     i++;
   };
   if (i < n_logfn) {
@@ -176,11 +197,15 @@ void iofunctions::set_log_prefix(const char* prefix)
 
 void iofunctions::out(int level, const char *prefix, const char *fmt, va_list ap)
 {
-  char c=' ', *s;
+  char c = ' ', *s;
+  char tmpstr[80], msgpfx[80], msg[1024];
+
   assert(magic==MAGIC_LOGNUM);
   assert(this != NULL);
   assert(logfd != NULL);
 
+  BX_LOCK(logio_mutex);
+
   switch (level) {
     case LOGLEV_INFO: c='i'; break;
     case LOGLEV_PANIC: c='p'; break;
@@ -189,48 +214,54 @@ void iofunctions::out(int level, const char *prefix, const char *fmt, va_list ap
     default: break;
   }
 
-  s=logprefix;
-  while(*s) {
-    switch(*s) {
+  s = logprefix;
+  msgpfx[0] = 0;
+  while (*s) {
+    switch (*s) {
       case '%':
         if(*(s+1)) s++;
         else break;
         switch(*s) {
           case 'd':
-            fprintf(logfd, "%s", prefix==NULL?"":prefix);
+            sprintf(tmpstr, "%s", prefix==NULL?"":prefix);
             break;
           case 't':
-            fprintf(logfd, FMT_TICK, bx_pc_system.time_ticks());
+            sprintf(tmpstr, FMT_TICK, bx_pc_system.time_ticks());
             break;
           case 'i':
 #if BX_SUPPORT_SMP == 0
-            fprintf(logfd, "%08x", BX_CPU(0)->get_eip());
+            sprintf(tmpstr, "%08x", BX_CPU(0)->get_eip());
 #endif
             break;
           case 'e':
-            fprintf(logfd, "%c", c);
+            sprintf(tmpstr, "%c", c);
             break;
           case '%':
-            fprintf(logfd,"%%");
+            sprintf(tmpstr,"%%");
             break;
           default:
-            fprintf(logfd,"%%%c",*s);
+            sprintf(tmpstr,"%%%c",*s);
         }
         break;
-      default :
-        fprintf(logfd,"%c",*s);
+      default:
+        sprintf(tmpstr,"%c",*s);
     }
+    strcat(msgpfx, tmpstr);
     s++;
   }
 
-  fprintf(logfd," ");
+  fprintf(logfd,"%s ", msgpfx);
 
   if(level==LOGLEV_PANIC)
     fprintf(logfd, ">>PANIC<< ");
 
-  vfprintf(logfd, fmt, ap);
-  fprintf(logfd, "\n");
+  vsnprintf(msg, sizeof(msg), fmt, ap);
+  fprintf(logfd, "%s\n", msg);
   fflush(logfd);
+  if (SIM->has_log_viewer()) {
+    SIM->log_msg(msgpfx, level, msg);
+  }
+  BX_UNLOCK(logio_mutex);
 }
 
 iofunctions::iofunctions(FILE *fs)
@@ -258,6 +289,8 @@ iofunctions::iofunctions()
 
 iofunctions::~iofunctions(void)
 {
+  BX_FINI_MUTEX(logio_mutex);
+
   // flush before erasing magic number, or flush does nothing.
   flush();
   magic=0;
@@ -270,7 +303,7 @@ int logfunctions::default_onoff[N_LOGLEV] =
   ACT_IGNORE,  // ignore debug
   ACT_REPORT,  // report info
   ACT_REPORT,  // report error
-#if BX_WITH_WX || BX_WITH_WIN32 || BX_WITH_X11
+#if BX_WITH_SDL2 || BX_WITH_WX || BX_WITH_WIN32 || BX_WITH_X11
   ACT_ASK      // on panic, ask user what to do
 #else
   ACT_FATAL    // on panic, quit
@@ -322,15 +355,19 @@ void logfunctions::setio(iofunc_t *i)
 
 void logfunctions::put(const char *p)
 {
-  const char *n = p;
-  put(n, p);
+  char *n = strdup(p);
+
+  for (unsigned i=0; i<strlen(p); i++)
+    n[i] = tolower(p[i]);
+
+  put((const char*)n, p);
+  free(n);
 }
 
 void logfunctions::put(const char *n, const char *p)
 {
-  char *tmpbuf=strdup("[     ]");   // if we ever have more than 32 chars,
-                                    // we need to rethink this
-
+  char *tmpbuf=strdup(BX_NULL_PREFIX); // if we ever have more than 32 chars,
+                                       // we need to rethink this
   if (tmpbuf == NULL)
     return;                         // allocation not successful
 
@@ -362,15 +399,13 @@ void logfunctions::info(const char *fmt, ...)
 
   assert(logio != NULL);
 
-  if(!onoff[LOGLEV_INFO]) return;
+  if (onoff[LOGLEV_INFO] == ACT_IGNORE) return;
 
   va_start(ap, fmt);
   logio->out(LOGLEV_INFO, prefix, fmt, ap);
-  if (onoff[LOGLEV_INFO] == ACT_ASK)
-    ask(LOGLEV_INFO, prefix, fmt, ap);
-  if (onoff[LOGLEV_INFO] == ACT_FATAL)
-    fatal(prefix, fmt, ap, 1);
   va_end(ap);
+
+  // the actions warn(), ask() and fatal() are not supported here
 }
 
 void logfunctions::error(const char *fmt, ...)
@@ -379,15 +414,25 @@ void logfunctions::error(const char *fmt, ...)
 
   assert(logio != NULL);
 
-  if(!onoff[LOGLEV_ERROR]) return;
+  if (onoff[LOGLEV_ERROR] == ACT_IGNORE) return;
 
   va_start(ap, fmt);
   logio->out(LOGLEV_ERROR, prefix, fmt, ap);
-  if (onoff[LOGLEV_ERROR] == ACT_ASK)
-    ask(LOGLEV_ERROR, prefix, fmt, ap);
-  if (onoff[LOGLEV_ERROR] == ACT_FATAL)
-    fatal(prefix, fmt, ap, 1);
   va_end(ap);
+
+  if (onoff[LOGLEV_ERROR] == ACT_WARN) {
+    va_start(ap, fmt);
+    warn(LOGLEV_ERROR, prefix, fmt, ap);
+    va_end(ap);
+  } else if (onoff[LOGLEV_ERROR] == ACT_ASK) {
+    va_start(ap, fmt);
+    ask(LOGLEV_ERROR, prefix, fmt, ap);
+    va_end(ap);
+  }
+  if (onoff[LOGLEV_ERROR] == ACT_FATAL) {
+    va_start(ap, fmt);
+    fatal(LOGLEV_ERROR, prefix, fmt, ap, 1);
+  }
 }
 
 void logfunctions::panic(const char *fmt, ...)
@@ -396,22 +441,26 @@ void logfunctions::panic(const char *fmt, ...)
 
   assert(logio != NULL);
 
-  // Special case for panics since they are so important.  Always print
+  // Special case for panics since they are so important. Always print
   // the panic to the log, no matter what the log action says.
-  //if(!onoff[LOGLEV_PANIC]) return;
 
   va_start(ap, fmt);
   logio->out(LOGLEV_PANIC, prefix, fmt, ap);
-
-  // This fixes a funny bug on linuxppc where va_list is no pointer but a struct
   va_end(ap);
-  va_start(ap, fmt);
 
-  if (onoff[LOGLEV_PANIC] == ACT_ASK)
+  if (onoff[LOGLEV_PANIC] == ACT_WARN) {
+    va_start(ap, fmt);
+    warn(LOGLEV_PANIC, prefix, fmt, ap);
+    va_end(ap);
+  } else if (onoff[LOGLEV_PANIC] == ACT_ASK) {
+    va_start(ap, fmt);
     ask(LOGLEV_PANIC, prefix, fmt, ap);
-  if (onoff[LOGLEV_PANIC] == ACT_FATAL)
-    fatal(prefix, fmt, ap, 1);
-  va_end(ap);
+    va_end(ap);
+  }
+  if (onoff[LOGLEV_PANIC] == ACT_FATAL) {
+    va_start(ap, fmt);
+    fatal(LOGLEV_PANIC, prefix, fmt, ap, 1);
+  }
 }
 
 void logfunctions::ldebug(const char *fmt, ...)
@@ -420,15 +469,47 @@ void logfunctions::ldebug(const char *fmt, ...)
 
   assert(logio != NULL);
 
-  if(!onoff[LOGLEV_DEBUG]) return;
+  if (onoff[LOGLEV_DEBUG] == ACT_IGNORE) return;
 
   va_start(ap, fmt);
   logio->out(LOGLEV_DEBUG, prefix, fmt, ap);
-  if (onoff[LOGLEV_DEBUG] == ACT_ASK)
-    ask(LOGLEV_DEBUG, prefix, fmt, ap);
-  if (onoff[LOGLEV_DEBUG] == ACT_FATAL)
-    fatal(prefix, fmt, ap, 1);
   va_end(ap);
+
+  // the actions warn(), ask() and fatal() are not supported here
+}
+
+void logfunctions::warn(int level, const char *prefix, const char *fmt, va_list ap)
+{
+  // Guard against reentry on warn() function.  The danger is that some
+  // function that's called within warn() could trigger another
+  // BX_ERROR that could call warn() again, leading to infinite
+  // recursion and infinite asks.
+  static char in_warn_already = 0;
+  char buf1[1024];
+  if (in_warn_already) {
+    fprintf(stderr, "logfunctions::warn() should not reenter!!\n");
+    return;
+  }
+  in_warn_already = 1;
+  vsnprintf(buf1, sizeof(buf1), fmt, ap);
+  // FIXME: facility set to 0 because it's unknown.
+
+  // update vga screen.  This is useful because sometimes useful messages
+  // are printed on the screen just before a panic.  It's also potentially
+  // dangerous if this function calls ask again...  That's why I added
+  // the reentry check above.
+  SIM->refresh_vga();
+
+  // ensure the text screen is showing
+  SIM->set_display_mode(DISP_MODE_CONFIG);
+  int val = SIM->log_dlg(prefix, level, buf1, BX_LOG_DLG_WARN);
+  if (val == BX_LOG_ASK_CHOICE_CONTINUE_ALWAYS) {
+    // user said continue, and don't "ask" for this facility again.
+    setonoff(level, ACT_REPORT);
+  }
+  // return to simulation mode
+  SIM->set_display_mode(DISP_MODE_SIM);
+  in_warn_already = 0;
 }
 
 void logfunctions::ask(int level, const char *prefix, const char *fmt, va_list ap)
@@ -455,7 +536,7 @@ void logfunctions::ask(int level, const char *prefix, const char *fmt, va_list a
 
   // ensure the text screen is showing
   SIM->set_display_mode(DISP_MODE_CONFIG);
-  int val = SIM->log_msg(prefix, level, buf1);
+  int val = SIM->log_dlg(prefix, level, buf1, BX_LOG_DLG_ASK);
   switch(val)
   {
     case BX_LOG_ASK_CHOICE_CONTINUE:
@@ -467,11 +548,10 @@ void logfunctions::ask(int level, const char *prefix, const char *fmt, va_list a
     case BX_LOG_ASK_CHOICE_DIE:
     case BX_LOG_NOTIFY_FAILED:
       bx_user_quit = (val==BX_LOG_ASK_CHOICE_DIE)?1:0;
-      in_ask_already = 0;  // because fatal will longjmp out
-      fatal(prefix, buf1, ap, 1);
-      // should never get here
-      BX_PANIC(("in ask(), fatal() should never return!"));
-      break;
+      // fatal() quits the simulation in the calling method
+      setonoff(level, ACT_FATAL);
+      in_ask_already = 0;
+      return;
     case BX_LOG_ASK_CHOICE_DUMP_CORE:
       fprintf(stderr, "User chose to dump core...\n");
 #if BX_HAVE_ABORT
@@ -548,37 +628,51 @@ static void carbonFatalDialog(const char *error, const char *exposition)
 }
 #endif
 
-void logfunctions::fatal(const char *prefix, const char *fmt, va_list ap, int exit_status)
+void logfunctions::fatal1(const char *fmt, ...)
+{
+  va_list ap;
+
+  assert(logio != NULL);
+
+  va_start(ap, fmt);
+  logio->out(LOGLEV_PANIC, prefix, fmt, ap);
+  va_end(ap);
+
+  va_start(ap, fmt);
+  fatal(LOGLEV_PANIC, prefix, fmt, ap, 1);
+}
+
+void logfunctions::fatal(int level, const char *prefix, const char *fmt, va_list ap, int exit_status)
 {
-#if !BX_WITH_WX
-  // store prefix and message in 'exit_msg' before unloading device plugins
   char tmpbuf[1024];
   char exit_msg[1024];
 
-  vsprintf(tmpbuf, fmt, ap);
-  sprintf(exit_msg, "%s %s", prefix, tmpbuf);
-#endif
+  vsnprintf(tmpbuf, sizeof(tmpbuf), fmt, ap);
+  va_end(ap);
+  if (!bx_user_quit) {
+    SIM->log_dlg(prefix, level, tmpbuf, BX_LOG_DLG_QUIT);
+  }
+  if (!SIM->is_wx_selected()) {
+    // store prefix and message in 'exit_msg' before unloading device plugins
+    sprintf(exit_msg, "%s %s", prefix, tmpbuf);
+  }
 #if !BX_DEBUGGER
   bx_atexit();
 #endif
 #if BX_WITH_CARBON
-  if(!isatty(STDIN_FILENO) && !SIM->get_init_done())
-  {
-    char buf1[1024];
-    char buf2[1024];
-    vsnprintf(buf1, sizeof(buf1), fmt, ap);
-    snprintf(buf2, sizeof(buf2), "Bochs startup error\n%s", buf1);
-    carbonFatalDialog(buf2,
+  if (!isatty(STDIN_FILENO) && !SIM->get_init_done()) {
+    snprintf(exit_msg, sizeof(exit_msg), "Bochs startup error\n%s", tmpbuf);
+    carbonFatalDialog(exit_msg,
       "For more information, try running Bochs within Terminal by clicking on \"bochs.scpt\".");
   }
 #endif
-#if !BX_WITH_WX
-  static const char *divider = "========================================================================";
-  fprintf(stderr, "%s\n", divider);
-  fprintf(stderr, "Bochs is exiting with the following message:\n");
-  fprintf(stderr, "%s", exit_msg);
-  fprintf(stderr, "\n%s\n", divider);
-#endif
+  if (!SIM->is_wx_selected()) {
+    static const char *divider = "========================================================================";
+    fprintf(stderr, "%s\n", divider);
+    fprintf(stderr, "Bochs is exiting with the following message:\n");
+    fprintf(stderr, "%s", exit_msg);
+    fprintf(stderr, "\n%s\n", divider);
+  }
 #if !BX_DEBUGGER
   BX_EXIT(exit_status);
 #else
