@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.cc 11221 2012-06-18 11:41:26Z sshwarts $
+// $Id: memory.cc 12569 2014-12-18 17:52:40Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -176,10 +176,25 @@ inc_one:
 #endif
 
     }
-  }
-  else {
+  } else if (BX_MEM_THIS bios_write_enabled && (a20addr >= (bx_phy_address)~BIOS_MASK)) {
+    // volatile BIOS write support
+#ifdef BX_LITTLE_ENDIAN
+    data_ptr = (Bit8u *) data;
+#else // BX_BIG_ENDIAN
+    data_ptr = (Bit8u *) data + (len - 1);
+#endif
+    for (unsigned i = 0; i < len; i++) {
+      BX_MEM_THIS rom[a20addr & BIOS_MASK] = *data_ptr;
+      a20addr++;
+#ifdef BX_LITTLE_ENDIAN
+      data_ptr++;
+#else // BX_BIG_ENDIAN
+      data_ptr--;
+#endif
+    }
+  } else {
     // access outside limits of physical memory, ignore
-    BX_DEBUG(("Write outside the limits of physical memory (0x"FMT_PHY_ADDRX") (ignore)", a20addr));
+    BX_DEBUG(("Write outside the limits of physical memory (0x" FMT_PHY_ADDRX ") (ignore)", a20addr));
   }
 }
 
