@@ -2,7 +2,7 @@
 //
 // misc/niclist.c
 // by Don Becker <x-odus@iname.com>
-// $Id: niclist.c 10409 2011-06-19 05:37:30Z vruppert $
+// $Id: niclist.c 12365 2014-06-07 11:42:47Z vruppert $
 //
 // This program is for win32 only.  It lists the network interface cards
 // that you can use in the "ethdev" field of the ne2k line in your bochsrc.
@@ -16,9 +16,21 @@
 #error Niclist will only work on WIN32 platforms.
 #endif
 
+#ifndef CDECL
+#if defined(_MSC_VER)
+  #define CDECL __cdecl
+#else
+  #define CDECL
+#endif
+#endif
+
 #include <windows.h>
 #include <stdio.h>
+#ifdef __CYGWIN__
+#include <wchar.h>
+#else
 #include <conio.h>
+#endif
 #include <stdlib.h>
 
 #if defined(_MSC_VER)
@@ -43,13 +55,18 @@ typedef struct {
 NIC_INFO_NT niNT[MAX_ADAPTERS];
 NIC_INFO_9X ni9X[MAX_ADAPTERS];
 
-BOOLEAN (*PacketGetAdapterNames)(PTSTR, PULONG) = NULL;
-PCHAR   (*PacketGetVersion)() = NULL;
+typedef BOOLEAN (CDECL *PGetAdapterNames)(PTSTR, PULONG);
+typedef PCHAR   (CDECL *PGetVersion)();
+
+PGetAdapterNames PacketGetAdapterNames = NULL;
+PGetVersion      PacketGetVersion = NULL;
 
 void myexit (int code)
 {
+#ifndef __CYGWIN__
   printf ("\nPress any key to continue\n");
   getch();
+#endif
   exit(code);
 }
 
@@ -59,7 +76,7 @@ int CDECL main(int argc, char **argv)
     HINSTANCE     hPacket;
     DWORD         dwVersion, dwMajorVersion;
     char          AdapterInfo[NIC_BUFFER_SIZE] = { '\0','\0' };
-    unsigned long AdapterLength = NIC_BUFFER_SIZE;
+    ULONG         AdapterLength = NIC_BUFFER_SIZE;
     LPWSTR        wstrName;
     LPSTR         strName, strDesc;
     int           nAdapterCount;
@@ -72,8 +89,8 @@ int CDECL main(int argc, char **argv)
     if(hPacket)
     {
         // Now look up the address
-        PacketGetAdapterNames = (BOOLEAN (*)(PTSTR, PULONG))GetProcAddress(hPacket, "PacketGetAdapterNames");
-        PacketGetVersion = (PCHAR (*)())GetProcAddress(hPacket, "PacketGetVersion");
+        PacketGetAdapterNames = (PGetAdapterNames)GetProcAddress(hPacket, "PacketGetAdapterNames");
+        PacketGetVersion = (PGetVersion)GetProcAddress(hPacket, "PacketGetVersion");
     }
     else {
         printf("Could not load WinPCap driver!\n");
