@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: osdep.h 11689 2013-05-24 17:58:49Z vruppert $
+// $Id: osdep.h 12574 2014-12-23 11:53:35Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2013  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -43,9 +43,6 @@ extern "C" {
 //////////////////////////////////////////////////////////////////////
 #ifdef WIN32
 
-// Definitions that are needed for all WIN32 compilers.
-#  define ssize_t long
-
 #ifndef __MINGW32__
 
 // Definitions that are needed for WIN32 compilers EXCEPT FOR
@@ -71,6 +68,7 @@ extern "C" {
 #  define S_ISCHR(m)      (((m) & S_IFMT) == S_IFCHR)
 #endif
 
+#if defined(_MSC_VER)
 // win32 has snprintf though with different name.
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
@@ -79,7 +77,6 @@ extern "C" {
 #define BX_HAVE_SNPRINTF 1
 #define BX_HAVE_VSNPRINTF 1
 
-#if defined(_MSC_VER)
 #define access _access
 #define fdopen _fdopen
 #define mktemp _mktemp
@@ -97,7 +94,10 @@ extern "C" {
 #define strrev _strrev
 #define stricmp _stricmp
 #define getch _getch
+#define strtoll _strtoi64
 #define strtoull _strtoui64
+#define isatty _isatty
+#define fileno _fileno
 #endif
 
 #else   /* __MINGW32__ defined */
@@ -201,6 +201,11 @@ extern "C" {
 typedef int socklen_t;
 #endif
 
+#if !BX_HAVE_SSIZE_T
+// needed on Windows
+typedef Bit64s ssize_t;
+#endif
+
 #if !BX_HAVE_MKSTEMP
 #define mkstemp bx_mkstemp
   BOCHSAPI_MSVCONLY extern int bx_mkstemp(char *tpl);
