@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.cc 12082 2013-12-29 20:04:16Z vruppert $
+// $Id: plugin.cc 13160 2017-03-30 18:08:15Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2013  The Bochs Project
+//  Copyright (C) 2002-2017  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -36,14 +36,24 @@
 
 #define LOG_THIS genlog->
 
-#define PLUGIN_INIT_FMT_STRING  "lib%s_LTX_plugin_init"
-#define PLUGIN_FINI_FMT_STRING  "lib%s_LTX_plugin_fini"
-#define PLUGIN_PATH ""
+#define PLUGIN_INIT_FMT_STRING       "lib%s_LTX_plugin_init"
+#define PLUGIN_FINI_FMT_STRING       "lib%s_LTX_plugin_fini"
+#define GUI_PLUGIN_INIT_FMT_STRING   "lib%s_gui_plugin_init"
+#define GUI_PLUGIN_FINI_FMT_STRING   "lib%s_gui_plugin_fini"
+#define SOUND_PLUGIN_INIT_FMT_STRING "lib%s_sound_plugin_init"
+#define SOUND_PLUGIN_FINI_FMT_STRING "lib%s_sound_plugin_fini"
+#define NET_PLUGIN_INIT_FMT_STRING   "lib%s_net_plugin_init"
+#define NET_PLUGIN_FINI_FMT_STRING   "lib%s_net_plugin_fini"
+#define PLUGIN_PATH                  ""
 
 #ifndef WIN32
-#define PLUGIN_FILENAME_FORMAT "libbx_%s.so"
+#define PLUGIN_FILENAME_FORMAT       "libbx_%s.so"
+#define SOUND_PLUGIN_FILENAME_FORMAT "libbx_sound%s.so"
+#define NET_PLUGIN_FILENAME_FORMAT   "libbx_eth_%s.so"
 #else
-#define PLUGIN_FILENAME_FORMAT "bx_%s.dll"
+#define PLUGIN_FILENAME_FORMAT       "bx_%s.dll"
+#define SOUND_PLUGIN_FILENAME_FORMAT "bx_sound%s.dll"
+#define NET_PLUGIN_FILENAME_FORMAT   "bx_eth_%s.dll"
 #endif
 
 logfunctions *pluginlog;
@@ -76,10 +86,6 @@ int (*pluginRegisterDefaultIOReadHandler)(void *thisPtr, ioReadHandler_t callbac
                             const char *name, Bit8u mask) = 0;
 int (*pluginRegisterDefaultIOWriteHandler)(void *thisPtr, ioWriteHandler_t callback,
                              const char *name, Bit8u mask) = 0;
-int (*pluginRegisterTimer)(void *this_ptr, void (*funct)(void *),
-                            Bit32u useconds, bx_bool continuous,
-bx_bool active, const char* name) = 0;
-                            void (*pluginActivateTimer)(unsigned id, Bit32u usec, bx_bool continuous) = 0;
 
 void (*pluginHRQHackCallback)(void);
 unsigned pluginHRQ = 0;
@@ -246,89 +252,15 @@ builtinRegisterDefaultIOWriteHandler(void *thisPtr, ioWriteHandler_t callback,
   return 0;
 }
 
-  static int
-builtinRegisterTimer(void *this_ptr, void (*funct)(void *),
-                        Bit32u useconds, bx_bool continuous,
-                        bx_bool active, const char* name)
-{
-  int id = bx_pc_system.register_timer (this_ptr, funct, useconds, continuous, active, name);
-  pluginlog->ldebug("plugin %s registered timer %d", name, id);
-  return id;
-}
-
-  static void
-builtinActivateTimer(unsigned id, Bit32u usec, bx_bool continuous)
-{
-  bx_pc_system.activate_timer (id, usec, continuous);
-  pluginlog->ldebug("plugin activated timer %d", id);
-}
-
 #if BX_PLUGINS
 /************************************************************************/
 /* Plugin initialization / deinitialization                             */
 /************************************************************************/
 
-void plugin_init_all (void)
-{
-  plugin_t *plugin;
-
-  pluginlog->info("Initializing plugins");
-
-  for (plugin = plugins; plugin; plugin = plugin->next)
-  {
-    char *arg_ptr = plugin->args;
-
-    /* process the command line */
-    plugin->argc = 0;
-    while (plugin->argc < MAX_ARGC)
-    {
-      while (*arg_ptr && isspace (*arg_ptr))
-        arg_ptr++;
-
-      if (!*arg_ptr) break;
-      plugin->argv[plugin->argc++] = arg_ptr;
-
-      while (*arg_ptr && !isspace (*arg_ptr))
-        arg_ptr++;
-
-      if (!*arg_ptr) break;
-      *arg_ptr++ = '\0';
-    }
-
-    /* initialize the plugin */
-    if (plugin->plugin_init (plugin, plugin->type, plugin->argc, plugin->argv))
-    {
-      pluginlog->panic("Plugin initialization failed for %s", plugin->name);
-      plugin_abort();
-    }
-
-    plugin->initialized = 1;
-  }
-}
-
 void plugin_init_one(plugin_t *plugin)
 {
-  char *arg_ptr = plugin->args;
-
-  /* process the command line */
-  plugin->argc = 0;
-  while (plugin->argc < MAX_ARGC)
-  {
-    while (*arg_ptr && isspace (*arg_ptr))
-      arg_ptr++;
-
-    if (!*arg_ptr) break;
-    plugin->argv[plugin->argc++] = arg_ptr;
-
-    while (*arg_ptr && !isspace (*arg_ptr))
-      arg_ptr++;
-
-    if (!*arg_ptr) break;
-    *arg_ptr++ = '\0';
-  }
-
   /* initialize the plugin */
-  if (plugin->plugin_init (plugin, plugin->type, plugin->argc, plugin->argv))
+  if (plugin->plugin_init(plugin, plugin->type))
   {
     pluginlog->info("Plugin initialization failed for %s", plugin->name);
     plugin_abort();
@@ -354,19 +286,12 @@ plugin_t *plugin_unload(plugin_t *plugin)
 
   dead_plug = plugin;
   plugin = plugin->next;
-  free(dead_plug);
+  delete dead_plug;
 
   return plugin;
 }
 
-void plugin_fini_all (void)
-{
-  plugin_t *plugin;
-
-  for (plugin = plugins; plugin; plugin = plugin_unload(plugin));
-}
-
-void plugin_load(char *name, char *args, plugintype_t type)
+void plugin_load(char *name, plugintype_t type)
 {
   plugin_t *plugin, *temp;
 #if defined(WIN32)
@@ -385,18 +310,20 @@ void plugin_load(char *name, char *args, plugintype_t type)
     }
   }
 
-  plugin = (plugin_t *)malloc(sizeof(plugin_t));
-  if (!plugin) {
-    BX_PANIC(("malloc plugin_t failed"));
-  }
+  plugin = new plugin_t;
 
   plugin->type = type;
   plugin->name = name;
-  plugin->args = args;
   plugin->initialized = 0;
 
   char plugin_filename[BX_PATHNAME_LEN], tmpname[BX_PATHNAME_LEN];
-  sprintf(tmpname, PLUGIN_FILENAME_FORMAT, name);
+  if (type == PLUGTYPE_SOUND) {
+    sprintf(tmpname, SOUND_PLUGIN_FILENAME_FORMAT, name);
+  } else if (type == PLUGTYPE_NETWORK) {
+    sprintf(tmpname, NET_PLUGIN_FILENAME_FORMAT, name);
+  } else {
+    sprintf(tmpname, PLUGIN_FILENAME_FORMAT, name);
+  }
   sprintf(plugin_filename, "%s%s", PLUGIN_PATH, tmpname);
 
   // Set context so that any devices that the plugin registers will
@@ -420,8 +347,9 @@ void plugin_load(char *name, char *args, plugintype_t type)
   BX_INFO(("DLL handle is %p", plugin->handle));
   if (!plugin->handle) {
     current_plugin_context = NULL;
-    BX_PANIC(("LoadLibrary failed for module '%s': error=%d", name, GetLastError()));
-    free(plugin);
+    BX_PANIC(("LoadLibrary failed for module '%s' (%s): error=%d", name,
+              plugin_filename, GetLastError()));
+    delete plugin;
     return;
   }
 #else
@@ -429,13 +357,20 @@ void plugin_load(char *name, char *args, plugintype_t type)
   BX_INFO(("lt_dlhandle is %p", plugin->handle));
   if (!plugin->handle) {
     current_plugin_context = NULL;
-    BX_PANIC(("dlopen failed for module '%s': %s", name, lt_dlerror()));
-    free(plugin);
+    BX_PANIC(("dlopen failed for module '%s' (%s): %s", name, plugin_filename,
+              lt_dlerror()));
+    delete plugin;
     return;
   }
 #endif
 
-  if (type != PLUGTYPE_USER) {
+  if (type == PLUGTYPE_GUI) {
+    sprintf(tmpname, GUI_PLUGIN_INIT_FMT_STRING, name);
+  } else if (type == PLUGTYPE_SOUND) {
+    sprintf(tmpname, SOUND_PLUGIN_INIT_FMT_STRING, name);
+  } else if (type == PLUGTYPE_NETWORK) {
+    sprintf(tmpname, NET_PLUGIN_INIT_FMT_STRING, name);
+  } else if (type != PLUGTYPE_USER) {
     sprintf(tmpname, PLUGIN_INIT_FMT_STRING, name);
   } else {
     sprintf(tmpname, PLUGIN_INIT_FMT_STRING, "user");
@@ -454,7 +389,13 @@ void plugin_load(char *name, char *args, plugintype_t type)
   }
 #endif
 
-  if (type != PLUGTYPE_USER) {
+  if (type == PLUGTYPE_GUI) {
+    sprintf(tmpname, GUI_PLUGIN_FINI_FMT_STRING, name);
+  } else if (type == PLUGTYPE_SOUND) {
+    sprintf(tmpname, SOUND_PLUGIN_FINI_FMT_STRING, name);
+  } else if (type == PLUGTYPE_NETWORK) {
+    sprintf(tmpname, NET_PLUGIN_FINI_FMT_STRING, name);
+  } else if (type != PLUGTYPE_USER) {
     sprintf(tmpname, PLUGIN_FINI_FMT_STRING, name);
   } else {
     sprintf(tmpname, PLUGIN_FINI_FMT_STRING, "user");
@@ -533,9 +474,6 @@ plugin_startup(void)
   pluginRegisterDefaultIOReadHandler = builtinRegisterDefaultIOReadHandler;
   pluginRegisterDefaultIOWriteHandler = builtinRegisterDefaultIOWriteHandler;
 
-  pluginRegisterTimer = builtinRegisterTimer;
-  pluginActivateTimer = builtinActivateTimer;
-
   pluginlog = new logfunctions();
   pluginlog->put("PLUGIN");
 #if BX_PLUGINS && !defined(WIN32)
@@ -554,13 +492,9 @@ plugin_startup(void)
 
 void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmodel_c *devmodel, const char *name)
 {
-  device_t *device, **devlist;
+  device_t **devlist;
 
-  device = (device_t *)malloc(sizeof(device_t));
-  if (!device)
-  {
-    pluginlog->panic("can't allocate device_t");
-  }
+  device_t *device = new device_t;
 
   device->name = name;
   BX_ASSERT(devmodel != NULL);
@@ -610,7 +544,7 @@ void pluginUnregisterDeviceDevmodel(const char *name)
       } else {
         prev->next = device->next;
       }
-      free(device);
+      delete device;
       break;
     } else {
       prev = device;
@@ -643,7 +577,7 @@ int bx_load_plugin(const char *name, plugintype_t type)
 {
   char *namecopy = new char[1+strlen(name)];
   strcpy(namecopy, name);
-  plugin_load(namecopy, (char*)"", type);
+  plugin_load(namecopy, type);
   return 1;
 }
 
@@ -761,7 +695,7 @@ void bx_unload_plugins()
 #endif
     }
     next = device->next;
-    free(device);
+    delete device;
     device = next;
   }
   devices = NULL;
@@ -781,7 +715,7 @@ void bx_unload_core_plugins()
       delete device->devmodel;
     }
     next = device->next;
-    free(device);
+    delete device;
     device = next;
   }
   core_devices = NULL;
@@ -837,71 +771,169 @@ void bx_plugins_after_restore_state()
 
 #if !BX_PLUGINS
 
-// special code for loading optional plugins when plugins are turned off
+// Special code for loading gui, optional and sound plugins when plugin support
+// is turned off.
 
 typedef struct {
-  const char *name;
+  const char*   name;
+  plugintype_t  type;
   plugin_init_t plugin_init;
   plugin_fini_t plugin_fini;
   bx_bool       status;
 } builtin_plugin_t;
 
-#define BUILTIN_PLUGIN_ENTRY(mod) {#mod, lib##mod##_LTX_plugin_init, lib##mod##_LTX_plugin_fini, 0}
+#define BUILTIN_GUI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_GUI, lib##mod##_gui_plugin_init, lib##mod##_gui_plugin_fini, 0}
+#define BUILTIN_OPT_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_OPTIONAL, lib##mod##_LTX_plugin_init, lib##mod##_LTX_plugin_fini, 0}
+#define BUILTIN_SND_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_SOUND, lib##mod##_sound_plugin_init, lib##mod##_sound_plugin_fini, 0}
+#define BUILTIN_NET_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_NETWORK, lib##mod##_net_plugin_init, lib##mod##_net_plugin_fini, 0}
 
-static builtin_plugin_t builtin_opt_plugins[] = {
-  BUILTIN_PLUGIN_ENTRY(unmapped),
-  BUILTIN_PLUGIN_ENTRY(biosdev),
-  BUILTIN_PLUGIN_ENTRY(speaker),
-  BUILTIN_PLUGIN_ENTRY(extfpuirq),
-  BUILTIN_PLUGIN_ENTRY(parallel),
-  BUILTIN_PLUGIN_ENTRY(serial),
+static builtin_plugin_t builtin_plugins[] = {
+#if BX_WITH_AMIGAOS
+  BUILTIN_GUI_PLUGIN_ENTRY(amigaos),
+#endif
+#if BX_WITH_CARBON
+  BUILTIN_GUI_PLUGIN_ENTRY(carbon),
+#endif
+#if BX_WITH_MACOS
+  BUILTIN_GUI_PLUGIN_ENTRY(macos),
+#endif
+#if BX_WITH_NOGUI
+  BUILTIN_GUI_PLUGIN_ENTRY(nogui),
+#endif
+#if BX_WITH_RFB
+  BUILTIN_GUI_PLUGIN_ENTRY(rfb),
+#endif
+#if BX_WITH_SDL
+  BUILTIN_GUI_PLUGIN_ENTRY(sdl),
+#endif
+#if BX_WITH_SDL2
+  BUILTIN_GUI_PLUGIN_ENTRY(sdl2),
+#endif
+#if BX_WITH_SVGA
+  BUILTIN_GUI_PLUGIN_ENTRY(svga),
+#endif
+#if BX_WITH_TERM
+  BUILTIN_GUI_PLUGIN_ENTRY(term),
+#endif
+#if BX_WITH_VNCSRV
+  BUILTIN_GUI_PLUGIN_ENTRY(vncsrv),
+#endif
+#if BX_WITH_WIN32
+  BUILTIN_GUI_PLUGIN_ENTRY(win32),
+#endif
+#if BX_WITH_WX
+  BUILTIN_GUI_PLUGIN_ENTRY(wx),
+#endif
+#if BX_WITH_X11
+  BUILTIN_GUI_PLUGIN_ENTRY(x),
+#endif
+  BUILTIN_OPT_PLUGIN_ENTRY(unmapped),
+  BUILTIN_OPT_PLUGIN_ENTRY(biosdev),
+  BUILTIN_OPT_PLUGIN_ENTRY(speaker),
+  BUILTIN_OPT_PLUGIN_ENTRY(extfpuirq),
+  BUILTIN_OPT_PLUGIN_ENTRY(parallel),
+  BUILTIN_OPT_PLUGIN_ENTRY(serial),
+#if BX_SUPPORT_BUSMOUSE
+  BUILTIN_OPT_PLUGIN_ENTRY(busmouse),
+#endif
 #if BX_SUPPORT_E1000
-  BUILTIN_PLUGIN_ENTRY(e1000),
+  BUILTIN_OPT_PLUGIN_ENTRY(e1000),
 #endif
 #if BX_SUPPORT_ES1370
-  BUILTIN_PLUGIN_ENTRY(es1370),
+  BUILTIN_OPT_PLUGIN_ENTRY(es1370),
 #endif
 #if BX_SUPPORT_GAMEPORT
-  BUILTIN_PLUGIN_ENTRY(gameport),
+  BUILTIN_OPT_PLUGIN_ENTRY(gameport),
 #endif
 #if BX_SUPPORT_IODEBUG
-  BUILTIN_PLUGIN_ENTRY(iodebug),
+  BUILTIN_OPT_PLUGIN_ENTRY(iodebug),
 #endif
 #if BX_SUPPORT_NE2K
-  BUILTIN_PLUGIN_ENTRY(ne2k),
+  BUILTIN_OPT_PLUGIN_ENTRY(ne2k),
 #endif
 #if BX_SUPPORT_PCIDEV
-  BUILTIN_PLUGIN_ENTRY(pcidev),
+  BUILTIN_OPT_PLUGIN_ENTRY(pcidev),
 #endif
 #if BX_SUPPORT_PCIPNIC
-  BUILTIN_PLUGIN_ENTRY(pcipnic),
+  BUILTIN_OPT_PLUGIN_ENTRY(pcipnic),
 #endif
 #if BX_SUPPORT_SB16
-  BUILTIN_PLUGIN_ENTRY(sb16),
-#endif
-#if BX_SUPPORT_USB_OHCI
-  BUILTIN_PLUGIN_ENTRY(usb_ohci),
+  BUILTIN_OPT_PLUGIN_ENTRY(sb16),
 #endif
 #if BX_SUPPORT_USB_UHCI
-  BUILTIN_PLUGIN_ENTRY(usb_uhci),
+  BUILTIN_OPT_PLUGIN_ENTRY(usb_uhci),
+#endif
+#if BX_SUPPORT_USB_OHCI
+  BUILTIN_OPT_PLUGIN_ENTRY(usb_ohci),
+#endif
+#if BX_SUPPORT_USB_EHCI
+  BUILTIN_OPT_PLUGIN_ENTRY(usb_ehci),
 #endif
 #if BX_SUPPORT_USB_XHCI
-  BUILTIN_PLUGIN_ENTRY(usb_xhci),
+  BUILTIN_OPT_PLUGIN_ENTRY(usb_xhci),
 #endif
 #if BX_SUPPORT_VOODOO
-  BUILTIN_PLUGIN_ENTRY(voodoo),
+  BUILTIN_OPT_PLUGIN_ENTRY(voodoo),
 #endif
-  {"NULL", NULL, NULL, 0}
+#if BX_SUPPORT_SOUNDLOW
+  BUILTIN_SND_PLUGIN_ENTRY(dummy),
+  BUILTIN_SND_PLUGIN_ENTRY(file),
+#if BX_HAVE_SOUND_ALSA
+  BUILTIN_SND_PLUGIN_ENTRY(alsa),
+#endif
+#if BX_HAVE_SOUND_OSS
+  BUILTIN_SND_PLUGIN_ENTRY(oss),
+#endif
+#if BX_HAVE_SOUND_OSX
+  BUILTIN_SND_PLUGIN_ENTRY(osx),
+#endif
+#if BX_HAVE_SOUND_SDL
+  BUILTIN_SND_PLUGIN_ENTRY(sdl),
+#endif
+#if BX_HAVE_SOUND_WIN
+  BUILTIN_SND_PLUGIN_ENTRY(win),
+#endif
+#endif
+#if BX_NETWORKING
+#if BX_NETMOD_FBSD
+  BUILTIN_NET_PLUGIN_ENTRY(fbsd),
+#endif
+#if BX_NETMOD_LINUX
+  BUILTIN_NET_PLUGIN_ENTRY(linux),
+#endif
+  BUILTIN_NET_PLUGIN_ENTRY(null),
+#if BX_NETMOD_SLIRP
+  BUILTIN_NET_PLUGIN_ENTRY(slirp),
+#endif
+#if BX_NETMOD_SOCKET
+  BUILTIN_NET_PLUGIN_ENTRY(socket),
+#endif
+#if BX_NETMOD_TAP
+  BUILTIN_NET_PLUGIN_ENTRY(tap),
+#endif
+#if BX_NETMOD_TUNTAP
+  BUILTIN_NET_PLUGIN_ENTRY(tuntap),
+#endif
+#if BX_NETMOD_VDE
+  BUILTIN_NET_PLUGIN_ENTRY(vde),
+#endif
+  BUILTIN_NET_PLUGIN_ENTRY(vnet),
+#if BX_NETMOD_WIN32
+  BUILTIN_NET_PLUGIN_ENTRY(win32),
+#endif
+#endif
+  {"NULL", PLUGTYPE_GUI, NULL, NULL, 0}
 };
 
-int bx_load_opt_plugin(const char *name)
+int bx_load_plugin2(const char *name, plugintype_t type)
 {
   int i = 0;
-  while (strcmp(builtin_opt_plugins[i].name, "NULL")) {
-    if (!strcmp(name, builtin_opt_plugins[i].name)) {
-      if (builtin_opt_plugins[i].status == 0) {
-        builtin_opt_plugins[i].plugin_init(NULL, PLUGTYPE_OPTIONAL, 0, NULL);
-        builtin_opt_plugins[i].status = 1;
+  while (strcmp(builtin_plugins[i].name, "NULL")) {
+    if ((!strcmp(name, builtin_plugins[i].name)) &&
+        (type == builtin_plugins[i].type)) {
+      if (builtin_plugins[i].status == 0) {
+        builtin_plugins[i].plugin_init(NULL, type);
+        builtin_plugins[i].status = 1;
       }
       return 1;
     }
@@ -913,14 +945,15 @@ int bx_load_opt_plugin(const char *name)
 int bx_unload_opt_plugin(const char *name, bx_bool devflag)
 {
   int i = 0;
-  while (strcmp(builtin_opt_plugins[i].name, "NULL")) {
-    if (!strcmp(name, builtin_opt_plugins[i].name)) {
-      if (builtin_opt_plugins[i].status == 1) {
+  while (strcmp(builtin_plugins[i].name, "NULL")) {
+    if ((!strcmp(name, builtin_plugins[i].name)) &&
+        (builtin_plugins[i].type == PLUGTYPE_OPTIONAL)) {
+      if (builtin_plugins[i].status == 1) {
         if (devflag) {
-          pluginUnregisterDeviceDevmodel(builtin_opt_plugins[i].name);
+          pluginUnregisterDeviceDevmodel(builtin_plugins[i].name);
         }
-        builtin_opt_plugins[i].plugin_fini();
-        builtin_opt_plugins[i].status = 0;
+        builtin_plugins[i].plugin_fini();
+        builtin_plugins[i].status = 0;
       }
       return 1;
     }
